@@ -37,8 +37,15 @@
 
 ;;; Code:
 
+;; it is now safe to put the `provide' anywhere.  if an error occurs while
+;; loading, all provides (and fsets) will be undone.  put it first to
+;; prevent require/provide loop with custom and cus-face.
+(provide 'custom)
+
 (eval-when-compile
-  (load "cl-macs" nil t))
+  (load "cl-macs" nil t)
+  ;; To elude warnings.
+  (require 'cus-face))
 
 (autoload 'custom-declare-face "cus-face")
 (autoload 'defun* "cl-macs")
@@ -396,7 +403,7 @@ LOAD should be either a library file name, or a feature name."
 
 Define a theme labeled by SYMBOL THEME. The optional argument DOC is a
 doc string describing the theme. It is optionally followed by the
-following keyboard arguments
+following keyword arguments
 
 :short-description DESC
       DESC is a short (one line) description of the theme. If not given DOC
@@ -423,7 +430,7 @@ following keyboard arguments
   (memq theme custom-known-themes))
 
 (defsubst custom-check-theme (theme)
-  "Check whether THEME is valid and signal an error if NOT"
+  "Check whether THEME is valid and signal an error if NOT."
   (unless (custom-theme-p theme)
     (error "Unknown theme `%s'" theme)))
 
@@ -565,7 +572,7 @@ BODY is as with custom-theme-load-themes."
 
 
 (defsubst copy-upto-last (elt list)
-  "Copy all the elements of the list upto the last occurrence of elt"
+  "Copy all the elements of the list upto the last occurrence of elt."
   ;; Is it faster to do more work in C than to do less in elisp?
   (nreverse (cdr (member elt (reverse list)))))
 
@@ -637,6 +644,4 @@ The ARGS are as in `custom-theme-reset-variables'."
 
 ;;; The End.
 
-(provide 'custom)
-
 ;; custom.el ends here
