@@ -30,6 +30,7 @@
 	"objects"
 	"extents"
 	"events"
+	"hash-table"
 	"text-props"
 	"process" ;; This is bad. network-streams may not be defined.
 	(when-feature multicast "multicast") ; #+network-streams implicitly true
@@ -54,10 +55,8 @@
 				;  `emacs-user-extension-dir'
 	"misc"
 	;; (pureload "profile")
-	(unless-feature mule "help-nomule")
 	"help"
 	;; (pureload "hyper-apropos")  Soon...
-	(unless-feature file-coding "files-nomule")
 	"files"
 	"lib-complete"
 	"format"
@@ -87,6 +86,9 @@
 				; auto-gc-threshold
 	"itimer-autosave"
 	"printer"
+	"behavior"
+	"behavior-defs"
+	"diagnose"
 
 	;;;;;;;;;;;;;;;;;; GUI support
 	(when-feature window-system "gui")
@@ -113,24 +115,28 @@
 	(when-feature (and (not infodock) dialog) "dialog-items")
 
 	;;;;;;;;;;;;;;;;;; Coding-system support
-	(when-feature file-coding "coding")
-	(when-feature file-coding "code-files")
+	"coding"
+	"code-files"
 	;; Handle process with encoding/decoding coding-system.
-	(when-feature file-coding "code-process")
+	"code-process"
 	;; Provide basic commands to set coding systems to user
-	(when-feature file-coding "code-cmds")
+	"code-cmds"
+	"unicode"
 	;;;;;;;;;;;;;;;;;; MULE support
 	(when-feature mule "mule-charset")
+	(when-feature mule "mule-cmds") ; to sync with Emacs 20.1
 	(when-feature mule "mule-coding")
-	;; All files after this can have extended characters in them.
-	(when-feature mule "mule-help")
-	(when-feature mule "mule-category")
-	(when-feature mule "mule-ccl")
-	(when-feature mule "mule-misc")
-	(when-feature mule "kinsoku")
+	(when-feature mule "mule-composite-stub")
+	(when-feature mule "mule-composite")
+	;; may initialize coding systems
 	(when-feature (and mule x) "mule-x-init")
 	(when-feature (and mule tty) "mule-tty-init")
-	(when-feature mule "mule-cmds") ; to sync with Emacs 20.1
+	(when-feature (and mule mswindows) "mule-msw-init")
+	"code-init" ; set up defaults
+	;; All files after this can have extended characters in them.
+	(when-feature mule "mule-category")
+	(when-feature mule "mule-ccl")
+	(when-feature mule "kinsoku")
 
 ;; after this goes the specific lisp routines for a particular input system
 ;; 97.2.5 JHod Shouldn't these go into a site-load file to allow site
@@ -145,23 +151,30 @@
 ;;     (pureload "canna")
 ;;   )
 
-;; Now load files to set up all the different languages/environments
-;; that Mule knows about.
+;; Now load files to set up all the different languages/environments that
+;; Mule knows about.  Formerly we had to worry about files shadowed by
+;; those of the same name in leim/quail.el, but no longer, since we now
+;; compile with -no-packages.
 
 	(when-feature mule "arabic")
 	(when-feature mule "chinese")
-	(when-feature mule "mule/cyrillic") ; overloaded in leim/quail
+	(when-feature mule "cyrillic")
 	(when-feature mule "english")
 	(when-feature mule "ethiopic")
 	(when-feature mule "european")
-	(when-feature mule "mule/greek") ; overloaded in leim/quail
+	(when-feature mule "greek")
 	(when-feature mule "hebrew")
+	(when-feature mule "indian")
+	(when-feature mule "devanagari") ; must be loaded after indian.el
 	(when-feature mule "japanese")
 	(when-feature mule "korean")
+	(when-feature mule "lao")
+	(when-feature mule "latin")
 	(when-feature mule "misc-lang")
-	(when-feature mule "thai-xtis-chars")
-	(when-feature mule "mule/thai-xtis") ; overloaded in leim/quail
-	(when-feature mule "viet-chars")
+	;; #### merge thai and thai-xtis!!!
+	;(when-feature mule "thai")
+	(when-feature mule "thai-xtis")
+	(when-feature mule "tibetan")
 	(when-feature mule "vietnamese")
 
 	;; Specialized language support
@@ -174,13 +187,14 @@
 ;; SKK is now a package
 ;	(when-feature mule "skk-leim")
 
-;; Set up the XEmacs environment for Mule.
-;; Assumes the existence of various stuff above.
-	(when-feature mule "mule-init")
-
 ;; Enable Mule capability for Gnus, mail, etc...
 ;; Moved to sunpro-load.el - the default only for Sun.
 ;;(pureload "mime-setup")
+
+	;; needs access to the charsets created by the above
+	;; language-specific files.
+	(when-feature (and mule mswindows) "mule-msw-init-late")
+
 ;;; mule-load.el ends here
 
 ;; preload InfoDock stuff.  should almost certainly not be here if
@@ -229,6 +243,7 @@
 	;; "ediff-hook"		; Packaged.
 	"fontl-hooks"
 	"auto-show"
+	"resize-minibuffer"
 	(when-feature ldap "ldap")
 
 ;; (when-feature energize "energize/energize-load.el")
