@@ -46,6 +46,11 @@
 
 ;;; Code:
 
+;; To elude the warnings for font functions. (Normally autoloaded when
+;; font-create-object is called)
+(eval-when-compile
+  (require 'font))
+
 (defgroup faces nil
   "Support for multiple text attributes (fonts, colors, ...)
 Such a collection of attributes is called a \"face\"."
@@ -1803,7 +1808,6 @@ in that frame; otherwise change each frame."
 (make-face 'secondary-selection "Secondary selection face.")
 
 ;; Several useful color faces.
-(eval-when-compile (load "cl-macs"))
 (dolist (color '(red green blue yellow))
   (make-face color (concat (symbol-name color) " text."))
   (set-face-foreground color (symbol-name color) nil 'color))
