@@ -2,6 +2,7 @@
 
 ;; Copyright (C) 1985, 1996, 1997 Free Software Foundation, Inc.
 ;; Copyright (C) 1995 Tinker Systems.
+;; Copyright (C) 2002 Ben Wing.
 
 ;; Maintainer: FSF
 ;; Keywords: lisp, languages, dumped
@@ -61,6 +62,20 @@
 	["Evaluate %_Whole Buffer" ,(popup-wrap 'eval-current-buffer)]
 	["Evaluate Last %_S-expression" ,(popup-wrap 'eval-last-sexp)]
 	"---"
+	["%_Indent Region or Balanced Expression"
+	 ,(popup-wrap '(if (region-exists-p)
+			   (call-interactively 'indent-region)
+			 (call-interactively 'indent-sexp)))]
+	["I%_ndent Defun"
+	 ,(popup-wrap '(progn
+			 (beginning-of-defun)
+			 (indent-sexp)))]
+	"---"
+	["%_Comment Out Region" comment-region :active (region-exists-p)]
+	["Unc%_omment Region" (comment-region (region-beginning)
+					      (region-end) '(4))
+	 :active (region-exists-p)]
+	"---"
 	,@(if popup-p
 	    '(["%_Find Function"
 	       (find-function (menu-call-at-event '(function-at-point)))
@@ -100,19 +115,10 @@
 	["%_Untrace All Functions" untrace-all
 	 :active (fboundp 'untrace-all)]
 	"---"
-	["%_Comment Out Region" comment-region :active (region-exists-p)]
-	["Unc%_omment Region" (comment-region (region-beginning)
-					      (region-end) '(4))
-	 :active (region-exists-p)]
-	"---"
-	["%_Indent Region or Balanced Expression"
-	 ,(popup-wrap '(if (region-exists-p)
-			   (call-interactively 'indent-region)
-			 (call-interactively 'indent-sexp)))]
-	["I%_ndent Defun"
-	 ,(popup-wrap '(progn
-			 (beginning-of-defun)
-			 (indent-sexp)))]
+	["Display %_Macro Expansion of Balanced Expression"
+	 ,(popup-wrap 'macroexpand-sexp)]
+	["Display Recursive Macro E%_xpansion of Balanced Expression"
+	 ,(popup-wrap 'macroexpand-all-sexp)]
 	"---"
 	"Look for debug-on-error under Options->Troubleshooting"
 	)))
@@ -699,7 +705,7 @@ of the start of the containing expression."
 		  (backward-up-list 1)
 		  (backward-up-list 1)
 		  (backward-up-list 1)
-		  (looking-at "(flet\\s-"))
+		  (looking-at "(\\(flet\\|macrolet\\|labels\\)\\s-"))
 	      (error nil))
 	    (setq method 'defun)
 	  (setq method (or (get (intern-soft function) 'lisp-indent-function)
@@ -776,6 +782,9 @@ of the start of the containing expression."
 ;; like defun if the first form is placed on the next line, otherwise
 ;; it is indented like any other form (i.e. forms line up under first).
 
+;; NOTE: All def* functions are handled specially, no need to put them here.
+;; NOTE: All cl-* constructs are handled in cl.el.
+
 (put 'lambda 'lisp-indent-function 'defun)
 (put 'autoload 'lisp-indent-function 'defun)
 (put 'progn 'lisp-indent-function 0)
@@ -792,7 +801,6 @@ of the start of the containing expression."
 (put 'let 'lisp-indent-function 1)
 (put 'let* 'lisp-indent-function 1)
 (put 'let-specifier 'lisp-indent-function 1)
-(put 'flet 'lisp-indent-function 1)
 (put 'while 'lisp-indent-function 1)
 (put 'if 'lisp-indent-function 2)
 (put 'catch 'lisp-indent-function 1)
