@@ -3,6 +3,7 @@
 ;; Copyright (C) 1995 Electrotechnical Laboratory, JAPAN.
 ;; Licensed to the Free Software Foundation.
 ;; Copyright (C) 1997 MORIOKA Tomohiko
+;; Copyright (C) 2000, 2001, 2002 Ben Wing.
 
 ;; Keywords: multilingual, Chinese
 
@@ -30,6 +31,79 @@
 
 ;;; Code:
 
+(eval-when-compile
+  (require 'china-util))
+
+; (make-charset 'chinese-gb2312 
+; 	      "GB2312 Chinese simplified: ISO-IR-58"
+; 	      '(dimension
+; 		2
+; 		registry "GB2312.1980"
+; 		chars 94
+; 		columns 2
+; 		direction l2r
+; 		final ?A
+; 		graphic 0
+; 		short-name "GB2312"
+; 		long-name "GB2312: ISO-IR-58"
+; 		))
+
+; (make-charset 'chinese-cns11643-1 
+; 	      "CNS11643 Plane 1 Chinese traditional: ISO-IR-171"
+; 	      '(dimension
+; 		2
+; 		registry "CNS11643.1992-1"
+; 		chars 94
+; 		columns 2
+; 		direction l2r
+; 		final ?G
+; 		graphic 0
+; 		short-name "CNS11643-1"
+; 		long-name "CNS11643-1 (Chinese traditional): ISO-IR-171"
+; 		))
+
+; (make-charset 'chinese-cns11643-2 
+; 	      "CNS11643 Plane 2 Chinese traditional: ISO-IR-172"
+; 	      '(dimension
+; 		2
+; 		registry "CNS11643.1992-2"
+; 		chars 94
+; 		columns 2
+; 		direction l2r
+; 		final ?H
+; 		graphic 0
+; 		short-name "CNS11643-2"
+; 		long-name "CNS11643-2 (Chinese traditional): ISO-IR-172"
+; 		))
+
+; (make-charset 'chinese-big5-1 
+; 	      "Frequently used part (A141-C67F) of Big5 (Chinese traditional)"
+; 	      '(dimension
+; 		2
+; 		registry "Big5"
+; 		chars 94
+; 		columns 2
+; 		direction l2r
+; 		final ?0
+; 		graphic 0
+; 		short-name "Big5 (Level-1)"
+; 		long-name "Big5 (Level-1) A141-C67F"
+; 		))
+
+; (make-charset 'chinese-big5-2 
+; 	      "Less frequently used part (C940-FEFE) of Big5 (Chinese traditional)"
+; 	      '(dimension
+; 		2
+; 		registry "Big5"
+; 		chars 94
+; 		columns 2
+; 		direction l2r
+; 		final ?1
+; 		graphic 0
+; 		short-name "Big5 (Level-2)"
+; 		long-name "Big5 (Level-2) C940-FEFE"
+; 		))
+
 ;; Syntax of Chinese characters.
 (modify-syntax-entry 'chinese-gb2312 "w")
 (loop for row in '(33 34 41)
@@ -44,6 +118,23 @@
 (modify-syntax-entry 'chinese-big5-1 "w")
 (modify-syntax-entry 'chinese-big5-2 "w")
 
+; ;; Chinese CNS11643 Plane3 thru Plane7.  Although these are official
+; ;; character sets, the use is rare and don't have to be treated
+; ;; space-efficiently in the buffer.
+; (make-charset 'chinese-cns11643-3 
+; 	      "CNS11643 Plane 3 Chinese Traditional: ISO-IR-183"
+; 	      '(dimension
+; 		2
+; 		registry "CNS11643.1992-3"
+; 		chars 94
+; 		columns 2
+; 		direction l2r
+; 		final ?I
+; 		graphic 0
+; 		short-name "CNS11643-3"
+; 		long-name "CNS11643-3 (Chinese traditional): ISO-IR-183"
+; 		))
+
 ;; CNS11643 Plane3 thru Plane7
 ;; These represent more and more obscure Chinese characters.
 ;; By the time you get to Plane 7, we're talking about characters
@@ -60,7 +151,10 @@
          dimension 2
          chars 94
          final ,final
-         graphic 0))
+         graphic 0
+	 short-name ,(concat "CNS11643-" plane)
+	 long-name ,(format "CNS11643-%s (Chinese traditional): ISO-IR-183"
+			    plane)))
       (modify-syntax-entry   name "w")
       (modify-category-entry name ?t)
       ))
@@ -74,22 +168,31 @@
 ;; ISO-IR-165 (CCITT Extended GB)
 ;;    It is based on CCITT Recommendation T.101, includes GB 2312-80 +
 ;;    GB 8565-88 table A4 + 293 characters.
-(make-charset
+(make-charset ;; not in FSF 21.1
  'chinese-isoir165
  "ISO-IR-165 (CCITT Extended GB; Chinese simplified)"
  `(registry "isoir165"
    dimension 2
    chars 94
    final ?E
-   graphic 0))
+   graphic 0
+   short-name "ISO-IR-165"
+   long-name "ISO-IR-165 (CCITT Extended GB; Chinese simplified)"))
 
 ;; PinYin-ZhuYin
-(make-charset 'sisheng "PinYin-ZhuYin"
-	      '(registry "sisheng_cwnn\\|OMRON_UDC_ZH"
-		dimension 1
+(make-charset 'chinese-sisheng 
+	      "SiSheng characters for PinYin/ZhuYin"
+	      '(dimension
+		1
+		;; XEmacs addition: second half of registry spec
+		registry "sisheng_cwnn\\|OMRON_UDC_ZH"
 		chars 94
+		columns 1
+		direction l2r
 		final ?0
 		graphic 0
+		short-name "SiSheng"
+		long-name "SiSheng (PinYin/ZhuYin)"
 		))
 
 ;; If you prefer QUAIL to EGG, please modify below as you wish.
@@ -147,12 +250,20 @@
 
 (make-coding-system
  'cn-gb-2312 'iso2022
- "Coding-system of Chinese EUC (Extended Unix Code)."
+ "Chinese EUC"
  '(charset-g0 ascii
    charset-g1 chinese-gb2312
-   charset-g2 sisheng
+   charset-g2 chinese-sisheng
    charset-g3 t
    mnemonic "Zh-GB/EUC"
+   documentation
+   "Chinese EUC (Extended Unix Code), the standard Chinese encoding on Unix.
+This follows the same overall EUC principles as Japanese EUC (see the
+description under Japanese EUC), but specifies different character sets:
+
+G0: ASCII
+G1: Chinese-GB2312
+G2: Sisheng (PinYin - ZhuYin)"
    ))
 
 ;; (define-coding-system-alias 'cn-gb-2312 'chinese-iso-8bit)
@@ -175,11 +286,13 @@
 
 (make-coding-system
  'hz-gb-2312 'no-conversion
- "Coding-system of Hz/ZW used for Chinese."
+ "Hz/ZW (Chinese)"
  '(mnemonic "Zh-GB/Hz"
    eol-type lf
    post-read-conversion post-read-decode-hz
-   pre-write-conversion pre-write-encode-hz))
+   pre-write-conversion pre-write-encode-hz
+   documentation "Hz/ZW 7-bit encoding for Chinese GB2312 (MIME:HZ-GB-2312)"
+))
 
 ;; (define-coding-system-alias 'hz-gb-2312 'chinese-hz)
 ;; (define-coding-system-alias 'hz 'chinese-hz)
@@ -189,7 +302,8 @@
 (defun post-read-decode-hz (len)
   (let ((pos (point))
 	(buffer-modified-p (buffer-modified-p))
-	last-coding-system-used)
+	;last-coding-system-used
+	)
     (prog1
 	(decode-hz-region pos (+ pos len))
       (set-buffer-modified-p buffer-modified-p))))
@@ -200,19 +314,29 @@
     (if (stringp from)
 	(insert from)
       (insert-buffer-substring buf from to))
-    (let (last-coding-system-used)
-      (encode-hz-region 1 (point-max)))
+    ;(let (last-coding-system-used)
+    (encode-hz-region 1 (point-max))
     nil))
-	   
+
 (set-language-info-alist
  "Chinese-GB" '((setup-function . setup-chinese-gb-environment-internal)
-		(charset chinese-gb2312 sisheng)
+		(charset chinese-gb2312 chinese-sisheng)
 		(coding-system cn-gb-2312 iso-2022-7bit hz-gb-2312)
 		(coding-priority cn-gb-2312 big5 iso-2022-7bit)
+		(cygwin-locale "zh")
+		(locale "zh_CN.eucCN" "zh_CN.EUC" "zh_CN"
+			"chinese-s" "zh"
+			  #'(lambda (arg)
+			      (and arg (let ((case-fold-search t))
+					 (string-match "^zh_.*.GB.*" arg)))))
+		(mswindows-locale ("CHINESE" . "CHINESE_SIMPLIFIED"))
+		(native-coding-system cn-gb-2312)
 		(input-method . "chinese-py-punct")
 		(features china-util)
 		(sample-text . "Chinese ($AVPND(B,$AFUM(;0(B,$A::So(B)	$ADc:C(B")
-		(documentation . "Support for Chinese GB2312 character set."))
+		(documentation .
+"Supports Simplified Chinese, used in mainland China.
+Uses the GB2312 character set."))
  '("Chinese"))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -229,8 +353,18 @@
 
 (make-coding-system
  'big5 'big5
- "Coding-system of BIG5."
- '(mnemonic "Zh/Big5"))
+ "Big5"
+ '(mnemonic "Zh/Big5"
+   documentation
+   "A non-modal encoding formed by five large Taiwanese companies
+\(hence \"Big5\") to produce a character set and encoding for
+traditional Chinese writing.  Big5 encodes some 13,000+ characters.
+ASCII is encoded as normal, and Chinese characters as two bytes, but
+Chinese characters do not exclusively use the high half.  The first
+byte is in the high half standard position A1-FE, but the second byte
+is in either low 40-7E or high A1-FE.  Thus Big5 suffers from the
+classic \"it might look like a slash, but it's really the second byte
+of a Chinese character\"."))
 
 ;; (define-coding-system-alias 'big5 'chinese-big5)
 ;; (define-coding-system-alias 'cn-big5 'chinese-big5)
@@ -255,17 +389,28 @@
 ;; (setq font-ccl-encoder-alist
 ;;       (cons (cons "big5" ccl-encode-big5-font) font-ccl-encoder-alist))
 
-(set-charset-ccl-program 'chinese-big5-1 ccl-encode-big5-font)
-(set-charset-ccl-program 'chinese-big5-2 ccl-encode-big5-font)
+(set-charset-ccl-program 'chinese-big5-1 'ccl-encode-big5-font)
+(set-charset-ccl-program 'chinese-big5-2 'ccl-encode-big5-font)
 
 (set-language-info-alist
  "Chinese-BIG5" '((charset chinese-big5-1 chinese-big5-2)
 		  (coding-system big5 iso-2022-7bit)
 		  (coding-priority big5 cn-gb-2312 iso-2022-7bit)
+		  (cygwin-locale "zh_TW")
+		  (locale "zh_TW.Big5" "zh_TW.big5" "zh_CN.big5" "zh_TW"
+			  "chinese-t"
+			  #'(lambda (arg)
+			      (and arg (let ((case-fold-search t))
+					 (string-match "^zh_.*.BIG5.*" arg)))))
+		  (mswindows-locale ("CHINESE" . "CHINESE_TRADITIONAL"))
+		  (native-coding-system big5)
 		  (input-method . "chinese-py-punct-b5")
 		  (features china-util)
 		  (sample-text . "Cantonese ($(0GnM$(B,$(0N]0*Hd(B)	$(0*/=((B, $(0+$)p(B")
-		  (documentation . "Support for Chinese Big5 character set."))
+		  (documentation .
+"Supports Traditional Chinese, used in Taiwan, Hong Kong, and Singapore.
+Uses the Chinese Big5 character set."
+))
  '("Chinese"))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
