@@ -3,6 +3,7 @@
 ;; Copyright (C) 1985-1986, 1990, 1992-1997 Free Software Foundation, Inc.
 ;; Copyright (c) 1993, 1994 Sun Microsystems, Inc.
 ;; Copyright (C) 1995 Board of Trustees, University of Illinois
+;; Copyright (C) 2001, 2002 Ben Wing.
 
 ;; Maintainer: XEmacs Development Team
 ;; Keywords: internal, dumped
@@ -214,9 +215,9 @@ after, and will not be true at any time before.")
 
 (defvar command-switch-alist
   '(("-help"	. command-line-do-help)
-    ("-version". command-line-do-version)
+    ("-version" . command-line-do-version)
     ("-V"	. command-line-do-version)
-    ("-funcall". command-line-do-funcall)
+    ("-funcall" . command-line-do-funcall)
     ("-f"	. command-line-do-funcall)
     ("-e"	. command-line-do-funcall-1)
     ("-eval"	. command-line-do-eval)
@@ -225,6 +226,8 @@ after, and will not be true at any time before.")
     ("-insert"	. command-line-do-insert)
     ("-i"	. command-line-do-insert)
     ("-kill"	. command-line-do-kill)
+    ("-eol"     . command-line-do-enable-eol-detection)
+    ("-enable-eol-detection" . command-line-do-enable-eol-detection)
     ;; Options like +35 are handled specially.
     ;; Window-system, site, or package-specific code might add to this.
     ;; X11 handles its options by letting Xt remove args from this list.
@@ -254,31 +257,95 @@ command line options plus the following:
 
 In addition, the")
        "The"))
-    (princ " following options are accepted:
+    (let ((l command-switch-alist)
+	  (options " following options are accepted:
+
+Display options:
+
+  -nw                   Open the initial frame on the current TTY, instead of
+                        a window system.
+  -t <device>           Use TTY <device> instead of the current TTY for input
+                        and output.  This implies the -nw option.
+  -display <display>    Standard X option, to specify the display connection.
+                        If this option is given, or if the environment
+                        variable DISPLAY is set, an initial X frame will be
+                        created.  Otherwise, an initial Windows frame will be
+                        created if Windows support exists and neither -nw nor
+                        -t is given.  Otherwise, a TTY frame is created.
+  -unmapped             Do not display the initial frame.  Useful to create
+                        a \"server\" that can accept `gnuclient' connections.
+
+Noninteractive options:
+
+  {-help}
+  {-version}
+  {-V}
+  -batch                Execute noninteractively (messages go to stderr, no
+                        initial frame created).
+  {-funcall}
+                        (The function may parse the rest of the command line
+                        for its arguments.)
+  {-f}
+  {-eval}
+  {-load}
+  {-l}
+  {-insert}
+  {-i}
+  {-kill}
   -sd                   Show dump ID.  Ignored when configured without --pdump.
   -nd                   Don't load the dump file.  Roughly like old temacs.
                         Ignored when configured without --pdump.
-  -t <device>           Use TTY <device> instead of the terminal for input
-                        and output.  This implies the -nw option.
-  -nw                   Inhibit the use of any window-system-specific
-                        display code: use the current tty.
-  -batch                Execute noninteractively (messages go to stderr).
-  -debug-init           Enter the debugger if an error in the init file occurs.
-  -unmapped             Do not map the initial frame.
-  -no-site-file         Do not load the site-specific init file (site-start.el).
+
+Initialization files:
+
   -no-init-file         Do not load the user-specific init file.
-  -no-early-packages	Do not process early packages.
+  -q                    Same as -no-init-file.
+  -debug-init           Enter the debugger if an error in the init file occurs.
+  -user-init-file <file>
+                        Use <file> as init file.
+  -user-init-directory <directory>
+                        Use <directory> as init directory.
+  -user <user>          Load user's init file instead of your own.
+  -u <user>             Same as -user.
+  -no-site-file         Do not load the site-specific init file
+                        (site-start.el).
+
+Package/module options:
+
+  -vanilla		Equivalent to -q -no-site-file -no-early-packages.
+                        Useful if you think some user-init or site-init code
+                        is messing things up, or when running XEmacs in
+                        batch mode.
   -no-autoloads		Do not load global symbol files (auto-autoloads) at
 			startup.  Also implies `-vanilla'.
-  -vanilla		Equivalent to -q -no-site-file -no-early-packages.
-  -q                    Same as -no-init-file.
-  -user-init-file <file> Use <file> as init file.
-  -user-init-directory <directory> Use <directory> as init directory.
-  -user <user>          Load user's init file instead of your own.
-  -u <user>             Same as -user.\n")
-   (let ((l command-switch-alist)
+  -no-packages          Pretend like the packages don't exist.  Don't put
+                        any packages in the load path or set up any package
+                        autoloads.  Also Implies `-vanilla'.  Use this when
+                        running XEmacs in batch mode when you aren't using
+                        any functionality in packages and want to make sure
+                        that you get no interference from packages
+                        (e.g. Lisp files that shadow core Lisp files).
+  -no-early-packages	Do not process early packages.
+  -debug-paths          Display info about the runtime values of various
+                        directory variables (e.g. for loading packages).
+  -no-site-modules      Do not search site-modules directories for modules
+                        at startup.  Only applies when modules support is
+                        compiled into XEmacs.
+
+Encoding options:
+
+  -eol                  Turn on EOL detection (only applies to Unix, no
+                        international support; otherwise EOL detection is
+                        already on).
+  -nuni                 Under MS Windows, disable use of the Unicode versions
+                        of API calls.  Not for Windows 95/98/ME.  This is
+                        mostly only useful for debugging purposes.
+
+Misc:
+
+  +N <file>             Start displaying <file> at line N.
+")
 	  (insert (lambda (&rest x)
-		    (princ "  ")
 		    (let ((len 2))
 		      (while x
 			(princ (car x))
@@ -289,41 +356,48 @@ In addition, the")
 		      (while (< len 24)
 			(princ " ")
 			(incf len))))))
-      (while l
-        (let ((name (car (car l)))
-              (fn (cdr (car l)))
-	      doc arg cons)
-	  (cond
-	   ((and (symbolp fn) (get fn 'undocumented)) nil)
-	   (t
-	    (setq doc (documentation fn))
-	    (if (member doc '(nil "")) (setq doc "(undocumented)"))
-	    (cond ((string-match "\n\\(<.*>\\)\n?\\'" doc)
-		   ;; Doc of the form "The frobber switch\n<arg1> <arg2>"
-		   (setq arg (substring doc (match-beginning 1) (match-end 1))
-			 doc (substring doc 0 (match-beginning 0))))
-		  ((string-match "\n+\\'" doc)
-		   (setq doc (substring doc 0 (match-beginning 0)))))
-	    (if (and (setq cons (rassq fn command-switch-alist))
-		     (not (eq cons (car l))))
-		(setq doc (format "Same as %s." (car cons))))
-	    (if arg
-		(funcall insert name " " arg)
-	      (funcall insert name))
-	    (princ doc)
-	    (terpri))))
-        (setq l (cdr l))))
+      (princ
+       (with-temp-buffer
+	 (insert options)
+	 (while l
+	   (let ((name (car (car l)))
+		 (fn (cdr (car l)))
+		 doc arg cons)
+	     (cond
+	      ((and (symbolp fn) (get fn 'undocumented)) nil)
+	      (t
+	       (setq doc (documentation fn))
+	       (if (member doc '(nil "")) (setq doc "(undocumented)"))
+	       (cond ((string-match "\n\\(<.*>\\)\n?\\'" doc)
+		      ;; Doc of the form "The frobber switch\n<arg1> <arg2>"
+		      (setq arg (substring doc (match-beginning 1) (match-end 1))
+			    doc (substring doc 0 (match-beginning 0))))
+		     ((string-match "\n+\\'" doc)
+		      (setq doc (substring doc 0 (match-beginning 0)))))
+	       (if (and (setq cons (rassq fn command-switch-alist))
+			(not (eq cons (car l))))
+		   (setq doc (format "Same as %s." (car cons))))
+	       (goto-char (point-min))
+	       (when (search-forward (format "{%s}" name) nil t)
+		 (delete-region (match-beginning 0) (match-end 0))
+		 (let ((standard-output (current-buffer)))
+		   (if arg
+		       (funcall insert name " " arg)
+		     (funcall insert name))
+		   (princ doc))))))
+	   (setq l (cdr l)))
+	 (buffer-string))))
     (princ (concat "\
-  +N <file>             Start displaying <file> at line N.
 
 Anything else is considered a file name, and is placed into a buffer for
 editing.
 
 " (emacs-name) " has an online tutorial and manuals.  Type ^Ht (Control-h t) after
 starting XEmacs to run the tutorial.  Type ^Hi to enter the manual browser.
-Type ^H^H^H (Control-h Control-h Control-h) to get more help options.\n")
+Type ^H^H^H (Control-h Control-h Control-h) to get more help options.\n"))
 
-    (kill-emacs 0))))
+    (kill-emacs 0)
+    ))
 
 (defun command-line-do-funcall (arg)
   "Invoke the named lisp function with no arguments.
@@ -361,6 +435,10 @@ Type ^H^H^H (Control-h Control-h Control-h) to get more help options.\n")
   (princ (concat (emacs-version) "\n"))
   (kill-emacs 0))
 
+(defun command-line-do-enable-eol-detection (arg)
+  "Turn on EOL detection (only applies to Unix)."
+  (set-eol-detection t))
+
 
 ;;; Processing the command line and loading various init files
 
@@ -409,7 +487,7 @@ Type ^H^H^H (Control-h Control-h Control-h) to get more help options.\n")
 	(princ "\n\n" stream)))
     (when (not suppress-early-error-handler-backtrace)
       (backtrace stream t)))
-  (if (fboundp 'mswindows-message-box)
+  (if-fboundp 'mswindows-message-box
       (mswindows-message-box "Initialization error"))
   (kill-emacs -1))
 
@@ -417,6 +495,11 @@ Type ^H^H^H (Control-h Control-h Control-h) to get more help options.\n")
   (if command-line-processed
       (message "Back to top level.")
     (setq command-line-processed t)
+    ;; Do this first for maximum likelihood of catching errors.  The main
+    ;; purpose of this is so that debug-on-error can be set to catch errors
+    ;; during normal noninteractive running.
+    (when (getenv "XEMACSDEBUG")
+      (eval (read (getenv "XEMACSDEBUG"))))
     ;; Canonicalize HOME (PWD is canonicalized by init_buffer in buffer.c)
     (let ((value (user-home-directory)))
       (if (and value
@@ -444,9 +527,12 @@ Type ^H^H^H (Control-h Control-h Control-h) to get more help options.\n")
 	  (startup-find-roots-warning))
       (startup-setup-paths emacs-roots
 			   user-init-directory
-			   inhibit-early-packages
+			   (cond (inhibit-all-packages t)
+				 (inhibit-early-packages '(early))
+				 (t nil))
 			   inhibit-site-lisp
-			   debug-paths)
+			   debug-paths
+			   nil)
       (startup-setup-paths-warning))
 
     (startup-load-autoloads)
@@ -518,36 +604,33 @@ Type ^H^H^H (Control-h Control-h Control-h) to get more help options.\n")
   ;;      (progn
   ;;	(standard-display-european t)
   ;;	(require 'iso-syntax)))
-
-  (setq load-user-init-file-p (not (noninteractive)))
-
+  
+  (if vanilla-inhibiting ;; set in main_1()
+      (setq load-user-init-file-p nil
+	    site-start-file nil)
+    (setq load-user-init-file-p (not (noninteractive))))
+  
   ;; Allow (at least) these arguments anywhere in the command line
-  (let ((new-args nil)
-	(arg      nil))
-    (while args
-      (setq arg (pop args))
+  (macrolet ((long-argmatch (match)
+	       ;; use a macro to avoid lots of concatting at runtime
+	       `(or (string= arg ,match)
+		    (string= arg ,(concat "-" match)))))
+    (let ((new-args nil)
+	  (arg      nil))
+      (while args
+	(setq arg (pop args))
       (cond
        ((or (string= arg "-q")
-	    (string= arg "-no-init-file"))
+	    (long-argmatch "-no-init-file"))
 	(setq load-user-init-file-p nil))
-       ((string= arg "-no-site-file")
+       ((long-argmatch "-no-site-file")
 	(setq site-start-file nil))
-       ((or (string= arg "-no-early-packages")
-	    (string= arg "--no-early-packages"))
-	(setq inhibit-early-packages t))
-       ((or (string= arg "-vanilla")
-	    (string= arg "--vanilla")
-	    ;; Some work on this one already done in emacs.c.
-	    (string= arg "-no-autoloads")
-	    (string= arg "--no-autoloads"))
-	(setq load-user-init-file-p nil
-	      site-start-file nil))
-       ((string= arg "-user-init-file")
+       ((long-argmatch "-user-init-file")
 	(setq user-init-file (pop args)))
-       ((string= arg "-user-init-directory")
+       ((long-argmatch "-user-init-directory")
 	(setq user-init-directory (file-name-as-directory (pop args))))
        ((or (string= arg "-u")
- 	    (string= arg "-user"))
+	    (long-argmatch "-user"))
 	(let* ((user (pop args))
 	       (home-user (concat "~" user)))
 	  (setq user-init-directory (file-name-as-directory
@@ -557,13 +640,10 @@ Type ^H^H^H (Control-h Control-h Control-h) to get more help options.\n")
 		(find-user-init-file user-init-directory home-user))
 	  (setq custom-file
 		(make-custom-file-name user-init-file))))
-       ((string= arg "-debug-init")
+       ((long-argmatch "-debug-init")
 	(setq init-file-debug t))
-       ((string= arg "-unmapped")
+       ((long-argmatch "-unmapped")
 	(setq initial-frame-unmapped-p t))
-       ((or (string= arg "-debug-paths")
-	    (string= arg "--debug-paths"))
-	t)
        ((or (string= arg "--") (string= arg "-"))
 	(while args
 	  (push (pop args) new-args)))
@@ -572,7 +652,7 @@ Type ^H^H^H (Control-h Control-h Control-h) to get more help options.\n")
     (with-obsolete-variable 'init-file-user
       (setq init-file-user (and load-user-init-file-p "")))
 
-    (nreverse new-args)))
+    (nreverse new-args))))
 
 (defconst initial-scratch-message "\
 ;; This buffer is for notes you don't want to save, and for Lisp evaluation.
@@ -601,7 +681,7 @@ If this is nil, no message will be displayed.")
       (setq command-line-args-left (command-line-early command-line-args-left))
 
       (when (eq system-type 'windows-nt)
-	(init-mswindows-at-startup))
+	(declare-fboundp (init-mswindows-at-startup)))
 
       ;; Setup the toolbar icon directory
       (when (featurep 'toolbar)
@@ -629,6 +709,20 @@ If this is nil, no message will be displayed.")
     ;; does nothing.
     (startup-initialize-custom-faces)
 
+    ;; A couple of other things need to be initted.
+    ;; (RMS writes about internally using hooks for this, in reference
+    ;; to frame-initialize and frame-notice-user-settings:
+    ;;
+    ;; These are now called explicitly at the proper times,
+    ;; since that is easier to understand.
+    ;; Actually using hooks within Emacs is bad for future maintenance. --rms.
+    ;;
+    ;; In this case, I completely agree. --ben
+    (init-menubar-at-startup)
+    ;; perhaps this should go earlier in the proecess?
+    (if (featurep 'mule)
+	(declare-fboundp (init-mule-at-startup)))
+
     ;;
     ;; We have normality, I repeat, we have normality.  Anything you still
     ;; can't cope with is therefore your own problem.  (And we don't need
@@ -851,25 +945,35 @@ directory which will load the relocated initialization code.")
 	  (progn
 	    ;; Do this without a condition-case if the user wants to debug.
 	    (load-user-init-file))
-	(condition-case error
-	    (progn
-	      (if load-user-init-file-p
-		  (load-user-init-file))
-	      (setq init-file-had-error nil))
-          (error
-	   (message "Error in init file: %s" (error-message-string error))
-	   (display-warning 'initialization
-	     (format "\
+	(condition-case nil
+	    (call-with-condition-handler
+		#'(lambda (__load_init_file_arg__)
+		    (let ((errstr (error-message-string
+				   __load_init_file_arg__)))
+		      (message "Error in init file: %s" errstr)
+		      (lwarn 'initialization 'error
+			"\
 An error has occurred while loading %s:
 
 %s
 
+Backtrace follows:
+
+%s
+
 To ensure normal operation, you should investigate the cause of the error
 in your initialization file and remove it.  Use the `-debug-init' option
-to XEmacs to view a complete error backtrace."
-		     user-init-file (error-message-string error))
-	     'error)
-	   (setq init-file-had-error t))))
+to XEmacs to enter the debugger when the error occurs and investigate the
+exact problem."
+			user-init-file errstr
+			(backtrace-in-condition-handler-eliminating-handler
+			 '__load_init_file_arg__)))
+		    (setq init-file-had-error t))
+		#'(lambda ()
+		    (if load-user-init-file-p
+			(load-user-init-file))
+		    (setq init-file-had-error nil)))
+	  (error nil)))
       ;; If we can tell that the init file altered debug-on-error,
       ;; arrange to preserve the value that it set up.
       (or (eq debug-on-error debug-on-error-initial)
@@ -1170,7 +1274,7 @@ Copyright (C) 1995-1996 Ben Wing\n"))
      "Things that you should know rather quickly...\n\n"
      ((key find-file) ": visit a file\n")
      ((key save-buffer) ": save changes\n")
-     ((key advertised-undo) ": undo changes\n")
+     ((key undo) ": undo changes\n")
      ((key save-buffers-kill-emacs) ": exit XEmacs\n"))
     ])
 
@@ -1249,119 +1353,6 @@ Copyright (C) 1995-1996 Ben Wing\n"))
 	;; don't let /tmp_mnt/... get into the load-path or exec-path.
 	(abbreviate-file-name invocation-directory)))
 
-(defun startup-setup-paths (roots user-init-directory
-				  &optional
-				  inhibit-early-packages inhibit-site-lisp
-				  debug-paths)
-  "Setup all the various paths.
-ROOTS is a list of plausible roots of the XEmacs directory hierarchy.
-If INHIBIT-PACKAGES is non-NIL, don't do packages.
-If INHIBIT-SITE-LISP is non-NIL, don't do site-lisp.
-If DEBUG-PATHS is non-NIL, print paths as they are detected.
-It's idempotent, so call this as often as you like!"
-
-  (apply #'(lambda (early late last)
-	     (setq early-packages (and (not inhibit-early-packages)
-				       early))
-	     (setq late-packages late)
-	     (setq last-packages last))
-	 (packages-find-packages
-	  roots
-	  (packages-compute-package-locations user-init-directory)))
-
-  (setq early-package-load-path (packages-find-package-load-path early-packages))
-  (setq late-package-load-path (packages-find-package-load-path late-packages))
-  (setq last-package-load-path (packages-find-package-load-path last-packages))
-
-  (if debug-paths
-      (progn
-	(princ (format "configure-package-path:\n%S\n" configure-package-path)
-	       'external-debugging-output)
-	(princ (format "early-packages and early-package-load-path:\n%S\n%S\n"
-		       early-packages early-package-load-path)
-	       'external-debugging-output)
-	(princ (format "late-packages and late-package-load-path:\n%S\n%S\n"
-		       late-packages late-package-load-path)
-	       'external-debugging-output)
-	(princ (format "last-packages and last-package-load-path:\n%S\n%S\n"
-		       last-packages last-package-load-path)
-	       'external-debugging-output)))
-
-  (setq lisp-directory (paths-find-lisp-directory roots))
-
-  (if debug-paths
-      (princ (format "lisp-directory:\n%S\n" lisp-directory)
-	     'external-debugging-output))
-
-  (if (featurep 'mule)
-      (progn
-	(setq mule-lisp-directory
-	      (paths-find-mule-lisp-directory roots
-					      lisp-directory))
-	(if debug-paths
-	    (princ (format "mule-lisp-directory:\n%S\n"
-			   mule-lisp-directory)
-		   'external-debugging-output)))
-    (setq mule-lisp-directory '()))
-
-  (setq site-directory (and (null inhibit-site-lisp)
-			    (paths-find-site-lisp-directory roots)))
-
-  (if (and debug-paths (null inhibit-site-lisp))
-      (princ (format "site-directory:\n%S\n" site-directory)
-	     'external-debugging-output))
-
-  (setq load-path (paths-construct-load-path roots
-					     early-package-load-path
-					     late-package-load-path
-					     last-package-load-path
-					     lisp-directory
-					     site-directory
-					     mule-lisp-directory))
-
-  (setq Info-directory-list
-	(paths-construct-info-path roots
-				   early-packages late-packages last-packages))
-
-
-  (if debug-paths
-      (princ (format "Info-directory-list:\n%S\n" Info-directory-list)
-	     'external-debugging-output))
-
-  (setq exec-directory (paths-find-exec-directory roots))
-
-  (if debug-paths
-      (princ (format "exec-directory:\n%s\n" exec-directory)
-	     'external-debugging-output))
-
-  (setq exec-path
-	(paths-construct-exec-path roots exec-directory
-				   early-packages late-packages last-packages))
-
-  (if debug-paths
-      (princ (format "exec-path:\n%S\n" exec-path)
-	     'external-debugging-output))
-
-  (setq doc-directory (paths-find-doc-directory roots))
-
-  (if debug-paths
-      (princ (format "doc-directory:\n%S\n" doc-directory)
-	     'external-debugging-output))
-
-  (setq data-directory (paths-find-data-directory roots))
-
-  (if debug-paths
-      (princ (format "data-directory:\n%S\n" data-directory)
-	     'external-debugging-output))
-
-  (setq data-directory-list (paths-construct-data-directory-list data-directory
-								 early-packages
-								 late-packages
-								 last-packages))
-  (if debug-paths
-      (princ (format "data-directory-list:\n%S\n" data-directory-list)
-	     'external-debugging-output)))
-
 (defun startup-find-roots-warning ()
   (save-excursion
     (set-buffer (get-buffer-create " *warning-tmp*"))
@@ -1412,7 +1403,7 @@ It's idempotent, so call this as often as you like!"
       (load (expand-file-name (file-name-sans-extension autoload-file-name)
 			      lisp-directory) nil t))
 
-  (if (not inhibit-autoloads)
+  (if (and (not inhibit-autoloads) (not inhibit-all-packages))
       (progn
 	(if (not inhibit-early-packages)
 	    (packages-load-package-auto-autoloads early-package-load-path))
