@@ -3,7 +3,7 @@
 ;; Copyright (C) 1985, 1986, 1992, 1994-5, 1997 Free Software Foundation, Inc.
 ;; Copyright (C) 1995 Tinker Systems and INS Engineering Corp.
 ;; Copyright (C) 1995 Sun Microsystems.
-;; Copyright (C) 2000, 2001 Ben Wing.
+;; Copyright (C) 2000, 2001, 2002 Ben Wing.
 
 ;; Maintainer: XEmacs Development Team
 ;; Keywords: extensions, dumped
@@ -362,6 +362,19 @@ NOTE: At some point, this will be moved into C and will be very fast."
 
 ;;;; String functions.
 
+;; XEmacs
+(defun string-equal-ignore-case (str1 str2)
+  "Return t if two strings have identical contents, ignoring case differences.
+Case is not significant.  Text properties and extents are ignored.
+Symbols are also allowed; their print names are used instead.
+
+See also `equalp'."
+  (if (symbolp str1)
+      (setq str1 (symbol-name str1)))
+  (if (symbolp str2)
+      (setq str2 (symbol-name str2)))
+  (eq t (compare-strings str1 nil nil str2 nil nil t)))
+
 ;; XEmacs
 (defun replace-in-string (str regexp newtext &optional literal)
   "Replace all matches in STR for REGEXP with NEWTEXT string,
@@ -506,6 +519,93 @@ The original current buffer is restored afterwards."
 (define-function 'char-int 'char-to-int)
 (define-function 'int-char 'int-to-char)
 
+(defun string-width (string)
+  "Return number of columns STRING occupies when displayed.
+With international (Mule) support, uses the charset-columns attribute of
+the characters in STRING, which may not accurately represent the actual
+display width when using a window system.  With no international support,
+simply returns the length of the string."
+  (if (featurep 'mule)
+      (let ((col 0)
+	    (len (length string))
+	    (i 0))
+	(with-fboundp '(charset-width char-charset)
+	  (while (< i len)
+	    (setq col (+ col (charset-width (char-charset (aref string i)))))
+	    (setq i (1+ i))))
+	col)
+    (length string)))
+
+(defun char-width (character)
+  "Return number of columns a CHARACTER occupies when displayed."
+  (if (featurep 'mule)
+      (with-fboundp '(charset-width char-charset)
+	(charset-width (char-charset character)))
+    1))
+
+;; The following several functions are useful in GNU Emacs 20 because
+;; of the multibyte "characters" the internal representation of which
+;; leaks into Lisp.  In XEmacs/Mule they are trivial and unnecessary.
+;; We provide them for compatibility reasons solely.
+
+(defun string-to-sequence (string type)
+  "Convert STRING to a sequence of TYPE which contains characters in STRING.
+TYPE should be `list' or `vector'."
+  (ecase type
+    (list
+     (mapcar #'identity string))
+    (vector
+     (mapvector #'identity string))))
+
+(defun string-to-list (string)
+  "Return a list of characters in STRING."
+  (mapcar #'identity string))
+
+(defun string-to-vector (string)
+  "Return a vector of characters in STRING."
+  (mapvector #'identity string))
+
+(defun store-substring (string idx obj)
+  "Embed OBJ (string or character) at index IDX of STRING."
+  (let* ((str (cond ((stringp obj) obj)
+		    ((characterp obj) (char-to-string obj))
+		    (t (error
+			"Invalid argument (should be string or character): %s"
+			obj))))
+	 (string-len (length string))
+	 (len (length str))
+	 (i 0))
+    (while (and (< i len) (< idx string-len))
+      (aset string idx (aref str i))
+      (setq idx (1+ idx) i (1+ i)))
+    string))
+
+;; #### This function is not compatible with FSF in some cases.  Hard
+;; to fix, because it is hard to trace the logic of the FSF function.
+;; In case we need the exact behavior, we can always copy the FSF
+;; version, which is very long and does lots of unnecessary stuff.
+(defun truncate-string-to-width (str end-column &optional start-column padding)
+  "Truncate string STR to end at column END-COLUMN.
+The optional 2nd arg START-COLUMN, if non-nil, specifies
+the starting column; that means to return the characters occupying
+columns START-COLUMN ... END-COLUMN of STR.
+
+The optional 3rd arg PADDING, if non-nil, specifies a padding character
+to add at the end of the result if STR doesn't reach column END-COLUMN,
+or if END-COLUMN comes in the middle of a character in STR.
+PADDING is also added at the beginning of the result
+if column START-COLUMN appears in the middle of a character in STR.
+
+If PADDING is nil, no padding is added in these cases, so
+the resulting string may be narrower than END-COLUMN."
+  (or start-column
+      (setq start-column 0))
+  (let ((len (length str)))
+    (concat (substring str (min start-column len) (min end-column len))
+	    (and padding (> end-column len)
+		 (make-string (- end-column len) padding)))))
+
+
 
 ;; alist/plist functions
 (defun plist-to-alist (plist)
@@ -525,6 +625,15 @@ The original plist is not modified.  See also `destructive-plist-to-alist'."
       (setq plist (cddr plist)))
     (nreverse alist)))
 
+(defun map-plist (_mp_fun _mp_plist)
+  "Map _MP_FUN (a function of two args) over each key/value pair in _MP_PLIST.
+Return a list of the results."
+  (let (_mp_result)
+    (while _mp_plist
+      (push (funcall _mp_fun (car _mp_plist) (cadr _mp_plist)) _mp_result)
+      (setq _mp_plist (cddr _mp_plist)))
+    (nreverse _mp_result)))
+
 (defun destructive-plist-to-alist (plist)
   "Convert property list PLIST into the equivalent association-list form.
 The alist is returned.  This converts from
@@ -774,6 +883,93 @@ yourself.]"
   "Returns non-nil if SYM names a currently-defined error."
   (and (symbolp sym) (not (null (get sym 'error-conditions)))))
 
+(defun backtrace-in-condition-handler-eliminating-handler (handler-arg-name)
+  "Return a backtrace inside of a condition handler, eliminating the handler.
+This is for use in the condition handler inside of call-with-condition-handler,
+when written like this:
+
+\(call-with-condition-handler
+    #'(lambda (__some_weird_arg__)
+	do the handling ...)
+    #'(lambda ()
+	do the stuff that might cause an error))
+
+Pass in the name (a symbol) of the argument used in the lambda function
+that specifies the handler, and make sure the argument name is unique, and
+this function generates a backtrace and strips off the part above where the
+error occurred (i.e. the handler itself)."
+  (let* ((bt (with-output-to-string (backtrace nil t)))
+	 (bt (save-match-data
+	       ;; Try to eliminate the part of the backtrace
+	       ;; above where the error occurred.
+	       (if (string-match
+		    (concat "bind (\\(?:.* \\)?" (symbol-name handler-arg-name)
+			    "\\(?:.* \\)?)[ \t\n]*\\(?:(lambda \\|#<compiled-function \\)("
+			    (symbol-name handler-arg-name)
+			    ").*\n\\(\\(?:.\\|\n\\)*\\)$")
+		    bt) (match-string 1 bt) bt))))
+    bt))
+
+(put 'with-trapping-errors 'lisp-indent-function 0)
+(defmacro with-trapping-errors (&rest keys-body)
+  "Trap errors in BODY, outputting a warning and a backtrace.
+Usage looks like
+
+\(with-trapping-errors
+    [:operation OPERATION]
+    [:error-form ERROR-FORM]
+    [:no-backtrace NO-BACKTRACE]
+    [:class CLASS]
+    [:level LEVEL]
+    [:resignal RESIGNAL]
+    BODY)
+
+Return value without error is whatever BODY returns.  With error, return
+result of ERROR-FORM (which will be evaluated only when the error actually
+occurs), which defaults to nil.  OPERATION is given in the warning message.
+CLASS and LEVEL are the warning class and level (default to class
+`general', level `warning').  If NO-BACKTRACE is given, no backtrace is
+displayed.  If RESIGNAL is given, the error is resignaled after the warning
+is displayed and the ERROR-FORM is executed."
+  (let ((operation "unknown")
+	(error-form nil)
+	(no-backtrace nil)
+	(class ''general)
+	(level ''warning)
+	(resignal nil))
+    (let* ((keys '(operation error-form no-backtrace class level resignal))
+	   (keys-with-colon
+	    (mapcar #'(lambda (sym)
+			(intern (concat ":" (symbol-name sym)))) keys)))
+      (while (memq (car keys-body) keys-with-colon)
+	(let* ((key-with-colon (pop keys-body))
+	       (key (intern (substring (symbol-name key-with-colon) 1))))
+	  (set key (pop keys-body)))))
+    `(condition-case ,(if resignal '__cte_cc_var__ nil)
+	 (call-with-condition-handler
+	     #'(lambda (__call_trapping_errors_arg__)
+		 (let ((errstr (error-message-string
+				__call_trapping_errors_arg__)))
+		   ,(if no-backtrace
+			`(lwarn ,class ,level
+			   (if (warning-level-<
+				,level
+				display-warning-minimum-level)
+			       "Error in %s: %s"
+			     "Error in %s:\n%s\n")
+			   ,operation errstr)
+		      `(lwarn ,class ,level
+			 "Error in %s: %s\n\nBacktrace follows:\n\n%s"
+			 ,operation errstr
+			 (backtrace-in-condition-handler-eliminating-handler
+			  '__call_trapping_errors_arg__)))))
+	     #'(lambda ()
+		 (progn ,@keys-body)))
+       (error
+	,error-form
+	,@(if resignal '((signal (car __cte_cc_var__) (cdr __cte_cc_var__)))))
+       )))
+
 ;;;; Miscellanea.
 
 ;; This is now in C.
