@@ -113,7 +113,7 @@
 	  (when (and (file-exists-p file-c)
 		     (not (file-exists-p file)))
 	    (message "Removing %s; no corresponding .el" file-c)
-	    (delete-file file-c))))
+	    (delete-file file-c)))))
 
     ;; We descend recursively
     (let ((dirs (directory-files dir t nil t))
@@ -124,8 +124,7 @@
                    (file-directory-p dir))
           (do-update-elc-2 dir compile-stage-p seen))))
 
-    )))
-
+    ))
 
 (defun batch-update-elc-2 ()
   (defvar command-line-args-left)
@@ -142,7 +141,22 @@
 	     dir)
     (message "Recompiling updated .els in directory tree `%s'..." dir)
     (do-update-elc-2 dir t nil)
-    (message "Recompiling updated .els in directory tree `%s'...done" dir))
+    (message "Recompiling updated .els in directory tree `%s'...done" dir)
+    ;; don't depend on being able to autoload `update-autoload-files'!
+    (load "autoload")
+    (update-autoload-files (list dir))
+    (byte-recompile-file (expand-file-name "auto-autoloads.el" dir) 0)
+    (when (featurep 'mule)
+      (update-autoload-files (list (expand-file-name "mule" dir)))
+      (byte-recompile-file (expand-file-name "mule/auto-autoloads.el" dir) 0))
+    ;; likewise here.
+    (load "cus-dep")
+    (Custom-make-dependencies dir)
+    (byte-recompile-file (expand-file-name "custom-load.el" dir) 0)
+    (when (featurep 'mule)
+      (Custom-make-dependencies (expand-file-name "mule" dir))
+      (byte-recompile-file (expand-file-name "mule/custom-load.el" dir) 0))
+    )
   (setq command-line-args-left nil))
 
 ;;; update-elc-2.el ends here
