@@ -235,7 +235,7 @@ merge_scrollbar_values (widget_value *old, widget_value *new)
 
 #endif /* NEED_SCROLLBARS */
 
-#ifdef HAVE_WIDGETS
+#ifdef HAVE_X_WIDGETS
 /*
  * Return true if old->args was not equivalent
  * to new->args.
@@ -264,10 +264,14 @@ merge_widget_value_args (widget_value *old, widget_value *new)
       lw_copy_widget_value_args (new, old);
       changed = True;
     }
+  else if (new->args && new->args == old->args && new->args->args_changed == True)
+    {
+      changed = True;
+    }
 
   return changed;
 }
-#endif /* HAVE_WIDGETS */
+#endif /* HAVE_X_WIDGETS */
 
 /* Make a complete copy of a widget_value tree.  Store CHANGE into
    the widget_value tree's `change' field. */
@@ -617,7 +621,7 @@ merge_widget_value (widget_value *val1, widget_value *val2, int level)
       change = max (change, INVISIBLE_CHANGE);
       val1->call_data = val2->call_data;
     }
-#ifdef HAVE_WIDGETS
+#ifdef HAVE_X_WIDGETS
   if (merge_widget_value_args (val1, val2))
     {
       EXPLAIN (val1->name, change, VISIBLE_CHANGE, "widget change", 0, 0);
@@ -756,7 +760,11 @@ update_all_widget_values (widget_info *info, Boolean deep_p)
     update_one_widget_instance (instance, deep_p);
 
   for (val = info->val; val; val = val->next)
-    val->change = NO_CHANGE;
+    {
+      val->change = NO_CHANGE;
+      if (val->args)
+	val->args->args_changed = False;
+    }
 }
 
 void
@@ -821,7 +829,11 @@ initialize_widget_instance (widget_instance *instance)
   update_one_widget_instance (instance, True);
 
   for (val = instance->info->val; val; val = val->next)
-    val->change = NO_CHANGE;
+    {
+      val->change = NO_CHANGE;
+      if (val->args)
+	val->args->args_changed = False;
+    }
 }
 
 /* strcasecmp() is not sufficiently portable or standard,
@@ -841,9 +853,9 @@ ascii_strcasecmp (const char *s1, const char *s2)
 }
 
 static widget_creation_function
-find_in_table (const char *type, widget_creation_entry *table)
+find_in_table (const char *type, const widget_creation_entry table[])
 {
-  widget_creation_entry *cur;
+  const widget_creation_entry *cur;
   for (cur = table; cur->type; cur++)
     if (!ascii_strcasecmp (type, cur->type))
       return cur->function;
@@ -1377,6 +1389,22 @@ void lw_add_value_args_to_args (widget_value* wv, ArgList addto, int* offset)
     }
 }
 
+XtArgVal lw_get_value_arg (widget_value* wv, String name)
+{
+  int i;
+  if (wv->args)
+    {
+      for (i = 0; i < wv->args->nargs; i++)
+	{
+	  if (!strcmp (wv->args->args[i].name, name))
+	    {
+	      return wv->args->args[i].value;
+	    }
+	}
+    }
+  return (XtArgVal)0;
+}
+
 void lw_add_widget_value_arg (widget_value* wv, String name, XtArgVal value)
 {
   int i = 0;
@@ -1393,6 +1421,8 @@ void lw_add_widget_value_arg (widget_value* wv, String name, XtArgVal value)
   if (wv->args->nargs > 10)
     return;
 
+  /* Register the change. */
+  wv->args->args_changed = True;
   /* If the arg is already there then we must replace it. */
   for (i = 0; i < wv->args->nargs; i++)
     {
