@@ -29,19 +29,21 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
-#include "console-x.h"
-#include "glyphs-x.h"
-#include "objects-x.h"
-#include <X11/Shell.h>
-#include "EmacsFrameP.h"
-#include "EmacsManager.h"	/* for EmacsManagerChangeSize */
-#include "xmu.h"
-
+#include "device.h"
 #include "faces.h"
 #include "frame.h"
 #include "toolbar.h"
 #include "window.h"
 
+#include "console-x.h"
+#include "glyphs-x.h"
+#include "objects-x.h"
+
+#include <X11/Shell.h>
+#include "EmacsFrameP.h"
+#include "EmacsManager.h"	/* for EmacsManagerChangeSize */
+#include "xmu.h"
+
 static void EmacsFrameClassInitialize (void);
 static void EmacsFrameInitialize (Widget, Widget, ArgList, Cardinal *);
 static void EmacsFrameRealize (Widget, XtValueMask*, XSetWindowAttributes*);
@@ -156,13 +158,8 @@ static XtResource resources[] = {
     XtRInt, sizeof (int),
     offset (interline), XtRImmediate, (XtPointer)0 },
   {
-#ifdef I18N4
-    XtNfontSet, XtCFontSet,
-    XtRFontSet, sizeof (XFontSet),
-#else
     XtNfont, XtCFont,
     XtRFontStruct, sizeof (XFontStruct *),
-#endif
     offset(font), XtRImmediate, (XtPointer)0
   },
   { XtNforeground, XtCForeground,
@@ -331,11 +328,6 @@ EmacsFrameRealize (Widget widget, XtValueMask *mask,
     EnterWindowMask;
 
 
-#ifdef I18N4
-  /* Make sure that events wanted by the input method are selected. */
-  attrs->event_mask |= input_method_event_mask;
-#endif
-
   *mask |= CWEventMask;
 
   if (ew->emacs_frame.use_backing_store)
@@ -408,9 +400,7 @@ EmacsFrameSetValues (Widget cur_widget, Widget req_widget, Widget new_widget,
   EmacsFrame cur = (EmacsFrame) cur_widget;
   EmacsFrame new = (EmacsFrame) new_widget;
   struct frame *f = new->emacs_frame.frame;
-  Lisp_Object frame;
-
-  XSETFRAME (frame, f);
+  Lisp_Object frame = wrap_frame (f);
   in_resource_setting++;
   /* This function does not need to do much.  Pretty much everything
      interesting will get done in the resize method, which will
@@ -640,7 +630,7 @@ EmacsFrameRecomputeCellSize (Widget w)
   if (! XtIsSubclass (w, emacsFrameClass))
     abort ();
 
-  default_face_height_and_width (make_frame (f), &ch, &cw);
+  default_face_height_and_width (wrap_frame (f), &ch, &cw);
   if (FRAME_X_TOP_LEVEL_FRAME_P (f))
     x_wm_set_cell_size (FRAME_X_SHELL_WIDGET (f), cw, ch);
 }
