@@ -1,6 +1,7 @@
 /* Execution of byte code produced by bytecomp.el.
    Implementation of compiled-function objects.
    Copyright (C) 1992, 1993 Free Software Foundation, Inc.
+   Copyright (C) 1995 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -871,7 +872,7 @@ execute_optimized_program (const Opbyte *program,
 	case Bunbind_all:
 	  /* To unbind back to the beginning of this frame.  Not used yet,
 	     but will be needed for tail-recursion elimination. */
-	  unbind_to (speccount, Qnil);
+	  unbind_to (speccount);
 	  break;
 
 	case Bnth:
@@ -1147,8 +1148,8 @@ execute_optimized_program (const Opbyte *program,
 
 	case Bcurrent_buffer:
 	  {
-	    Lisp_Object buffer;
-	    XSETBUFFER (buffer, current_buffer);
+	    Lisp_Object buffer = wrap_buffer (current_buffer);
+
 	    PUSH (buffer);
 	    break;
 	  }
@@ -1235,7 +1236,7 @@ execute_rare_opcode (Lisp_Object *stack_ptr,
 	record_unwind_protect (save_window_excursion_unwind,
 			       Fcurrent_window_configuration (Qnil));
 	TOP = Fprogn (TOP);
-	unbind_to (count, Qnil);
+	unbind_to (count);
 	break;
       }
 
@@ -1432,7 +1433,7 @@ execute_rare_opcode (Lisp_Object *stack_ptr,
 	TOP = arg;
 	/* GAG ME!! */
 	/* pop binding of standard-output */
-	unbind_to (specpdl_depth() - 1, Qnil);
+	unbind_to (specpdl_depth() - 1);
 	break;
       }
 
@@ -1898,7 +1899,6 @@ print_compiled_function (Lisp_Object obj, Lisp_Object printcharfun,
   int docp = f->flags.documentationp;
   int intp = f->flags.interactivep;
   struct gcpro gcpro1, gcpro2;
-  char buf[100];
   GCPRO2 (obj, printcharfun);
 
   write_c_string (print_readably ? "#[" : "#<compiled-function ", printcharfun);
@@ -1907,11 +1907,7 @@ print_compiled_function (Lisp_Object obj, Lisp_Object printcharfun,
     {
       Lisp_Object ann = compiled_function_annotation (f);
       if (!NILP (ann))
-	{
-	  write_c_string ("(from ", printcharfun);
-	  print_internal (ann, printcharfun, 1);
-	  write_c_string (") ", printcharfun);
-	}
+	write_fmt_string_lisp (printcharfun, "(from %S) ", 1, ann);
     }
 #endif /* COMPILED_FUNCTION_ANNOTATION_HACK */
   /* COMPILED_ARGLIST = 0 */
@@ -1926,9 +1922,8 @@ print_compiled_function (Lisp_Object obj, Lisp_Object printcharfun,
     if (STRINGP (instructions) && !print_readably)
       {
 	/* We don't usually want to see that junk in the bytecode. */
-	sprintf (buf, "\"...(%ld)\"",
-		 (long) XSTRING_CHAR_LENGTH (instructions));
-	write_c_string (buf, printcharfun);
+	write_fmt_string (printcharfun, "\"...(%ld)\"",
+			  (long) XSTRING_CHAR_LENGTH (instructions));
       }
     else
       print_internal (instructions, printcharfun, escapeflag);
@@ -1940,8 +1935,7 @@ print_compiled_function (Lisp_Object obj, Lisp_Object printcharfun,
   print_internal (compiled_function_constants (f), printcharfun, escapeflag);
 
   /* COMPILED_STACK_DEPTH = 3 */
-  sprintf (buf, " %d", compiled_function_stack_depth (f));
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, " %d", compiled_function_stack_depth (f));
 
   /* COMPILED_DOC_STRING = 4 */
   if (docp || intp)
@@ -2343,7 +2337,7 @@ If the byte code for compiled function FUNCTION is lazy-loaded, fetch it now.
       return function;
     }
   abort ();
-  return Qnil; /* not reached */
+  return Qnil; /* not (usually) reached */
 }
 
 DEFUN ("optimize-compiled-function", Foptimize_compiled_function, 1, 1, 0, /*
