@@ -1,6 +1,7 @@
 /* XEmacs routines to deal with case tables.
    Copyright (C) 1987, 1992, 1993, 1994 Free Software Foundation, Inc.
    Copyright (C) 1995 Sun Microsystems, Inc.
+   Copyright (C) 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -19,21 +20,22 @@ along with XEmacs; see the file COPYING.  If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
- /* Synched up with: FSF 19.28.  Between FSF 19.28 and 19.30, casetab.c
+/* Synched up with: FSF 19.28.  Between FSF 19.28 and 19.30, casetab.c
    was rewritten to use junky FSF char tables.  Meanwhile I rewrote it
-   to use more logical char tables.  RMS also discards the "list of four
-   tables" format and instead stuffs the other tables as "extra slots"
-   in the downcase table.  I've kept the four-lists format for now. */
+   to use more logical char tables. --ben */
 
 /* Written by Howard Gayle.  See some mythical and not-in-the-Emacs-
    distribution file chartab.c for details. */
 
 /* Modified for Mule by Ben Wing. */
 
-/* Case table consists of four char-table.  Those are for downcase,
+/* #### Someone (Yoshiki?) wrote the following comment, which I don't
+   understand.
+
+   Case tables consist of four char-tables.  These are for downcase,
    upcase, canonical and equivalent respectively.
 
-   It's entry is like this:
+   The entries are like this:
 
    downcase:	a -> a, A -> a.
    upcase:	a -> A, A -> a.  (The latter is for NOCASEP.)
@@ -72,12 +74,9 @@ static void
 print_case_table (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   Lisp_Case_Table *ct = XCASE_TABLE (obj);
-  char buf[200];
   if (print_readably)
     printing_unreadable_object ("#<case-table 0x%x", ct->header.uid);
-  write_c_string ("#<case-table ", printcharfun);
-  sprintf (buf, "0x%x>", ct->header.uid);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, "#<case-table 0x%x>", ct->header.uid);
 }
 
 static const struct lrecord_description case_table_description [] = {
@@ -95,7 +94,6 @@ DEFINE_LRECORD_IMPLEMENTATION ("case-table", case_table,
 static Lisp_Object
 allocate_case_table (void)
 {
-  Lisp_Object val;
   Lisp_Case_Table *ct =
     alloc_lcrecord_type (Lisp_Case_Table, &lrecord_case_table);
 
@@ -104,8 +102,7 @@ allocate_case_table (void)
   SET_CASE_TABLE_CANON (ct, Qnil);
   SET_CASE_TABLE_EQV (ct, Qnil);
 
-  XSETCASE_TABLE (val, ct);
-  return val;
+  return wrap_case_table (ct);
 }
 
 DEFUN ("case-table-p", Fcase_table_p, 1, 1, 0, /*
@@ -347,7 +344,7 @@ set_case_table (Lisp_Object table, int standard)
       temp = down;
       down = MAKE_TRT_TABLE ();
       for (i = 0; i < 256; i++)
-	SET_TRT_TABLE_CHAR_1 (down, i, string_char (XSTRING (temp), i));
+	SET_TRT_TABLE_CHAR_1 (down, i, XSTRING_CHAR (temp, i));
 
       if (NILP (up))
 	{
@@ -359,7 +356,7 @@ set_case_table (Lisp_Object table, int standard)
 	  temp = up;
 	  up = MAKE_TRT_TABLE ();
 	  for (i = 0; i < 256; i++)
-	    SET_TRT_TABLE_CHAR_1 (up, i, string_char (XSTRING (temp), i));
+	    SET_TRT_TABLE_CHAR_1 (up, i, XSTRING_CHAR (temp, i));
 	}
       if (NILP (canon))
 	{
@@ -381,7 +378,7 @@ set_case_table (Lisp_Object table, int standard)
 	  temp = canon;
 	  canon = MAKE_TRT_TABLE ();
 	  for (i = 0; i < 256; i++)
-	    SET_TRT_TABLE_CHAR_1 (canon, i, string_char (XSTRING (temp), i));
+	    SET_TRT_TABLE_CHAR_1 (canon, i, XSTRING_CHAR (temp, i));
 	}
 
       if (NILP (eqv))
@@ -394,7 +391,7 @@ set_case_table (Lisp_Object table, int standard)
 	  temp = eqv;
 	  eqv = MAKE_TRT_TABLE ();
 	  for (i = 0; i < 256; i++)
-	    SET_TRT_TABLE_CHAR_1 (eqv, i, string_char (XSTRING (temp), i));
+	    SET_TRT_TABLE_CHAR_1 (eqv, i, XSTRING_CHAR (temp, i));
 	}
 
       if (standard)
