@@ -1,7 +1,7 @@
 /* XEmacs routines to deal with char tables.
    Copyright (C) 1992, 1995 Free Software Foundation, Inc.
    Copyright (C) 1995 Sun Microsystems, Inc.
-   Copyright (C) 1995, 1996 Ben Wing.
+   Copyright (C) 1995, 1996, 2002 Ben Wing.
    Copyright (C) 1995, 1997, 1999 Electrotechnical Laboratory, JAPAN.
    Licensed to the Free Software Foundation.
 
@@ -154,6 +154,8 @@ mark_char_table (Lisp_Object obj)
   for (i = 0; i < NUM_LEADING_BYTES; i++)
     mark_object (ct->level1[i]);
 #endif
+  mark_object (ct->parent);
+  mark_object (ct->default_);
   return ct->mirror_table;
 }
 
@@ -212,7 +214,7 @@ symbol_to_char_table_type (Lisp_Object symbol)
 #endif
 
   invalid_constant ("Unrecognized char table type", symbol);
-  return CHAR_TABLE_TYPE_GENERIC; /* not reached */
+  RETURN_NOT_REACHED (CHAR_TABLE_TYPE_GENERIC)
 }
 
 static void
@@ -220,19 +222,10 @@ print_chartab_range (Emchar first, Emchar last, Lisp_Object val,
 		     Lisp_Object printcharfun)
 {
   if (first != last)
-    {
-      write_c_string (" (", printcharfun);
-      print_internal (make_char (first), printcharfun, 0);
-      write_c_string (" ", printcharfun);
-      print_internal (make_char (last), printcharfun, 0);
-      write_c_string (") ", printcharfun);
-    }
+    write_fmt_string_lisp (printcharfun, " (%s %s)", 2,
+			   make_char (first), make_char (last));
   else
-    {
-      write_c_string (" ", printcharfun);
-      print_internal (make_char (first), printcharfun, 0);
-      write_c_string (" ", printcharfun);
-    }
+    write_fmt_string_lisp (printcharfun, " %s ", 1, make_char (first));
   print_internal (val, printcharfun, 1);
 }
 
@@ -300,13 +293,9 @@ print_chartab_two_byte_charset (Lisp_Object charset,
 
       if (!CHAR_TABLE_ENTRYP (jen))
 	{
-	  char buf[100];
-
-	  write_c_string (" [", printcharfun);
-	  print_internal (XCHARSET_NAME (charset), printcharfun, 0);
-	  sprintf (buf, " %d] ", i);
-	  write_c_string (buf, printcharfun);
-	  print_internal (jen, printcharfun, 0);
+	  write_fmt_string_lisp (printcharfun, " [%s %d] %s",
+				 3, XCHARSET_NAME (charset),
+				 make_int (i), jen);
 	}
       else
 	print_chartab_charset_row (charset, i, XCHAR_TABLE_ENTRY (jen),
@@ -320,12 +309,9 @@ static void
 print_char_table (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   Lisp_Char_Table *ct = XCHAR_TABLE (obj);
-  char buf[200];
 
-  sprintf (buf, "#s(char-table type %s data (",
-	   string_data (symbol_name (XSYMBOL
-				     (char_table_type_to_symbol (ct->type)))));
-  write_c_string (buf, printcharfun);
+  write_fmt_string_lisp (printcharfun, "#s(char-table type %s data (",
+			 1, char_table_type_to_symbol (ct->type));
 
   /* Now write out the ASCII/Control-1 stuff. */
   {
@@ -369,11 +355,8 @@ print_char_table (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 	  continue;
 	if (!CHAR_TABLE_ENTRYP (ann))
 	  {
-	    write_c_string (" ", printcharfun);
-	    print_internal (XCHARSET_NAME (charset),
-			    printcharfun, 0);
-	    write_c_string (" ", printcharfun);
-	    print_internal (ann, printcharfun, 0);
+	    write_fmt_string_lisp (printcharfun, " %s %s", 2,
+				   XCHARSET_NAME (charset), ann);
 	  }
 	else
 	  {
@@ -431,6 +414,8 @@ static const struct lrecord_description char_table_description[] = {
 #ifdef MULE
   { XD_LISP_OBJECT_ARRAY, offsetof (Lisp_Char_Table, level1), NUM_LEADING_BYTES },
 #endif
+  { XD_LISP_OBJECT, offsetof (Lisp_Char_Table, parent) },
+  { XD_LISP_OBJECT, offsetof (Lisp_Char_Table, default_) },
   { XD_LISP_OBJECT, offsetof (Lisp_Char_Table, mirror_table) },
   { XD_LO_LINK,     offsetof (Lisp_Char_Table, next_table) },
   { XD_END }
@@ -444,36 +429,6 @@ DEFINE_LRECORD_IMPLEMENTATION ("char-table", char_table,
 
 DEFUN ("char-table-p", Fchar_table_p, 1, 1, 0, /*
 Return non-nil if OBJECT is a char table.
-
-A char table is a table that maps characters (or ranges of characters)
-to values.  Char tables are specialized for characters, only allowing
-particular sorts of ranges to be assigned values.  Although this
-loses in generality, it makes for extremely fast (constant-time)
-lookups, and thus is feasible for applications that do an extremely
-large number of lookups (e.g. scanning a buffer for a character in
-a particular syntax, where a lookup in the syntax table must occur
-once per character).
-
-When Mule support exists, the types of ranges that can be assigned
-values are
-
--- all characters
--- an entire charset
--- a single row in a two-octet charset
--- a single character
-
-When Mule support is not present, the types of ranges that can be
-assigned values are
-
--- all characters
--- a single character
-
-To create a char table, use `make-char-table'.
-To modify a char table, use `put-char-table' or `remove-char-table'.
-To retrieve the value for a particular character, use `get-char-table'.
-See also `map-char-table', `clear-char-table', `copy-char-table',
-`valid-char-table-type-p', `char-table-type-list',
-`valid-char-table-value-p', and `check-char-table-value'.
 */
        (object))
 {
@@ -482,7 +437,7 @@ See also `map-char-table', `clear-char-table', `copy-char-table',
 
 DEFUN ("char-table-type-list", Fchar_table_type_list, 0, 0, 0, /*
 Return a list of the recognized char table types.
-See `valid-char-table-type-p'.
+See `make-char-table'.
 */
        ())
 {
@@ -495,31 +450,7 @@ See `valid-char-table-type-p'.
 
 DEFUN ("valid-char-table-type-p", Fvalid_char_table_type_p, 1, 1, 0, /*
 Return t if TYPE if a recognized char table type.
-
-Each char table type is used for a different purpose and allows different
-sorts of values.  The different char table types are
-
-`category'
-	Used for category tables, which specify the regexp categories
-	that a character is in.  The valid values are nil or a
-	bit vector of 95 elements.  Higher-level Lisp functions are
-	provided for working with category tables.  Currently categories
-	and category tables only exist when Mule support is present.
-`char'
-	A generalized char table, for mapping from one character to
-	another.  Used for case tables, syntax matching tables,
-	`keyboard-translate-table', etc.  The valid values are characters.
-`generic'
-        An even more generalized char table, for mapping from a
-	character to anything.
-`display'
-	Used for display tables, which specify how a particular character
-	is to appear when displayed.  #### Not yet implemented.
-`syntax'
-	Used for syntax tables, which specify the syntax of a particular
-	character.  Higher-level Lisp functions are provided for
-	working with syntax tables.  The valid values are integers.
-
+See `make-char-table'.
 */
        (type))
 {
@@ -534,7 +465,7 @@ sorts of values.  The different char table types are
 
 DEFUN ("char-table-type", Fchar_table_type, 1, 1, 0, /*
 Return the type of CHAR-TABLE.
-See `valid-char-table-type-p'.
+See `make-char-table'.
 */
        (char_table))
 {
@@ -594,8 +525,60 @@ Reset CHAR-TABLE to its default state.
 
 DEFUN ("make-char-table", Fmake_char_table, 1, 1, 0, /*
 Return a new, empty char table of type TYPE.
-Currently recognized types are 'char, 'category, 'display, 'generic,
-and 'syntax.  See `valid-char-table-type-p'.
+
+A char table is a table that maps characters (or ranges of characters)
+to values.  Char tables are specialized for characters, only allowing
+particular sorts of ranges to be assigned values.  Although this
+loses in generality, it makes for extremely fast (constant-time)
+lookups, and thus is feasible for applications that do an extremely
+large number of lookups (e.g. scanning a buffer for a character in
+a particular syntax, where a lookup in the syntax table must occur
+once per character).
+
+When Mule support exists, the types of ranges that can be assigned
+values are
+
+-- all characters
+-- an entire charset
+-- a single row in a two-octet charset
+-- a single character
+
+When Mule support is not present, the types of ranges that can be
+assigned values are
+
+-- all characters
+-- a single character
+
+To create a char table, use `make-char-table'.
+To modify a char table, use `put-char-table' or `remove-char-table'.
+To retrieve the value for a particular character, use `get-char-table'.
+See also `map-char-table', `clear-char-table', `copy-char-table',
+`char-table-p', `valid-char-table-type-p', `char-table-type-list',
+`valid-char-table-value-p', and `check-char-table-value'.
+
+Each char table type is used for a different purpose and allows different
+sorts of values.  The different char table types are
+
+`category'
+	Used for category tables, which specify the regexp categories
+	that a character is in.  The valid values are nil or a
+	bit vector of 95 elements.  Higher-level Lisp functions are
+	provided for working with category tables.  Currently categories
+	and category tables only exist when Mule support is present.
+`char'
+	A generalized char table, for mapping from one character to
+	another.  Used for case tables, syntax matching tables,
+	`keyboard-translate-table', etc.  The valid values are characters.
+`generic'
+        An even more generalized char table, for mapping from a
+	character to anything.
+`display'
+	Used for display tables, which specify how a particular character
+	is to appear when displayed.  #### Not yet implemented.
+`syntax'
+	Used for syntax tables, which specify the syntax of a particular
+	character.  Higher-level Lisp functions are provided for
+	working with syntax tables.  The valid values are integers.
 */
        (type))
 {
@@ -614,7 +597,9 @@ and 'syntax.  See `valid-char-table-type-p'.
   else
     ct->mirror_table = Qnil;
   ct->next_table = Qnil;
-  XSETCHAR_TABLE (obj, ct);
+  ct->parent = Qnil;
+  ct->default_ = Qnil;
+  obj = wrap_char_table (ct);
   if (ty == CHAR_TABLE_TYPE_SYNTAX)
     {
       ct->next_table = Vall_syntax_tables;
@@ -629,7 +614,6 @@ and 'syntax.  See `valid-char-table-type-p'.
 static Lisp_Object
 make_char_table_entry (Lisp_Object initval)
 {
-  Lisp_Object obj;
   int i;
   Lisp_Char_Table_Entry *cte =
     alloc_lcrecord_type (Lisp_Char_Table_Entry, &lrecord_char_table_entry);
@@ -637,15 +621,13 @@ make_char_table_entry (Lisp_Object initval)
   for (i = 0; i < 96; i++)
     cte->level2[i] = initval;
 
-  XSETCHAR_TABLE_ENTRY (obj, cte);
-  return obj;
+  return wrap_char_table_entry (cte);
 }
 
 static Lisp_Object
 copy_char_table_entry (Lisp_Object entry)
 {
   Lisp_Char_Table_Entry *cte = XCHAR_TABLE_ENTRY (entry);
-  Lisp_Object obj;
   int i;
   Lisp_Char_Table_Entry *ctenew =
     alloc_lcrecord_type (Lisp_Char_Table_Entry, &lrecord_char_table_entry);
@@ -659,8 +641,7 @@ copy_char_table_entry (Lisp_Object entry)
 	ctenew->level2[i] = new;
     }
 
-  XSETCHAR_TABLE_ENTRY (obj, ctenew);
-  return obj;
+  return wrap_char_table_entry (ctenew);
 }
 
 #endif /* MULE */
@@ -680,6 +661,8 @@ as CHAR-TABLE.  The values will not themselves be copied.
   ct = XCHAR_TABLE (char_table);
   ctnew = alloc_lcrecord_type (Lisp_Char_Table, &lrecord_char_table);
   ctnew->type = ct->type;
+  ctnew->parent = ct->parent;
+  ctnew->default_ = ct->default_;
 
   for (i = 0; i < NUM_ASCII_CHARS; i++)
     {
@@ -708,7 +691,7 @@ as CHAR-TABLE.  The values will not themselves be copied.
   else
     ctnew->mirror_table = ct->mirror_table;
   ctnew->next_table = Qnil;
-  XSETCHAR_TABLE (obj, ctnew);
+  obj = wrap_char_table (ctnew);
   if (ctnew->type == CHAR_TABLE_TYPE_SYNTAX)
     {
       ctnew->next_table = Vall_syntax_tables;
@@ -943,7 +926,7 @@ If there is more than one value, return MULTI (defaults to nil).
       abort ();
     }
 
-  return Qnil; /* not reached */
+  return Qnil; /* not (usually) reached */
 }
 
 static int
@@ -994,7 +977,7 @@ check_valid_char_table_value (Lisp_Object value, enum char_table_type type,
       abort ();
     }
 
-  return 0; /* not reached */
+  return 0; /* not (usually) reached */
 }
 
 static Lisp_Object
@@ -1148,7 +1131,7 @@ one of the following:
 -- A single character
 
 VALUE must be a value appropriate for the type of CHAR-TABLE.
-See `valid-char-table-type-p'.
+See `make-char-table'.
 */
        (range, value, char_table))
 {
@@ -1346,7 +1329,8 @@ map_char_table (Lisp_Char_Table *ct,
 
 	  for (i = start, retval = 0; i < stop && retval == 0; i++)
 	    {
-	      retval = map_over_other_charset (ct, i, fn, arg);
+	      if (i != LEADING_BYTE_ASCII && i != LEADING_BYTE_CONTROL_1)
+		retval = map_over_other_charset (ct, i, fn, arg);
 	    }
 	}
 #endif /* MULE */
@@ -1361,7 +1345,8 @@ map_char_table (Lisp_Char_Table *ct,
 
     case CHARTAB_RANGE_ROW:
       {
-	Lisp_Object val = ct->level1[XCHARSET_LEADING_BYTE (range->charset) - MIN_LEADING_BYTE];
+	Lisp_Object val = ct->level1[XCHARSET_LEADING_BYTE (range->charset) -
+				     MIN_LEADING_BYTE];
 	if (!CHAR_TABLE_ENTRYP (val))
 	  {
 	    struct chartab_range rainj;
@@ -1616,10 +1601,8 @@ check_category_char (Emchar ch, Lisp_Object table,
 {
   REGISTER Lisp_Object temp;
   Lisp_Char_Table *ctbl;
-#ifdef ERROR_CHECK_TYPECHECK
   if (NILP (Fcategory_table_p (table)))
     wtaerror ("Expected category table", table);
-#endif
   ctbl = XCHAR_TABLE (table);
   temp = get_char_table (ch, ctbl);
   if (NILP (temp))
@@ -1646,15 +1629,15 @@ use, and defaults to BUFFER's category table.
   CHECK_INT (position);
   CHECK_CATEGORY_DESIGNATOR (designator);
   des = XCHAR (designator);
-  ctbl = check_category_table (category_table, Vstandard_category_table);
+  ctbl = check_category_table (category_table, buf->category_table);
   ch = BUF_FETCH_CHAR (buf, XINT (position));
   return check_category_char (ch, ctbl, des, 0) ? Qt : Qnil;
 }
 
 DEFUN ("char-in-category-p", Fchar_in_category_p, 2, 3, 0, /*
-Return t if category of CHARACTER includes DESIGNATOR, else nil.
+Return non-nil if category of CHARACTER includes DESIGNATOR.
 Optional third arg CATEGORY-TABLE specifies the category table to use,
-and defaults to the standard category table.
+and defaults to the current buffer's category table.
 */
        (character, designator, category_table))
 {
@@ -1666,7 +1649,7 @@ and defaults to the standard category table.
   des = XCHAR (designator);
   CHECK_CHAR (character);
   ch = XCHAR (character);
-  ctbl = check_category_table (category_table, Vstandard_category_table);
+  ctbl = check_category_table (category_table, current_buffer->category_table);
   return check_category_char (ch, ctbl, des, 0) ? Qt : Qnil;
 }
 
