@@ -1,6 +1,6 @@
 /* The console object.
    Copyright (C) 1992, 1993, 1994 Free Software Foundation, Inc.
-   Copyright (C) 1996 Ben Wing.
+   Copyright (C) 1996, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -27,13 +27,15 @@ Boston, MA 02111-1307, USA.  */
 #include "lisp.h"
 
 #include "buffer.h"
-#include "console-tty.h"
+#include "device.h"
 #include "events.h"
 #include "frame.h"
 #include "redisplay.h"
 #include "sysdep.h"
 #include "window.h"
 
+#include "console-tty.h"
+
 Lisp_Object Vconsole_list, Vselected_console;
 
 Lisp_Object Vcreate_console_hook, Vdelete_console_hook;
@@ -119,22 +121,17 @@ static void
 print_console (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   struct console *con = XCONSOLE (obj);
-  char buf[256];
 
   if (print_readably)
     printing_unreadable_object ("#<console %s 0x%x>",
 				XSTRING_DATA (con->name), con->header.uid);
 
-  sprintf (buf, "#<%s-console", !CONSOLE_LIVE_P (con) ? "dead" :
-	   CONSOLE_TYPE_NAME (con));
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, "#<%s-console",
+		    !CONSOLE_LIVE_P (con) ? "dead" : CONSOLE_TYPE_NAME (con));
   if (CONSOLE_LIVE_P (con) && !NILP (CONSOLE_CONNECTION (con)))
-    {
-      write_c_string (" on ", printcharfun);
-      print_internal (CONSOLE_CONNECTION (con), printcharfun, 1);
-    }
-  sprintf (buf, " 0x%x>", con->header.uid);
-  write_c_string (buf, printcharfun);
+    write_fmt_string_lisp (printcharfun, " on %S", 1,
+			   CONSOLE_CONNECTION (con));
+  write_fmt_string (printcharfun, " 0x%x>", con->header.uid);
 }
 
 DEFINE_LRECORD_IMPLEMENTATION ("console", console,
@@ -150,11 +147,11 @@ allocate_console (void)
 
   copy_lcrecord (con, XCONSOLE (Vconsole_defaults));
 
-  XSETCONSOLE (console, con);
+  console = wrap_console (con);
   GCPRO1 (console);
 
   con->quit_char = 7; /* C-g */
-  con->command_builder = allocate_command_builder (console);
+  con->command_builder = allocate_command_builder (console, 1);
   con->function_key_map = Fmake_sparse_keymap (Qnil);
 
   UNGCPRO;
@@ -347,14 +344,6 @@ CONSOLE defaults to the selected console if omitted.
   return CONSOLE_CONNECTION (decode_console (console));
 }
 
-Lisp_Object
-make_console (struct console *con)
-{
-  Lisp_Object console;
-  XSETCONSOLE (console, con);
-  return console;
-}
-
 static Lisp_Object
 semi_canonicalize_console_connection (struct console_methods *meths,
 				      Lisp_Object name, Error_Behavior errb)
@@ -481,7 +470,7 @@ create_console (Lisp_Object name, Lisp_Object type, Lisp_Object connection,
     return console;
 
   con = allocate_console ();
-  XSETCONSOLE (console, con);
+  console = wrap_console (con);
 
   GCPRO1 (console);
 
@@ -613,7 +602,7 @@ delete_console_internal (struct console *con, int force,
   if (!CONSOLE_LIVE_P (con))
     return;
 
-  XSETCONSOLE (console, con);
+  console = wrap_console (con);
   GCPRO1 (console);
 
   if (!called_from_kill_emacs)
@@ -874,7 +863,7 @@ On such systems, Emacs will start a subshell and wait for it to exit.
   stuff_buffered_input (stuffstring);
   sys_suspend ();
   /* the console is un-reset inside of the unwind-protect. */
-  unbind_to (speccount, Qnil);
+  unbind_to (speccount);
 
 #ifdef SIGWINCH
   /* It is possible that a size change occurred while we were
@@ -1281,8 +1270,8 @@ common_init_complex_vars_of_console (void)
 
   staticpro_nodump (&Vconsole_defaults);
   staticpro_nodump (&Vconsole_local_symbols);
-  XSETCONSOLE (Vconsole_defaults, defs);
-  XSETCONSOLE (Vconsole_local_symbols, syms);
+  Vconsole_defaults = wrap_console (defs);
+  Vconsole_local_symbols = wrap_console (syms);
 
   nuke_all_console_slots (syms, Qnil);
   nuke_all_console_slots (defs, Qnil);
@@ -1342,7 +1331,7 @@ common_init_complex_vars_of_console (void)
 #define CONSOLE_SLOTS_COUNT (CONSOLE_SLOTS_SIZE / sizeof (Lisp_Object))
 
 void
-reinit_complex_vars_of_console (void)
+reinit_complex_vars_of_console_runtime_only (void)
 {
   struct console *defs, *syms;
 
