@@ -1,7 +1,7 @@
 /* Primitive operations on Lisp data types for XEmacs Lisp interpreter.
    Copyright (C) 1985, 1986, 1988, 1992, 1993, 1994, 1995
    Free Software Foundation, Inc.
-   Copyright (C) 2000, 2001 Ben Wing.
+   Copyright (C) 2000, 2001, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -31,12 +31,7 @@ Boston, MA 02111-1307, USA.  */
 #include "buffer.h"
 #include "bytecode.h"
 #include "syssignal.h"
-
-#ifdef LISP_FLOAT_TYPE
-/* Need to define a differentiating symbol -- see sysfloat.h */
-# define THIS_FILENAME data_c
-# include "sysfloat.h"
-#endif /* LISP_FLOAT_TYPE */
+#include "sysfloat.h"
 
 Lisp_Object Qnil, Qt, Qquote, Qlambda, Qunbound;
 Lisp_Object Qerror_conditions, Qerror_message;
@@ -368,10 +363,10 @@ If non-nil, the return value will be a list whose first element is
 */
        (subr))
 {
-  const char *prompt;
+  const CIntbyte *prompt;
   CHECK_SUBR (subr);
   prompt = XSUBR (subr)->prompt;
-  return prompt ? list2 (Qinteractive, build_string (prompt)) : Qnil;
+  return prompt ? list2 (Qinteractive, build_msg_string (prompt)) : Qnil;
 }
 
 
@@ -727,7 +722,7 @@ ARRAY may be a vector, bit vector, or string.  INDEX starts at 0.
   else if (STRINGP (array))
     {
       if (idx >= XSTRING_CHAR_LENGTH (array)) goto range_error;
-      return make_char (string_char (XSTRING (array), idx));
+      return make_char (XSTRING_CHAR (array, idx));
     }
 #ifdef LOSING_BYTECODE
   else if (COMPILED_FUNCTIONP (array))
@@ -745,7 +740,7 @@ ARRAY may be a vector, bit vector, or string.  INDEX starts at 0.
 
  range_error:
   args_out_of_range (array, index_);
-  return Qnil; /* not reached */
+  RETURN_NOT_REACHED (Qnil)
 }
 
 DEFUN ("aset", Faset, 3, 3, 0, /*
@@ -768,6 +763,7 @@ ARRAY may be a vector, bit vector, or string.  INDEX starts at 0.
 
   if (idx < 0) goto range_error;
 
+  CHECK_LISP_WRITEABLE (array);
   if (VECTORP (array))
     {
       if (idx >= XVECTOR_LENGTH (array)) goto range_error;
@@ -784,7 +780,7 @@ ARRAY may be a vector, bit vector, or string.  INDEX starts at 0.
     {
       CHECK_CHAR_COERCE_INT (newval);
       if (idx >= XSTRING_CHAR_LENGTH (array)) goto range_error;
-      set_string_char (XSTRING (array), idx, XCHAR (newval));
+      set_string_char (array, idx, XCHAR (newval));
       bump_string_modiff (array);
     }
   else
@@ -797,7 +793,7 @@ ARRAY may be a vector, bit vector, or string.  INDEX starts at 0.
 
  range_error:
   args_out_of_range (array, index_);
-  return Qnil; /* not reached */
+  RETURN_NOT_REACHED (Qnil)
 }
 
 
@@ -1269,7 +1265,7 @@ With one argument, reciprocates the argument.
 
  divide_by_zero:
   Fsignal (Qarith_error, Qnil);
-  return Qnil; /* not reached */
+  return Qnil; /* not (usually) reached */
 }
 
 DEFUN ("max", Fmax, 1, MANY, 0, /*
@@ -1494,7 +1490,7 @@ If either argument is a float, a float will be returned.
 
  divide_by_zero:
   Fsignal (Qarith_error, Qnil);
-  return Qnil; /* not reached */
+  return Qnil; /* not (usually) reached */
 }
 
 DEFUN ("ash", Fash, 2, 2, 0, /*
@@ -1593,12 +1589,9 @@ print_weak_list (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
   if (print_readably)
     printing_unreadable_object ("#<weak-list>");
 
-  write_c_string ("#<weak-list ", printcharfun);
-  print_internal (encode_weak_list_type (XWEAK_LIST (obj)->type),
-		  printcharfun, 0);
-  write_c_string (" ", printcharfun);
-  print_internal (XWEAK_LIST (obj)->list, printcharfun, escapeflag);
-  write_c_string (">", printcharfun);
+  write_fmt_string_lisp (printcharfun, "#<weak-list %s %S>", 2,
+			 encode_weak_list_type (XWEAK_LIST (obj)->type),
+			 XWEAK_LIST (obj)->list);
 }
 
 static int
@@ -1629,7 +1622,7 @@ make_weak_list (enum weak_list_type type)
 
   wl->list = Qnil;
   wl->type = type;
-  XSETWEAK_LIST (result, wl);
+  result = wrap_weak_list (wl);
   wl->next_weak = Vall_weak_lists;
   Vall_weak_lists = result;
   return result;
@@ -1920,7 +1913,7 @@ decode_weak_list_type (Lisp_Object symbol)
   if (EQ (symbol, Qfull_assoc))  return WEAK_LIST_FULL_ASSOC;
 
   invalid_constant ("Invalid weak list type", symbol);
-  return WEAK_LIST_SIMPLE; /* not reached */
+  RETURN_NOT_REACHED (WEAK_LIST_SIMPLE)
 }
 
 static Lisp_Object
@@ -1937,7 +1930,7 @@ encode_weak_list_type (enum weak_list_type type)
       abort ();
     }
 
-  return Qnil; /* not reached */
+  return Qnil; /* not (usually) reached */
 }
 
 DEFUN ("weak-list-p", Fweak_list_p, 1, 1, 0, /*
