@@ -75,7 +75,7 @@ static void gtk_device_init_x_specific_cruft (struct device *d);
 struct device *
 decode_gtk_device (Lisp_Object device)
 {
-  XSETDEVICE (device, decode_device (device));
+  device = wrap_device (decode_device (device));
   CHECK_GTK_DEVICE (device);
   return XDEVICE (device);
 }
@@ -121,18 +121,6 @@ gtk_init_device_class (struct device *d)
 extern void gdk_imlib_init(void);
 #endif
 
-extern void emacs_gtk_selection_handle (GtkWidget *,
-					GtkSelectionData *selection_data,
-					guint info,
-					guint time_stamp,
-					gpointer data);
-extern void emacs_gtk_selection_clear_event_handle (GtkWidget *widget,
-                                                    GdkEventSelection *event,
-                                                    gpointer data);
-extern void emacs_gtk_selection_received (GtkWidget *widget,
-					  GtkSelectionData *selection_data,
-					  gpointer user_data);
-
 #ifdef HAVE_BONOBO
 static CORBA_ORB orb;
 #endif
@@ -195,7 +183,7 @@ gtk_init_device (struct device *d, Lisp_Object props)
   GdkVisual *visual = NULL;
   GdkColormap *cmap = NULL;
 
-  XSETDEVICE (device, d);
+  device = wrap_device (d);
 
   /* gtk_init() and even gtk_check_init() are so brain dead that
      getting an empty argv array causes them to abort. */
@@ -351,7 +339,7 @@ gtk_delete_device (struct device *d)
   int checking_free;
 #endif
 
-  XSETDEVICE (device, d);
+  device = wrap_device (d);
   if (1)
     {
 #ifdef FREE_CHECKING
@@ -399,7 +387,7 @@ gtk_event_name (GdkEventType event_type)
 {
   GtkEnumValue *vals = gtk_type_enum_get_values (GTK_TYPE_GDK_EVENT_TYPE);
 
-  while (vals && (vals->value != event_type)) vals++;
+  while (vals && ((GdkEventType)(vals->value) != event_type)) vals++;
 
   if (vals)
     return (vals->value_nick);
