@@ -1,7 +1,7 @@
 /* Generic device functions.
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
    Copyright (C) 1994, 1995 Free Software Foundation, Inc.
-   Copyright (C) 1995, 1996 Ben Wing
+   Copyright (C) 1995, 1996, 2002 Ben Wing
 
 This file is part of XEmacs.
 
@@ -39,6 +39,7 @@ Boston, MA 02111-1307, USA.  */
 #include "redisplay.h"
 #include "specifier.h"
 #include "sysdep.h"
+#include "toolbar.h"
 #include "window.h"
 
 #ifdef HAVE_SCROLLBARS
@@ -99,22 +100,16 @@ static void
 print_device (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   struct device *d = XDEVICE (obj);
-  char buf[256];
 
   if (print_readably)
     printing_unreadable_object ("#<device %s 0x%x>",
 				XSTRING_DATA (d->name), d->header.uid);
 
-  sprintf (buf, "#<%s-device", !DEVICE_LIVE_P (d) ? "dead" :
-	   DEVICE_TYPE_NAME (d));
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, "#<%s-device", !DEVICE_LIVE_P (d) ? "dead" :
+		    DEVICE_TYPE_NAME (d));
   if (DEVICE_LIVE_P (d) && !NILP (DEVICE_CONNECTION (d)))
-    {
-      write_c_string (" on ", printcharfun);
-      print_internal (DEVICE_CONNECTION (d), printcharfun, 1);
-    }
-  sprintf (buf, " 0x%x>", d->header.uid);
-  write_c_string (buf, printcharfun);
+    write_fmt_string_lisp (printcharfun, " on %S", 1, DEVICE_CONNECTION (d));
+  write_fmt_string (printcharfun, " 0x%x>", d->header.uid);
 }
 
 DEFINE_LRECORD_IMPLEMENTATION ("device", device,
@@ -160,7 +155,7 @@ allocate_device (Lisp_Object console)
   struct device *d = alloc_lcrecord_type (struct device, &lrecord_device);
   struct gcpro gcpro1;
 
-  XSETDEVICE (device, d);
+  device = wrap_device (d);
   GCPRO1 (device);
 
   nuke_all_device_slots (d, Qnil);
@@ -277,7 +272,7 @@ If DEVICE is the selected device, this makes FRAME the selected frame.
 */
        (device, frame))
 {
-  XSETDEVICE (device, decode_device (device));
+  device = wrap_device (decode_device (device));
   CHECK_LIVE_FRAME (frame);
 
   if (! EQ (device, FRAME_DEVICE (XFRAME (frame))))
@@ -558,7 +553,7 @@ have no effect.
 
   con = XCONSOLE (console);
   d = allocate_device (console);
-  XSETDEVICE (device, d);
+  device = wrap_device (d);
 
   d->devmeths = con->conmeths;
 
@@ -597,7 +592,7 @@ have no effect.
   setup_device_initial_specifier_tags (d);
 
   UNGCPRO;
-  unbind_to(speccount, Qnil);
+  unbind_to (speccount);
   return device;
 }
 
@@ -691,7 +686,7 @@ delete_device_internal (struct device *d, int force,
   if (!DEVICE_LIVE_P (d))
     return;
 
-  XSETDEVICE (device, d);
+  device = wrap_device (d);
   GCPRO1 (device);
 
   c = XCONSOLE (DEVICE_CONSOLE (d));
@@ -870,7 +865,7 @@ behavior cannot necessarily be determined automatically.
        (device, class))
 {
   struct device *d = decode_device (device);
-  XSETDEVICE (device, d);
+  device = wrap_device (d);
   if (!DEVICE_TTY_P (d))
     gui_error ("Cannot change the class of this device", device);
   if (!EQ (class, Qcolor) && !EQ (class, Qmono) && !EQ (class, Qgrayscale))
@@ -1206,19 +1201,28 @@ handle_asynch_device_change (void)
 #endif
 }
 
+static Lisp_Object
+unlock_device (Lisp_Object d)
+{
+  UNLOCK_DEVICE (XDEVICE (d));
+  return Qnil;
+}
+
 void
 call_critical_lisp_code (struct device *d, Lisp_Object function,
 			 Lisp_Object object)
 {
-  int old_gc_currently_forbidden = gc_currently_forbidden;
-  Lisp_Object old_inhibit_quit = Vinhibit_quit;
+  int count = begin_gc_forbidden ();
+  specbind (Qinhibit_quit, Qt);
+  record_unwind_protect (unlock_device, wrap_device (d));
 
-  /* There's no reason to bother doing specbinds here, because if
+  /* [[There's no real reason to bother doing unwind-protects, because if
      initialize-*-faces signals an error, emacs is going to crash
-     immediately.
+     immediately.]] But this sucks!  This code is called not only during
+     the initial device, but for other devices as well!  #### When dealing
+     with non-initial devices, we should signal an error but NOT kill
+     ourselves! --ben
      */
-  gc_currently_forbidden = 1;
-  Vinhibit_quit = Qt;
   LOCK_DEVICE (d);
 
   /* But it's useful to have an error handler; otherwise an infinite
@@ -1228,9 +1232,7 @@ call_critical_lisp_code (struct device *d, Lisp_Object function,
   else
     call0_with_handler (Qreally_early_error_handler, function);
 
-  UNLOCK_DEVICE (d);
-  Vinhibit_quit = old_inhibit_quit;
-  gc_currently_forbidden = old_gc_currently_forbidden;
+  unbind_to (count);
 }
 
 
