@@ -138,7 +138,6 @@ struct device
 
 DECLARE_LRECORD (device, struct device);
 #define XDEVICE(x) XRECORD (x, device, struct device)
-#define XSETDEVICE(x, p) XSETRECORD (x, p, device)
 #define wrap_device(p) wrap_record (p, device)
 #define DEVICEP(x) RECORDP (x, device)
 #define CHECK_DEVICE(x) CHECK_RECORD (x, device)
@@ -157,7 +156,7 @@ DECLARE_LRECORD (device, struct device);
 
 #define DEVICE_TYPE_P(d, type)	EQ (DEVICE_TYPE (d), Q##type)
 
-#ifdef ERROR_CHECK_TYPECHECK
+#ifdef ERROR_CHECK_TYPES
 INLINE_HEADER struct device *
 error_check_device_type (struct device *d, Lisp_Object sym);
 INLINE_HEADER struct device *
