@@ -83,7 +83,7 @@ signal_ldap_error (LDAP *ld, LDAPMessage *res, int ldap_err)
 #endif
     }
   invalid_operation ("LDAP error",
-                       build_string (ldap_err2string (ldap_err)));
+		     build_string (ldap_err2string (ldap_err)));
 }
 
 
@@ -94,9 +94,7 @@ signal_ldap_error (LDAP *ld, LDAPMessage *res, int ldap_err)
 static Lisp_Object
 make_ldap (Lisp_LDAP *ldap)
 {
-  Lisp_Object lisp_ldap;
-  XSETLDAP (lisp_ldap, ldap);
-  return lisp_ldap;
+  return wrap_ldap (ldap);
 }
 
 static Lisp_Object
@@ -108,19 +106,15 @@ mark_ldap (Lisp_Object obj)
 static void
 print_ldap (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
-  char buf[32];
-
   Lisp_LDAP *ldap = XLDAP (obj);
 
   if (print_readably)
     printing_unreadable_object ("#<ldap %s>", XSTRING_DATA (ldap->host));
 
-  write_c_string ("#<ldap ", printcharfun);
-  print_internal (ldap->host, printcharfun, 1);
+  write_fmt_string_lisp (printcharfun, "#<ldap %S", 1, ldap->host);
   if (!ldap->ld)
     write_c_string ("(dead) ",printcharfun);
-  sprintf (buf, " 0x%lx>", (long)ldap);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, " 0x%lx>", (long)ldap);
 }
 
 static Lisp_LDAP *
@@ -333,7 +327,7 @@ the LDAP library XEmacs was compiled with: `simple', `krbv41' and `krbv42'.
       Intbyte *interrmess;
       EXTERNAL_TO_C_STRING (ldap_err2string (err), interrmess, Qnative);
       signal_error (Qprocess_error, "Failed binding to the server",
-		    build_string (interrmess));
+		    build_intstring (interrmess));
     }
 
   ldap = allocate_ldap ();
@@ -582,7 +576,7 @@ entry according to the value of WITHDN.
   if (! NILP (verbose))
     message ("Parsing ldap results... done");
 
-  unbind_to (speccount, Qnil);
+  unbind_to (speccount);
   UNGCPRO;
   return Fnreverse (result);
 }
