@@ -1,6 +1,6 @@
 /* Implementation of the hash table lisp object type.
    Copyright (C) 1992, 1993, 1994 Free Software Foundation, Inc.
-   Copyright (C) 1995, 1996 Ben Wing.
+   Copyright (C) 1995, 1996, 2002 Ben Wing.
    Copyright (C) 1997 Free Software Foundation, Inc.
 
 This file is part of XEmacs.
@@ -120,15 +120,7 @@ struct Lisp_Hash_Table
 	  (probe = entries, !HENTRY_CLEAR_P (probe)) : 0);	\
        probe++)
 
-#ifndef ERROR_CHECK_HASH_TABLE
-# ifdef ERROR_CHECK_TYPECHECK
-#  define ERROR_CHECK_HASH_TABLE 1
-# else
-#  define ERROR_CHECK_HASH_TABLE 0
-# endif
-#endif
-
-#if ERROR_CHECK_HASH_TABLE
+#ifdef ERROR_CHECK_STRUCTURES
 static void
 check_hash_table_invariants (Lisp_Hash_Table *ht)
 {
@@ -335,8 +327,7 @@ print_hash_table_data (Lisp_Hash_Table *ht, Lisp_Object printcharfun)
 	    break;
 	  }
 	print_internal (e->key, printcharfun, 1);
-	write_c_string (" ", printcharfun);
-	print_internal (e->value, printcharfun, 1);
+	write_fmt_string_lisp (printcharfun, " %S", 1, e->value);
 	count++;
       }
 
@@ -347,7 +338,6 @@ static void
 print_hash_table (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   Lisp_Hash_Table *ht = XHASH_TABLE (obj);
-  char buf[128];
 
   write_c_string (print_readably ? "#s(hash-table" : "#<hash-table",
 		  printcharfun);
@@ -367,21 +357,21 @@ print_hash_table (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
   if (ht->count || !print_readably)
     {
       if (print_readably)
-	sprintf (buf, " size %ld", (long) ht->count);
+	write_fmt_string (printcharfun, " size %ld", (long) ht->count);
       else
-	sprintf (buf, " size %ld/%ld", (long) ht->count, (long) ht->size);
-      write_c_string (buf, printcharfun);
+	write_fmt_string (printcharfun, " size %ld/%ld", (long) ht->count,
+			  (long) ht->size);
     }
 
   if (ht->weakness != HASH_TABLE_NON_WEAK)
     {
-      sprintf (buf, " weakness %s",
-	       (ht->weakness == HASH_TABLE_WEAK		  ? "key-and-value" :
-		ht->weakness == HASH_TABLE_KEY_WEAK	  ? "key" :
-		ht->weakness == HASH_TABLE_VALUE_WEAK	  ? "value" :
-		ht->weakness == HASH_TABLE_KEY_VALUE_WEAK ? "key-or-value" :
-		"you-d-better-not-see-this"));
-      write_c_string (buf, printcharfun);
+      write_fmt_string
+	(printcharfun, " weakness %s",
+	 (ht->weakness == HASH_TABLE_WEAK	    ? "key-and-value" :
+	  ht->weakness == HASH_TABLE_KEY_WEAK	    ? "key" :
+	  ht->weakness == HASH_TABLE_VALUE_WEAK	    ? "value" :
+	  ht->weakness == HASH_TABLE_KEY_VALUE_WEAK ? "key-or-value" :
+	  "you-d-better-not-see-this"));
     }
 
   if (ht->count)
@@ -391,15 +381,14 @@ print_hash_table (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
     write_c_string (")", printcharfun);
   else
     {
-      sprintf (buf, " 0x%x>", ht->header.uid);
-      write_c_string (buf, printcharfun);
+      write_fmt_string (printcharfun, " 0x%x>", ht->header.uid);
     }
 }
 
 static void
 free_hentries (hentry *hentries, size_t size)
 {
-#if ERROR_CHECK_HASH_TABLE
+#ifdef ERROR_CHECK_STRUCTURES
   /* Ensure a crash if other code uses the discarded entries afterwards. */
   hentry *e, *sentinel;
 
@@ -540,7 +529,7 @@ make_general_lisp_hash_table (hash_table_hash_function_t hash_function,
   /* We leave room for one never-occupied sentinel hentry at the end.  */
   ht->hentries = xnew_array_and_zero (hentry, ht->size + 1);
 
-  XSETHASH_TABLE (hash_table, ht);
+  hash_table = wrap_hash_table (ht);
 
   if (weakness == HASH_TABLE_NON_WEAK)
     ht->next_weak = Qunbound;
@@ -628,7 +617,7 @@ decode_hash_table_weakness (Lisp_Object obj)
   if (EQ (obj, Qvalue_weak))		return HASH_TABLE_VALUE_WEAK;
 
   invalid_constant ("Invalid hash table weakness", obj);
-  return HASH_TABLE_NON_WEAK; /* not reached */
+  RETURN_NOT_REACHED (HASH_TABLE_NON_WEAK)
 }
 
 static int
@@ -654,7 +643,7 @@ decode_hash_table_test (Lisp_Object obj)
   if (EQ (obj, Qeql))	return HASH_TABLE_EQL;
 
   invalid_constant ("Invalid hash table test", obj);
-  return HASH_TABLE_EQ; /* not reached */
+  RETURN_NOT_REACHED (HASH_TABLE_EQ)
 }
 
 static int
@@ -944,7 +933,7 @@ The keys and values will not themselves be copied.
   ht->hentries = xnew_array (hentry, ht_old->size + 1);
   memcpy (ht->hentries, ht_old->hentries, (ht_old->size + 1) * sizeof (hentry));
 
-  XSETHASH_TABLE (hash_table, ht);
+  hash_table = wrap_hash_table (ht);
 
   if (! EQ (ht->next_weak, Qunbound))
     {
@@ -1324,7 +1313,7 @@ may remhash or puthash the entry currently being processed by FUNCTION.
       Ffuncall (countof (args), args);
     }
 
-  unbind_to (speccount, Qnil);
+  unbind_to (speccount);
   UNGCPRO;
 
   return Qnil;
@@ -1368,7 +1357,7 @@ elisp_maphash (maphash_function_t function,
     if (function (pobj[0], pobj[1], extra_arg))
       break;
 
-  unbind_to (speccount, Qnil);
+  unbind_to (speccount);
   UNGCPRO;
 }
 
@@ -1392,7 +1381,7 @@ elisp_map_remhash (maphash_function_t predicate,
     if (predicate (pobj[0], pobj[1], extra_arg))
       Fremhash (pobj[0], hash_table);
 
-  unbind_to (speccount, Qnil);
+  unbind_to (speccount);
   UNGCPRO;
 }
 
@@ -1642,8 +1631,6 @@ The value is returned as (HIGH . LOW).
 void
 syms_of_elhash (void)
 {
-  INIT_LRECORD_IMPLEMENTATION (hash_table);
-
   DEFSUBR (Fhash_table_p);
   DEFSUBR (Fmake_hash_table);
   DEFSUBR (Fcopy_hash_table);
@@ -1689,8 +1676,10 @@ syms_of_elhash (void)
 }
 
 void
-vars_of_elhash (void)
+init_elhash_once_early (void)
 {
+  INIT_LRECORD_IMPLEMENTATION (hash_table);
+
   /* This must NOT be staticpro'd */
   Vall_weak_hash_tables = Qnil;
   dump_add_weak_object_chain (&Vall_weak_hash_tables);
