@@ -1,7 +1,7 @@
 /* The event_stream interface for X11 with gtk, and/or tty frames.
    Copyright (C) 1991-5, 1997 Free Software Foundation, Inc.
    Copyright (C) 1995 Sun Microsystems, Inc.
-   Copyright (C) 1996 Ben Wing.
+   Copyright (C) 1996, 2001, 2002 Ben Wing.
    Copyright (C) 2000 William Perry.
 
 This file is part of XEmacs.
@@ -47,10 +47,8 @@ Boston, MA 02111-1307, USA.  */
 #include "systime.h"
 #include "sysproc.h" /* for MAXDESC */
 
-#ifdef FILE_CODING
 #include "lstream.h"
 #include "file-coding.h"
-#endif
 
 #include <gdk/gdkkeysyms.h>
 
@@ -62,10 +60,10 @@ Boston, MA 02111-1307, USA.  */
 #include "offix.h"
 #endif
 
-#include "events-mod.h"
-
 #include <gdk/gdkx.h>
 
+#include "event-gtk.h"
+
 static struct event_stream *gtk_event_stream;
 
 /* Do we accept events sent by other clients? */
@@ -128,7 +126,7 @@ handle_focus_event_1 (struct frame *f, int in_p)
 	Lisp_Object conser;
 	struct gcpro gcpro1;
 
-	XSETFRAME (frm, f);
+	frm = wrap_frame (f);
 	conser = Fcons (frm, Fcons (FRAME_DEVICE (f), in_p ? Qt : Qnil));
 	GCPRO1 (conser);
 
@@ -144,9 +142,8 @@ handle_focus_event_1 (struct frame *f, int in_p)
 static void
 change_frame_visibility (struct frame *f, int is_visible)
 {
-  Lisp_Object frame;
+  Lisp_Object frame = wrap_frame (f);
 
-  XSETFRAME (frame, f);
 
   if (!FRAME_VISIBLE_P (f) && is_visible)
     {
@@ -175,9 +172,8 @@ change_frame_visibility (struct frame *f, int is_visible)
 static void
 handle_map_event (struct frame *f, GdkEvent *event)
 {
-  Lisp_Object frame;
+  Lisp_Object frame = wrap_frame (f);
 
-  XSETFRAME (frame, f);
   if (event->any.type == GDK_MAP)
     {
       FRAME_GTK_TOTALLY_VISIBLE_P (f) = 1;
@@ -196,9 +192,8 @@ handle_map_event (struct frame *f, GdkEvent *event)
 static void
 handle_client_message (struct frame *f, GdkEvent *event)
 {
-  Lisp_Object frame;
+  Lisp_Object frame = wrap_frame (f);
 
-  XSETFRAME (frame, f);
 
   /* The event-Xt code used to handle WM_DELETE_WINDOW here, but we
      handle that directly in frame-gtk.c */
@@ -210,6 +205,39 @@ handle_client_message (struct frame *f, GdkEvent *event)
     }
 }
 
+static void
+emacs_gtk_format_magic_event (Lisp_Event *emacs_event, Lisp_Object pstream)
+{
+  Lisp_Object console = CDFW_CONSOLE (EVENT_CHANNEL (emacs_event));
+  if (CONSOLE_GTK_P (XCONSOLE (console)))
+    write_c_string (gtk_event_name (emacs_event->event.magic.underlying_gdk_event.type),
+		    pstream);
+}
+
+static int
+emacs_gtk_compare_magic_event (Lisp_Event *e1, Lisp_Event *e2)
+{
+  if (CONSOLE_GTK_P (XCONSOLE (CDFW_CONSOLE (EVENT_CHANNEL (e1)))) &&
+      CONSOLE_GTK_P (XCONSOLE (CDFW_CONSOLE (EVENT_CHANNEL (e2)))))
+    return (!memcmp (&e1->event.magic.underlying_gdk_event,
+		     &e2->event.magic.underlying_gdk_event,
+		     sizeof (GdkEvent)));
+  if (CONSOLE_GTK_P (XCONSOLE (CDFW_CONSOLE (EVENT_CHANNEL (e1)))) ||
+      CONSOLE_GTK_P (XCONSOLE (CDFW_CONSOLE (EVENT_CHANNEL (e2)))))
+    return 0;
+  return 1;
+}
+
+static Hashcode
+emacs_gtk_hash_magic_event (Lisp_Event *e)
+{
+  Lisp_Object console = CDFW_CONSOLE (EVENT_CHANNEL (e));
+  if (CONSOLE_GTK_P (XCONSOLE (console)))
+    return memory_hash (&e->event.magic.underlying_gdk_event,
+			sizeof (GdkEvent));
+  return 0;
+}
+
 static void
 emacs_gtk_handle_magic_event (struct Lisp_Event *emacs_event)
 {
@@ -238,9 +266,8 @@ emacs_gtk_handle_magic_event (struct Lisp_Event *emacs_event)
     case GDK_ENTER_NOTIFY:
       if (event->crossing.detail != GDK_NOTIFY_INFERIOR)
 	{
-	  Lisp_Object frame;
+	  Lisp_Object frame = wrap_frame (f);
 
-	  XSETFRAME (frame, f);
 	  /* FRAME_X_MOUSE_P (f) = 1; */
 	  va_run_hook_with_args (Qmouse_enter_frame_hook, 1, frame);
 	}
@@ -249,9 +276,8 @@ emacs_gtk_handle_magic_event (struct Lisp_Event *emacs_event)
     case GDK_LEAVE_NOTIFY:
       if (event->crossing.detail != GDK_NOTIFY_INFERIOR)
 	{
-	  Lisp_Object frame;
+	  Lisp_Object frame = wrap_frame (f);
 
-	  XSETFRAME (frame, f);
 	  /* FRAME_X_MOUSE_P (f) = 0; */
 	  va_run_hook_with_args (Qmouse_leave_frame_hook, 1, frame);
 	}
@@ -417,8 +443,7 @@ gtk_to_emacs_keysym (struct device *d, GdkEventKey *event, int simple_p)
      /* simple_p means don't try too hard (ASCII only) */
 {
   if (event->length != 1)
-  {
-#ifdef FILE_CODING
+    {
       /* Generate multiple emacs events */
       Emchar ch;
       Lisp_Object instream, fb_instream;
@@ -426,18 +451,19 @@ gtk_to_emacs_keysym (struct device *d, GdkEventKey *event, int simple_p)
       struct gcpro gcpro1, gcpro2;
 
       fb_instream =
-          make_fixed_buffer_input_stream ((unsigned char *) event->string, event->length);
+	make_fixed_buffer_input_stream ((unsigned char *) event->string, event->length);
 
-      /* ### Use Fget_coding_system (Vcomposed_input_coding_system) */
+      /* #### Use get_coding_system_for_text_file
+         (Vcomposed_input_coding_system, 0) */
       instream =
-	  make_decoding_input_stream (XLSTREAM (fb_instream),
-				      Fget_coding_system (Qundecided));
+	make_coding_input_stream (XLSTREAM (fb_instream),
+				  Qundecided, CODING_DECODE, 0);
       
       istr = XLSTREAM (instream);
 
       GCPRO2 (instream, fb_instream);
       while ((ch = Lstream_get_emchar (istr)) != EOF)
-      {
+	{
 	  Lisp_Object emacs_event = Fmake_event (Qnil, Qnil);
 	  struct Lisp_Event *ev = XEVENT (emacs_event);
 	  ev->channel       = DEVICE_CONSOLE (d);
@@ -446,35 +472,21 @@ gtk_to_emacs_keysym (struct device *d, GdkEventKey *event, int simple_p)
 	  ev->event.key.modifiers = 0;
 	  ev->event.key.keysym    = make_char (ch);
 	  enqueue_gtk_dispatch_event (emacs_event);
-      }
+	}
       Lstream_close (istr);
       UNGCPRO;
       Lstream_delete (istr);
       Lstream_delete (XLSTREAM (fb_instream));
-#else
-      int i;
-      for (i = 0; i < event->length; i++)
-      {
-	  Lisp_Object emacs_event = Fmake_event (Qnil, Qnil);
-	  struct Lisp_Event *ev = XEVENT (emacs_event);
-	  ev->channel       = DEVICE_CONSOLE (d);
-	  ev->event_type    = key_press_event;
-	  ev->timestamp	    = event->time;
-	  ev->event.key.modifiers = 0;
-	  ev->event.key.keysym    = make_char (event->string[i]);
-	  enqueue_gtk_dispatch_event (emacs_event);
-      }
-#endif
       if (IS_MODIFIER_KEY (event->keyval) || (event->keyval == GDK_Mode_switch))
-	  return (Qnil);
+	return (Qnil);
       return (gtk_keysym_to_emacs_keysym (event->keyval, simple_p));
-  }
+    }
   else
-  {
+    {
       if (IS_MODIFIER_KEY (event->keyval) || (event->keyval == GDK_Mode_switch))
-	  return (Qnil);
+	return (Qnil);
       return (gtk_keysym_to_emacs_keysym (event->keyval, simple_p));
-  }
+    }
 }
 
 
@@ -771,7 +783,7 @@ emacs_gtk_select_process (struct Lisp_Process *p)
   Lisp_Object process;
   int infd = event_stream_unixoid_select_process (p);
 
-  XSETPROCESS (process, p);
+  process = wrap_process (p);
   select_filedesc (infd, process);
 }
 
@@ -855,7 +867,7 @@ emacs_gtk_select_console (struct console *con)
   if (CONSOLE_GTK_P (con))
     return; /* Gtk consoles are automatically selected for when we initialize them */
   infd = event_stream_unixoid_select_console (con);
-  XSETCONSOLE (console, con);
+  console = wrap_console (con);
   select_filedesc (infd, console);
 }
 
@@ -868,7 +880,7 @@ emacs_gtk_unselect_console (struct console *con)
   if (CONSOLE_GTK_P (con))
 	return; /* X consoles are automatically selected for when we initialize them */
   infd = event_stream_unixoid_unselect_console (con);
-  XSETCONSOLE (console, con);
+  console = wrap_console (con);
   unselect_filedesc (infd);
 }
 
@@ -894,8 +906,8 @@ gtk_tty_to_emacs_event (struct Lisp_Event *emacs_event)
 	  assert (tty_events_occurred > 0);
 	  tty_events_occurred--;
 	  filedesc_with_input[i] = Qnil;
-	  if (read_event_from_tty_or_stream_desc
-	      (emacs_event, XCONSOLE (console), i))
+	  if (read_event_from_tty_or_stream_desc (emacs_event,
+						  XCONSOLE (console)))
 	    return 1;
 	}
     }
@@ -937,7 +949,7 @@ dragndrop_data_received (GtkWidget          *widget,
   ev->event_type = misc_user_event;
   ev->timestamp = time;
 
-  XSETFRAME (ev->channel, f);
+  ev->channel = wrap_frame (f);
 
   ev->event.misc.x = x;
   ev->event.misc.y = y;
@@ -1142,7 +1154,7 @@ emacs_gtk_next_event (struct Lisp_Event *emacs_event)
   if (!NILP (dispatch_event_queue))
     {
       Lisp_Object event, event2;
-      XSETEVENT (event2, emacs_event);
+      event2 = wrap_event (emacs_event);
       event = dequeue_gtk_dispatch_event ();
       Fcopy_event (event, event2);
       Fdeallocate_event (event);
@@ -1348,7 +1360,7 @@ gtk_event_to_emacs_event (struct frame *frame, GdkEvent *gdk_event, struct Lisp_
 	else                    /* Mouse press/release event */
 	  {
 	    GdkEventButton *button_event = &gdk_event->button;
-	    XSETFRAME (emacs_event->channel, frame);
+	    emacs_event->channel = wrap_frame (frame);
 
 	    emacs_event->event_type = (button_event->type == GDK_BUTTON_RELEASE) ?
 	      button_release_event : button_press_event;
@@ -1383,7 +1395,7 @@ gtk_event_to_emacs_event (struct frame *frame, GdkEvent *gdk_event, struct Lisp_
 
         DEVICE_GTK_MOUSE_TIMESTAMP (d) = ev->time;
 
-        XSETFRAME (emacs_event->channel, frame);
+        emacs_event->channel = wrap_frame (frame);
         emacs_event->event_type	    = pointer_motion_event;
         emacs_event->timestamp      = ev->time;
         emacs_event->event.motion.x = x;
@@ -1493,19 +1505,19 @@ emacs_shell_event_handler (GtkWidget *wid /* unused */,
 #undef FROB
 
     emacs_event->event_type = magic_event;
-    XSETFRAME (emacs_event->channel, frame);
+    emacs_event->channel = wrap_frame (frame);
 
     if (ignore_p)
-    {
+      {
 	stderr_out ("Ignoring event... (%s)\n", event_name (event));
 	Fdeallocate_event (lisp_event);
 	return (FALSE);
-    }
+      }
     else
-    {
+      {
 	enqueue_gtk_dispatch_event (lisp_event);
 	return (TRUE);
-    }
+      }
 }
 
 
@@ -1535,7 +1547,7 @@ check_for_tty_quit_char (struct device *d)
 	return;
 
       event = Fmake_event (Qnil, Qnil);
-      if (!read_event_from_tty_or_stream_desc (XEVENT (event), con, infd))
+      if (!read_event_from_tty_or_stream_desc (XEVENT (event), con))
 	/* EOF, or something ... */
 	return;
       /* #### bogus.  quit-char should be allowed to be any sort
@@ -1711,6 +1723,14 @@ emacs_gtk_event_pending_p (int user_p)
   return 0;
 }
 
+static void
+emacs_gtk_force_event_pending (struct frame* f)
+{
+#if 0
+  stderr_out ("Force event pending called on frame %p!\n", f);
+#endif
+}
+
 
 /************************************************************************/
 /*                            initialization                            */
@@ -1729,6 +1749,9 @@ void reinit_vars_of_event_gtk (void)
   gtk_event_stream->event_pending_p 	= emacs_gtk_event_pending_p;
   gtk_event_stream->next_event_cb	= emacs_gtk_next_event;
   gtk_event_stream->handle_magic_event_cb= emacs_gtk_handle_magic_event;
+  gtk_event_stream->format_magic_event_cb= emacs_gtk_format_magic_event;
+  gtk_event_stream->compare_magic_event_cb= emacs_gtk_compare_magic_event;
+  gtk_event_stream->hash_magic_event_cb  = emacs_gtk_hash_magic_event;
   gtk_event_stream->add_timeout_cb 	= emacs_gtk_add_timeout;
   gtk_event_stream->remove_timeout_cb 	= emacs_gtk_remove_timeout;
   gtk_event_stream->select_console_cb 	= emacs_gtk_select_console;
@@ -1738,6 +1761,7 @@ void reinit_vars_of_event_gtk (void)
   gtk_event_stream->quit_p_cb		= emacs_gtk_quit_p;
   gtk_event_stream->create_stream_pair_cb= emacs_gtk_create_stream_pair;
   gtk_event_stream->delete_stream_pair_cb= emacs_gtk_delete_stream_pair;
+  gtk_event_stream->force_event_pending	 = emacs_gtk_force_event_pending;
 
   the_GTK_timeout_blocktype = Blocktype_new (struct GTK_timeout_blocktype);
 
@@ -1884,11 +1908,12 @@ gtk_reset_key_mapping (struct device *d)
 	continue;
 
       {
-	char *name = XKeysymToString (keysym[0]);
+	Extbyte *name = XKeysymToString (keysym[0]);
 	Lisp_Object sym = gtk_keysym_to_emacs_keysym (keysym[0], 0);
 	if (name)
 	  {
-	    Fputhash (build_string (name), Qsans_modifiers, hashtable);
+	    Fputhash (build_ext_string (name, Qnative), Qsans_modifiers,
+		      hashtable);
 	    Fputhash (sym, Qsans_modifiers, hashtable);
 	  }
       }
@@ -1898,11 +1923,11 @@ gtk_reset_key_mapping (struct device *d)
 	  if (keysym[j] != keysym[0] &&
 	      keysym[j] != NoSymbol)
 	    {
-	      char *name = XKeysymToString (keysym[j]);
+	      Extbyte *name = XKeysymToString (keysym[j]);
 	      Lisp_Object sym = gtk_keysym_to_emacs_keysym (keysym[j], 0);
 	      if (name && NILP (Fgethash (sym, hashtable, Qnil)))
 		{
-		  Fputhash (build_string (name), Qt, hashtable);
+		  Fputhash (build_ext_string (name, Qnative), Qt, hashtable);
 		  Fputhash (sym, Qt, hashtable);
 		}
 	    }
