@@ -1,6 +1,7 @@
 /* Events: printing them, converting them to and from characters.
    Copyright (C) 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
+   Copyright (C) 2001, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -27,18 +28,19 @@ Boston, MA 02111-1307, USA.  */
 #include "lisp.h"
 #include "buffer.h"
 #include "console.h"
-#include "console-tty.h" /* for stuff in character_to_event */
 #include "device.h"
-#include "console-x.h"	/* for x_event_name prototype */
-#include "console-gtk.h" /* for gtk_event_name prototype */
-#include "extents.h"	/* Just for the EXTENTP abort check... */
+#include "extents.h"
 #include "events.h"
 #include "frame.h"
 #include "glyphs.h"
 #include "keymap.h" /* for key_desc_list_to_event() */
+#include "lstream.h"
 #include "redisplay.h"
+#include "toolbar.h"
 #include "window.h"
 
+#include "console-tty.h" /* for stuff in character_to_event */
+
 /* Where old events go when they are explicitly deallocated.
    The event chain here is cut loose before GC, so these will be freed
    eventually.
@@ -55,8 +57,10 @@ Lisp_Object Qprocess_event_p;
 Lisp_Object Qkey_press, Qbutton_press, Qbutton_release, Qmisc_user;
 Lisp_Object Qascii_character;
 
-EXFUN (Fevent_x_pixel, 1);
-EXFUN (Fevent_y_pixel, 1);
+
+/************************************************************************/
+/*                       definition of event object                     */
+/************************************************************************/
 
 /* #### Ad-hoc hack.  Should be part of define_lrecord_implementation */
 void
@@ -133,10 +137,11 @@ mark_event (Lisp_Object obj)
 static void
 print_event_1 (const char *str, Lisp_Object obj, Lisp_Object printcharfun)
 {
-  char buf[255];
+  DECLARE_EISTRING_MALLOC (ei);
   write_c_string (str, printcharfun);
-  format_event_object (buf, XEVENT (obj), 0);
-  write_c_string (buf, printcharfun);
+  format_event_object (ei, XEVENT (obj), 0);
+  write_eistring (ei, printcharfun);
+  eifree (ei);
 }
 
 static void
@@ -162,40 +167,31 @@ print_event (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
       break;
     case pointer_motion_event:
       {
-	char buf[64];
 	Lisp_Object Vx, Vy;
 	Vx = Fevent_x_pixel (obj);
 	assert (INTP (Vx));
 	Vy = Fevent_y_pixel (obj);
 	assert (INTP (Vy));
-	sprintf (buf, "#<motion-event %ld, %ld", (long) XINT (Vx), (long) XINT (Vy));
-	write_c_string (buf, printcharfun);
+	write_fmt_string (printcharfun, "#<motion-event %ld, %ld",
+			  (long) XINT (Vx), (long) XINT (Vy));
 	break;
       }
     case process_event:
-	write_c_string ("#<process-event ", printcharfun);
-	print_internal (XEVENT (obj)->event.process.process, printcharfun, 1);
+	write_fmt_string_lisp (printcharfun, "#<process-event %S", 1, XEVENT (obj)->event.process.process);
 	break;
     case timeout_event:
-	write_c_string ("#<timeout-event ", printcharfun);
-	print_internal (XEVENT (obj)->event.timeout.object, printcharfun, 1);
+	write_fmt_string_lisp (printcharfun, "#<timeout-event %S", 1, XEVENT (obj)->event.timeout.object);
 	break;
     case empty_event:
 	write_c_string ("#<empty-event", printcharfun);
 	break;
     case misc_user_event:
-	write_c_string ("#<misc-user-event (", printcharfun);
-	print_internal (XEVENT (obj)->event.misc.function, printcharfun, 1);
-	write_c_string (" ", printcharfun);
-	print_internal (XEVENT (obj)->event.misc.object, printcharfun, 1);
-	write_c_string (")", printcharfun);
+	write_fmt_string_lisp (printcharfun, "#<misc-user-event (%S", 1, XEVENT (obj)->event.misc.function);
+	write_fmt_string_lisp (printcharfun, " %S)", 1, XEVENT (obj)->event.misc.object);
 	break;
     case eval_event:
-	write_c_string ("#<eval-event (", printcharfun);
-	print_internal (XEVENT (obj)->event.eval.function, printcharfun, 1);
-	write_c_string (" ", printcharfun);
-	print_internal (XEVENT (obj)->event.eval.object, printcharfun, 1);
-	write_c_string (")", printcharfun);
+	write_fmt_string_lisp (printcharfun, "#<eval-event (%S", 1, XEVENT (obj)->event.eval.function);
+	write_fmt_string_lisp (printcharfun, " %S)", 1, XEVENT (obj)->event.eval.object);
 	break;
     case dead_event:
 	write_c_string ("#<DEALLOCATED-EVENT", printcharfun);
@@ -265,34 +261,7 @@ event_equal (Lisp_Object obj1, Lisp_Object obj2, int depth)
 			      e2->event.magic_eval.object, 0));
 
     case magic_event:
-      {
-	struct console *con = XCONSOLE (CDFW_CONSOLE (e1->channel));
-
-#ifdef HAVE_GTK
- 	if (CONSOLE_GTK_P (con))
-	  return (!memcmp (&e1->event.magic.underlying_gdk_event,
-			   &e2->event.magic.underlying_gdk_event,
-			   sizeof (GdkEvent)));
-#endif
-#ifdef HAVE_X_WINDOWS
-	if (CONSOLE_X_P (con))
-	  return (e1->event.magic.underlying_x_event.xany.serial ==
-		  e2->event.magic.underlying_x_event.xany.serial);
-#endif
-#ifdef HAVE_TTY
-	if (CONSOLE_TTY_P (con))
-	  return (e1->event.magic.underlying_tty_event ==
-		  e2->event.magic.underlying_tty_event);
-#endif
-#ifdef HAVE_MS_WINDOWS
-	if (CONSOLE_MSWINDOWS_P (con))
-	  return (!memcmp(&e1->event.magic.underlying_mswindows_event,
-			  &e2->event.magic.underlying_mswindows_event,
-			  sizeof (union magic_data)));
-#endif
-	abort ();
-	return 1; /* not reached */
-      }
+      return event_stream_compare_magic_event (e1, e2);
 
     case empty_event:      /* Empty and deallocated events are equal. */
     case dead_event:
@@ -342,27 +311,7 @@ event_hash (Lisp_Object obj, int depth)
 		    internal_hash (e->event.magic_eval.object, depth + 1));
 
     case magic_event:
-      {
-	struct console *con = XCONSOLE (CDFW_CONSOLE (EVENT_CHANNEL (e)));
-#ifdef HAVE_GTK
- 	if (CONSOLE_GTK_P (con))
- 	  return HASH2 (hash, e->event.magic.underlying_gdk_event.type);
-#endif
-#ifdef HAVE_X_WINDOWS
-	if (CONSOLE_X_P (con))
-	  return HASH2 (hash, e->event.magic.underlying_x_event.xany.serial);
-#endif
-#ifdef HAVE_TTY
-	if (CONSOLE_TTY_P (con))
-	  return HASH2 (hash, e->event.magic.underlying_tty_event);
-#endif
-#ifdef HAVE_MS_WINDOWS
-	if (CONSOLE_MSWINDOWS_P (con))
-	  return HASH2 (hash, e->event.magic.underlying_mswindows_event);
-#endif
-	abort ();
-	return 0;
-      }
+      return HASH2 (hash, event_stream_hash_magic_event (e));
 
     case empty_event:
     case dead_event:
@@ -379,7 +328,6 @@ DEFINE_BASIC_LRECORD_IMPLEMENTATION ("event", event,
 				     mark_event, print_event, 0, event_equal,
 				     event_hash, 0, Lisp_Event);
 
-
 DEFUN ("make-event", Fmake_event, 0, 2, 0, /*
 Return a new event of type TYPE, with properties described by PLIST.
 
@@ -809,6 +757,9 @@ function `deallocate-event'.
 }
 
 
+/************************************************************************/
+/*                          event chain functions                       */
+/************************************************************************/
 
 /* Given a chain of events (or possibly nil), deallocate them all. */
 
@@ -945,6 +896,8 @@ event_chain_nth (Lisp_Object event_chain, int n)
   return Qnil;
 }
 
+/* Return a freshly allocated copy of all events in the given chain. */
+
 Lisp_Object
 copy_event_chain (Lisp_Object event_chain)
 {
@@ -961,7 +914,30 @@ copy_event_chain (Lisp_Object event_chain)
   return new_chain;
 }
 
+/* Given a pointer (maybe nil) into an old chain (also maybe nil, if
+   pointer is nil) and a new chain which is a copy of the old, return
+   the corresponding new pointer. */
+Lisp_Object
+transfer_event_chain_pointer (Lisp_Object pointer, Lisp_Object old_chain,
+			      Lisp_Object new_chain)
+{
+  if (NILP (pointer))
+    return Qnil;
+  assert (!NILP (old_chain));
+#ifdef ERROR_CHECK_STRUCTURES
+  /* make sure we're actually in the chain */
+  event_chain_find_previous (old_chain, pointer);
+  assert (event_chain_count (old_chain) == event_chain_count (new_chain));
+#endif /* ERROR_CHECK_STRUCTURES */
+  return event_chain_nth (new_chain,
+			  event_chain_count (old_chain) -
+			  event_chain_count (pointer));
+}
+
 
+/************************************************************************/
+/*                         higher level functions                       */
+/************************************************************************/
 
 Lisp_Object QKbackspace, QKtab, QKlinefeed, QKreturn, QKescape,
  QKspace, QKdelete;
@@ -1047,7 +1023,7 @@ character_to_event (Emchar c, Lisp_Event *event, struct console *con,
 
   event->event_type	     = key_press_event;
   event->timestamp	     = 0; /* #### */
-  event->channel	     = make_console (con);
+  event->channel	     = wrap_console (con);
   event->event.key.keysym    = (!NILP (k) ? k : make_char (c));
   event->event.key.modifiers = m;
 }
@@ -1211,7 +1187,7 @@ nth_of_key_sequence_as_event (Lisp_Object seq, int n, Lisp_Object event)
 
   if (STRINGP (seq))
     {
-      Emchar ch = string_char (XSTRING (seq), n);
+      Emchar ch = XSTRING_CHAR (seq, n);
       Fcharacter_to_event (make_char (ch), event, Qnil, Qnil);
     }
   else
@@ -1241,8 +1217,11 @@ key_sequence_to_event_chain (Lisp_Object seq)
   return head;
 }
 
+/* Concatenate a string description of EVENT onto the end of BUF.  If
+   BRIEF, use short forms for keys, e.g. C- instead of control-. */
+
 void
-format_event_object (char *buf, Lisp_Event *event, int brief)
+format_event_object (Eistring *buf, Lisp_Event *event, int brief)
 {
   int mouse_p = 0;
   int mod = 0;
@@ -1256,7 +1235,8 @@ format_event_object (char *buf, Lisp_Event *event, int brief)
         key = event->event.key.keysym;
         /* Hack. */
         if (! brief && CHARP (key) &&
-            mod & (XEMACS_MOD_CONTROL | XEMACS_MOD_META | XEMACS_MOD_SUPER | XEMACS_MOD_HYPER))
+            mod & (XEMACS_MOD_CONTROL | XEMACS_MOD_META | XEMACS_MOD_SUPER |
+		   XEMACS_MOD_HYPER))
 	{
 	  int k = XCHAR (key);
 	  if (k >= 'a' && k <= 'z')
@@ -1278,40 +1258,33 @@ format_event_object (char *buf, Lisp_Event *event, int brief)
       }
     case magic_event:
       {
-        const char *name = NULL;
+	Lisp_Object stream;
+	struct gcpro gcpro1;
+	GCPRO1 (stream);
 
-#ifdef HAVE_GTK
- 	{
- 	  Lisp_Object console = CDFW_CONSOLE (EVENT_CHANNEL (event));
- 	  if (CONSOLE_GTK_P (XCONSOLE (console)))
- 	    name = gtk_event_name (event->event.magic.underlying_gdk_event.type);
- 	}
-#endif
-#ifdef HAVE_X_WINDOWS
-	{
-	  Lisp_Object console = CDFW_CONSOLE (EVENT_CHANNEL (event));
-	  if (CONSOLE_X_P (XCONSOLE (console)))
-	    name = x_event_name (event->event.magic.underlying_x_event.type);
-	}
-#endif /* HAVE_X_WINDOWS */
-	if (name) strcpy (buf, name);
-	else strcpy (buf, "???");
+	stream = make_resizing_buffer_output_stream ();
+	event_stream_format_magic_event (event, stream);
+	Lstream_flush (XLSTREAM (stream));
+	eicat_raw (buf, resizing_buffer_stream_ptr (XLSTREAM (stream)),
+		   Lstream_byte_count (XLSTREAM (stream)));
+	Lstream_delete (XLSTREAM (stream));
+	UNGCPRO;
 	return;
       }
-    case magic_eval_event:	strcpy (buf, "magic-eval"); return;
-    case pointer_motion_event:	strcpy (buf, "motion");	    return;
-    case misc_user_event:	strcpy (buf, "misc-user");  return;
-    case eval_event:		strcpy (buf, "eval");	    return;
-    case process_event:		strcpy (buf, "process");    return;
-    case timeout_event:		strcpy (buf, "timeout");    return;
-    case empty_event:		strcpy (buf, "empty");	    return;
-    case dead_event:		strcpy (buf, "DEAD-EVENT"); return;
+    case magic_eval_event:	eicat_c (buf, "magic-eval"); return;
+    case pointer_motion_event:	eicat_c (buf, "motion");     return;
+    case misc_user_event:	eicat_c (buf, "misc-user");  return;
+    case eval_event:		eicat_c (buf, "eval");	     return;
+    case process_event:		eicat_c (buf, "process");    return;
+    case timeout_event:		eicat_c (buf, "timeout");    return;
+    case empty_event:		eicat_c (buf, "empty");	     return;
+    case dead_event:		eicat_c (buf, "DEAD-EVENT"); return;
     default:
       abort ();
       return;
     }
-#define modprint1(x)  do { strcpy (buf, (x)); buf += sizeof (x)-1; } while (0)
-#define modprint(x,y) do { if (brief) modprint1 (y); else modprint1 (x); } while (0)
+#define modprint(x,y) \
+  do { if (brief) eicat_c (buf, (y)); else eicat_c (buf, (x)); } while (0)
   if (mod & XEMACS_MOD_CONTROL) modprint ("control-", "C-");
   if (mod & XEMACS_MOD_META)    modprint ("meta-",    "M-");
   if (mod & XEMACS_MOD_SUPER)   modprint ("super-",   "S-");
@@ -1320,21 +1293,17 @@ format_event_object (char *buf, Lisp_Event *event, int brief)
   if (mod & XEMACS_MOD_SHIFT)   modprint ("shift-",   "Sh-");
   if (mouse_p)
     {
-      modprint1 ("button");
+      eicat_c (buf, "button");
       --mouse_p;
     }
 
 #undef modprint
-#undef modprint1
 
   if (CHARP (key))
-    {
-      buf += set_charptr_emchar ((Intbyte *) buf, XCHAR (key));
-      *buf = 0;
-    }
+    eicat_ch (buf, XCHAR (key));
   else if (SYMBOLP (key))
     {
-      const char *str = 0;
+      const Char_ASCII *str = 0;
       if (brief)
 	{
 	  if      (EQ (key, QKlinefeed))  str = "LFD";
@@ -1346,22 +1315,14 @@ format_event_object (char *buf, Lisp_Event *event, int brief)
 	  else if (EQ (key, QKbackspace)) str = "BS";
 	}
       if (str)
-	{
-	  int i = strlen (str);
-	  memcpy (buf, str, i+1);
-	  str += i;
-	}
+	eicat_c (buf, str);
       else
-	{
-	  Lisp_String *name = XSYMBOL (key)->name;
-	  memcpy (buf, string_data (name), string_length (name) + 1);
-	  str += string_length (name);
-	}
+	eicat_lstr (buf, XSYMBOL (key)->name);
     }
   else
     abort ();
   if (mouse_p)
-    strncpy (buf, "up", 4);
+    eicat_c (buf, "up");
 }
 
 DEFUN ("eventp", Feventp, 1, 1, 0, /*
@@ -1973,10 +1934,7 @@ The modeline is considered to be within the window it describes.
     return Qnil;
   else
     {
-      Lisp_Object window;
-
-      XSETWINDOW (window, w);
-      return window;
+      return wrap_window (w);
     }
 }
 
