@@ -1,7 +1,7 @@
 /* "Face" primitives
    Copyright (C) 1994 Free Software Foundation, Inc.
    Copyright (C) 1995 Board of Trustees, University of Illinois.
-   Copyright (C) 1995, 1996 Ben Wing.
+   Copyright (C) 1995, 1996, 2001, 2002 Ben Wing.
    Copyright (C) 1995 Sun Microsystems, Inc.
 
 This file is part of XEmacs.
@@ -103,19 +103,13 @@ print_face (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 
   if (print_readably)
     {
-      write_c_string ("#s(face name ", printcharfun);
-      print_internal (face->name, printcharfun, 1);
-      write_c_string (")", printcharfun);
+      write_fmt_string_lisp (printcharfun, "#s(face name %S)", 1, face->name);
     }
   else
     {
-      write_c_string ("#<face ", printcharfun);
-      print_internal (face->name, printcharfun, 1);
+      write_fmt_string_lisp (printcharfun, "#<face %S", 1, face->name);
       if (!NILP (face->doc_string))
-	{
-	  write_c_string (" ", printcharfun);
-	  print_internal (face->doc_string, printcharfun, 1);
-	}
+	write_fmt_string_lisp (printcharfun, " %S", 1, face->doc_string);
       write_c_string (">", printcharfun);
     }
 }
@@ -558,9 +552,10 @@ face_property_matching_instance (Lisp_Object face, Lisp_Object property,
 				 Error_Behavior errb, int no_fallback,
 				 Lisp_Object depth)
 {
-  Lisp_Object retval =
-    specifier_instance_no_quit (Fget (face, property, Qnil), charset,
-				domain, errb, no_fallback, depth);
+  Lisp_Object retval;
+
+  retval = specifier_instance_no_quit (Fget (face, property, Qnil), charset,
+				       domain, errb, no_fallback, depth);
 
   if (UNBOUNDP (retval) && !no_fallback)
     {
@@ -570,19 +565,19 @@ face_property_matching_instance (Lisp_Object face, Lisp_Object property,
 				  XFACE (face)->charsets_warned_about)))
 	    {
 #ifdef MULE
-	      if (! UNBOUNDP (charset))
+	      if (!UNBOUNDP (charset))
 		warn_when_safe
-		  (Qfont, Qwarning,
-		   "Unable to instantiate font for face %s, charset %s",
-		   string_data (symbol_name
-				(XSYMBOL (XFACE (face)->name))),
-		   string_data (symbol_name
-				(XSYMBOL (XCHARSET_NAME (charset)))));
+		  (Qfont, Qnotice,
+		   "Unable to instantiate font for charset %s, face %s",
+		   XSTRING_DATA (symbol_name
+				(XSYMBOL (XCHARSET_NAME (charset)))),
+		   XSTRING_DATA (symbol_name
+				(XSYMBOL (XFACE (face)->name))));
 	      else
 #endif
-		warn_when_safe (Qfont, Qwarning,
+		warn_when_safe (Qfont, Qnotice,
 				"Unable to instantiate font for face %s",
-				string_data (symbol_name
+				XSTRING_DATA (symbol_name
 					     (XSYMBOL (XFACE (face)->name))));
 	      XFACE (face)->charsets_warned_about =
 		Fcons (charset, XFACE (face)->charsets_warned_about);
@@ -782,7 +777,7 @@ If TEMPORARY is non-nil, this face will cease to exist if not in use.
     return face;
 
   f = allocate_face ();
-  XSETFACE (face, f);
+  face = wrap_face (f);
 
   f->name = name;
   f->doc_string = doc_string;
@@ -885,8 +880,8 @@ init_device_faces (struct device *d)
      loaded, so we can't do this. */
   if (initialized)
     {
-      Lisp_Object tdevice;
-      XSETDEVICE (tdevice, d);
+      Lisp_Object tdevice = wrap_device (d);
+
       call_critical_lisp_code (d, Qinit_device_faces, tdevice);
     }
 }
@@ -898,8 +893,8 @@ init_frame_faces (struct frame *frm)
      loaded, so we can't do this. */
   if (initialized)
     {
-      Lisp_Object tframe;
-      XSETFRAME (tframe, frm);
+      Lisp_Object tframe = wrap_frame (frm);
+
 
       /* DO NOT change the selected frame here.  If the debugger goes off
          it will try and display on the frame being created, but it is not
@@ -1083,16 +1078,17 @@ ensure_face_cachel_contains_charset (struct face_cachel *cachel,
     }
 
   new_val = face_property_matching_instance (face, Qfont, charset, domain,
-					     /* #### look into ERROR_ME_NOT */
-					     ERROR_ME_NOT, 1, Qzero);
+					     /* #### look into error flag */
+					     ERROR_ME_DEBUG_WARN, 1, Qzero);
   if (UNBOUNDP (new_val))
     {
       bound = 0;
       new_val = face_property_matching_instance (face, Qfont,
 						 charset, domain,
-						 /* #### look into
-						    ERROR_ME_NOT */
-						 ERROR_ME_NOT, 0, Qzero);
+						 /* #### look into error
+                                                    flag */
+						 ERROR_ME_DEBUG_WARN, 0,
+						 Qzero);
     }
   if (!UNBOUNDP (cachel->font[offs]) && !EQ (new_val, cachel->font[offs]))
     cachel->dirty = 1;
@@ -1206,7 +1202,7 @@ add_face_cachel (struct window *w, Lisp_Object face)
   Lisp_Object domain;
 
   reset_face_cachel (&new_cachel);
-  XSETWINDOW (domain, w);
+  domain = wrap_window (w);
   update_face_cachel_data (&new_cachel, domain, face);
   Dynarr_add (w->face_cachels, new_cachel);
 
@@ -1413,8 +1409,8 @@ get_builtin_face_cache_index (struct window *w, Lisp_Object face)
 
       if (EQ (cachel->face, face))
 	{
-	  Lisp_Object window;
-	  XSETWINDOW (window, w);
+	  Lisp_Object window = wrap_window (w);
+
 	  if (!cachel->updated)
 	    update_face_cachel_data (cachel, window, face);
 	  return elt;
@@ -1570,8 +1566,6 @@ get_extent_fragment_face_cache_index (struct window *w,
   struct face_cachel cachel;
   int len = Dynarr_length (ef->extents);
   face_index findex = 0;
-  Lisp_Object window;
-  XSETWINDOW (window, w);
 
   /* Optimize the default case. */
   if (len == 0)
@@ -1694,9 +1688,8 @@ update_EmacsFrames (Lisp_Object locale, Lisp_Object name)
 void
 update_frame_face_values (struct frame *f)
 {
-  Lisp_Object frm;
+  Lisp_Object frm = wrap_frame (f);
 
-  XSETFRAME (frm, f);
   update_EmacsFrame (frm, Qforeground);
   update_EmacsFrame (frm, Qbackground);
   update_EmacsFrame (frm, Qfont);
@@ -1874,9 +1867,11 @@ void
 vars_of_faces (void)
 {
   staticpro (&Vpermanent_faces_cache);
-  Vpermanent_faces_cache = Qnil;
+  Vpermanent_faces_cache =
+    make_lisp_hash_table (10, HASH_TABLE_NON_WEAK, HASH_TABLE_EQ);
   staticpro (&Vtemporary_faces_cache);
-  Vtemporary_faces_cache = Qnil;
+  Vtemporary_faces_cache =
+    make_lisp_hash_table (0, HASH_TABLE_WEAK, HASH_TABLE_EQ);
 
   staticpro (&Vdefault_face);
   Vdefault_face = Qnil;
@@ -1924,16 +1919,11 @@ vars_of_faces (void)
 void
 complex_vars_of_faces (void)
 {
-  Vpermanent_faces_cache =
-    make_lisp_hash_table (10, HASH_TABLE_NON_WEAK, HASH_TABLE_EQ);
-  Vtemporary_faces_cache =
-    make_lisp_hash_table (0, HASH_TABLE_WEAK, HASH_TABLE_EQ);
-
   /* Create the default face now so we know what it is immediately. */
 
   Vdefault_face = Qnil; /* so that Fmake_face() doesn't set up a bogus
 			   default value */
-  Vdefault_face = Fmake_face (Qdefault, build_string ("default face"),
+  Vdefault_face = Fmake_face (Qdefault, build_msg_string ("default face"),
 			      Qnil);
 
   /* Provide some last-resort fallbacks to avoid utter fuckage if
@@ -1981,7 +1971,7 @@ complex_vars_of_faces (void)
        (#### Perhaps we should remove the stuff from x-faces.el
        and only depend on this stuff here?  That should work.)
      */
-    const char *fonts[] =
+    const Char_ASCII *fonts[] =
     {
       "-*-courier-medium-r-*-*-*-120-*-*-*-*-iso8859-*",
       "-*-courier-medium-r-*-*-*-120-*-*-*-*-iso8859-*",
@@ -1999,7 +1989,7 @@ complex_vars_of_faces (void)
       "-*-*-*-*-*-*-*-120-*-*-*-*-*-*",
       "*"
     };
-    const char **fontptr;
+    const Char_ASCII **fontptr;
 
 #ifdef HAVE_X_WINDOWS
     for (fontptr = fonts + countof(fonts) - 1; fontptr >= fonts; fontptr--)
@@ -2014,25 +2004,44 @@ complex_vars_of_faces (void)
 #endif /* HAVE_GTK */
 #endif /* HAVE_X_WINDOWS || HAVE_GTK */
 
-
 #ifdef HAVE_TTY
     inst_list = Fcons (Fcons (list1 (Qtty), build_string ("normal")),
 		       inst_list);
 #endif /* HAVE_TTY */
-#ifdef HAVE_MS_WINDOWS
-    /* Fixedsys does not exist for printers */
-    inst_list = Fcons (Fcons (list1 (Qmsprinter),
-			      build_string ("Courier:Regular:10::Western")), inst_list);
-    inst_list = Fcons (Fcons (list1 (Qmsprinter),
-			      build_string ("Courier New:Regular:10::Western")), inst_list);
 
-    inst_list = Fcons (Fcons (list1 (Qmswindows),
-			      build_string ("Fixedsys:Regular:9::Western")), inst_list);
-    inst_list = Fcons (Fcons (list1 (Qmswindows),
-			      build_string ("Courier:Regular:10::Western")), inst_list);
-    inst_list = Fcons (Fcons (list1 (Qmswindows),
-			      build_string ("Courier New:Regular:10::Western")), inst_list);
+#ifdef HAVE_MS_WINDOWS
+    {
+       const Char_ASCII *mswfonts[] =
+ 	    {
+ 	      "Courier New:Regular:10::Western",
+ 	      "Courier:Regular:10::Western",
+ 	      "Courier New:Regular:10::",
+ 	      "Courier:Regular:10::",
+ 	      ":Regular:10::"
+ 	    };
+       const Char_ASCII **mswfontptr;
+ 
+       for (mswfontptr = mswfonts + countof (mswfonts) - 1;
+	    mswfontptr >= mswfonts; mswfontptr--)
+ 	{
+ 	  /* display device */
+ 	  inst_list = Fcons (Fcons (list1 (Qmswindows), 
+ 				    build_string (*mswfontptr)),
+ 			     inst_list);
+ 	  /* printer device */
+ 	  inst_list = Fcons (Fcons (list1 (Qmsprinter), 
+ 				    build_string (*mswfontptr)),
+ 			     inst_list);
+ 	}
+       /* Use Lucida Console rather than Courier New if it exists -- the
+          line spacing is much less, so many more lines fit with the same
+          size font. (And it's specifically designed for screens.) */
+       inst_list = Fcons (Fcons (list1 (Qmswindows), 
+				 build_string ("Lucida Console:Regular:10::")),
+			  inst_list);
+    }
 #endif /* HAVE_MS_WINDOWS */
+
     set_specifier_fallback (Fget (Vdefault_face, Qfont, Qnil), inst_list);
   }
 
@@ -2052,7 +2061,7 @@ complex_vars_of_faces (void)
   /* gui-element is the parent face of all gui elements such as
      modeline, vertical divider and toolbar. */
   Vgui_element_face = Fmake_face (Qgui_element,
-				  build_string ("gui element face"),
+				  build_msg_string ("gui element face"),
 				  Qnil);
 
   /* Provide some last-resort fallbacks for gui-element face which
@@ -2090,7 +2099,7 @@ complex_vars_of_faces (void)
      way since we need to get them anyway. */
 
   /* modeline is gui element. */
-  Vmodeline_face = Fmake_face (Qmodeline, build_string ("modeline face"),
+  Vmodeline_face = Fmake_face (Qmodeline, build_msg_string ("modeline face"),
 			       Qnil);
 
   set_specifier_fallback (Fget (Vmodeline_face, Qforeground, Qunbound),
@@ -2103,7 +2112,7 @@ complex_vars_of_faces (void)
 
   /* toolbar is another gui element */
   Vtoolbar_face = Fmake_face (Qtoolbar,
-			      build_string ("toolbar face"),
+			      build_msg_string ("toolbar face"),
 			      Qnil);
   set_specifier_fallback (Fget (Vtoolbar_face, Qforeground, Qunbound),
 			  Fget (Vgui_element_face, Qforeground, Qunbound));
@@ -2115,7 +2124,7 @@ complex_vars_of_faces (void)
 
   /* vertical divider is another gui element */
   Vvertical_divider_face = Fmake_face (Qvertical_divider,
-				       build_string ("vertical divider face"),
+				       build_msg_string ("vertical divider face"),
 				       Qnil);
 
   set_specifier_fallback (Fget (Vvertical_divider_face, Qforeground, Qunbound),
@@ -2129,7 +2138,7 @@ complex_vars_of_faces (void)
 
   /* widget is another gui element */
   Vwidget_face = Fmake_face (Qwidget,
-			     build_string ("widget face"),
+			     build_msg_string ("widget face"),
 			     Qnil);
   set_specifier_fallback (Fget (Vwidget_face, Qfont, Qunbound),
 			  Fget (Vgui_element_face, Qfont, Qunbound));
@@ -2140,17 +2149,17 @@ complex_vars_of_faces (void)
   /* We don't want widgets to have a default background pixmap. */
 
   Vleft_margin_face = Fmake_face (Qleft_margin,
-				  build_string ("left margin face"),
+				  build_msg_string ("left margin face"),
 				  Qnil);
   Vright_margin_face = Fmake_face (Qright_margin,
-				   build_string ("right margin face"),
+				   build_msg_string ("right margin face"),
 				   Qnil);
   Vtext_cursor_face = Fmake_face (Qtext_cursor,
-				  build_string ("face for text cursor"),
+				  build_msg_string ("face for text cursor"),
 				  Qnil);
   Vpointer_face =
     Fmake_face (Qpointer,
-		build_string
+		build_msg_string
 		("face for foreground/background colors of mouse pointer"),
 		Qnil);
 }
