@@ -175,7 +175,7 @@ struct frame *
 decode_gtk_frame (Lisp_Object frame)
 {
   if (NILP (frame))
-    XSETFRAME (frame, selected_frame ());
+    frame = wrap_frame (selected_frame ());
   CHECK_LIVE_FRAME (frame);
   /* this will also catch dead frames, but putting in the above check
      results in a more useful error */
@@ -226,7 +226,7 @@ gtk_frame_property (struct frame *f, Lisp_Object property)
     }
 #ifdef STUPID_X_SPECIFIC_GTK_STUFF
   if (EQ (Qwindow_id, property))
-    return Fgtk_window_id (make_frame (f));
+    return Fgtk_window_id (wrap_frame (f));
 #endif
 
   return Qunbound;
@@ -256,7 +256,7 @@ gtk_frame_properties (struct frame *f)
   props = cons3 (Qtext_widget, FRAME_GTK_LISP_WIDGETS (f)[2], props);
 
 #ifdef STUPID_X_SPECIFIC_GTK_STUFF
-  props = cons3 (Qwindow_id, Fgtk_window_id (make_frame (f)), props);
+  props = cons3 (Qwindow_id, Fgtk_window_id (wrap_frame (f)), props);
 #endif
 
   if (!GET_GTK_WIDGET_WINDOW (shell))
@@ -334,7 +334,7 @@ gtk_set_initial_frame_size (struct frame *f, int x, int y,
   if (GTK_IS_WINDOW (shell))
     {
       /* Deal with the cell size */
-      default_face_height_and_width (make_frame (f), &geometry.height_inc, &geometry.width_inc);
+      default_face_height_and_width (wrap_frame (f), &geometry.height_inc, &geometry.width_inc);
       geometry_mask |= GDK_HINT_RESIZE_INC;
 
       gtk_window_set_geometry_hints (GTK_WINDOW (shell),
@@ -384,7 +384,7 @@ gtk_set_frame_properties (struct frame *f, Lisp_Object plist)
 	      {
 		  Lisp_Object frm, font_spec;
 
-		  XSETFRAME (frm, f);
+		  frm = wrap_frame (f);
 		  font_spec = Fget (Fget_face (Qdefault), Qfont, Qnil);
 
 		  Fadd_spec_to_specifier (font_spec, val, frm, Qnil, Qnil);
@@ -455,14 +455,14 @@ gtk_set_frame_properties (struct frame *f, Lisp_Object plist)
       {
 	if (size_specified_p)
 	  {
-	    Lisp_Object frame;
-	    XSETFRAME (frame, f);
+	    Lisp_Object frame = wrap_frame (f);
+
 	    Fset_frame_size (frame, make_int (width), make_int (height), Qnil);
 	  }
 	if (position_specified_p)
 	  {
-	    Lisp_Object frame;
-	    XSETFRAME (frame, f);
+	    Lisp_Object frame = wrap_frame (f);
+
 	    Fset_frame_position (frame, make_int (x), make_int (y));
 	  }
       }
@@ -620,8 +620,8 @@ resize_event_cb (GtkWidget *w, GtkAllocation *allocation, gpointer user_data)
 
   if (FRAME_GTK_TEXT_WIDGET (f)->window)
     {
-      Lisp_Object frame;
-      XSETFRAME (frame, f);
+      Lisp_Object frame = wrap_frame (f);
+
       Fredraw_frame (frame, Qt);
     }
 
@@ -632,9 +632,8 @@ static gboolean
 delete_event_cb (GtkWidget *w, GdkEvent *ev, gpointer user_data)
 {
     struct frame *f = (struct frame *) user_data;
-    Lisp_Object frame;
+    Lisp_Object frame = wrap_frame (f);
 
-    XSETFRAME (frame, f);
     enqueue_misc_user_event (frame, Qeval, list3 (Qdelete_frame, frame, Qt));
 
     /* See if tickling the event queue helps us with our delays when
@@ -656,9 +655,8 @@ static void
 cleanup_deleted_frame (gpointer data)
 {
   struct frame *f = (struct frame *) data;
-  Lisp_Object frame;
+  Lisp_Object frame = wrap_frame (f);
 
-  XSETFRAME (frame, f);
   Fdelete_frame (frame, Qt);
 }
 
@@ -943,7 +941,8 @@ allocate_gtk_frame_struct (struct frame *f)
 /************************************************************************/
 
 static void
-gtk_init_frame_1 (struct frame *f, Lisp_Object props)
+gtk_init_frame_1 (struct frame *f, Lisp_Object props,
+		  int frame_name_is_defaulted)
 {
   /* This function can GC */
   Lisp_Object initially_unmapped;
@@ -1121,7 +1120,7 @@ gtk_set_frame_size (struct frame *f, int cols, int rows)
   if (GTK_IS_WINDOW (shell))
     {
       /* Update the cell size */
-      default_face_height_and_width (make_frame (f), &geometry.height_inc, &geometry.width_inc);
+      default_face_height_and_width (wrap_frame (f), &geometry.height_inc, &geometry.width_inc);
       geometry_mask |= GDK_HINT_RESIZE_INC;
 
       gtk_window_set_geometry_hints (GTK_WINDOW (shell),
@@ -1168,7 +1167,7 @@ gtk_get_mouse_position (struct device *d, Lisp_Object *frame, int *x, int *y)
 
     if (!f) return (0);
 
-    XSETFRAME (*frame, f);
+    *frame = wrap_frame (f);
 
     gdk_window_get_pointer (GET_GTK_WIDGET_WINDOW (FRAME_GTK_TEXT_WIDGET (f)),
 			    &win_x, &win_y, NULL);
@@ -1299,7 +1298,7 @@ gtk_recompute_cell_sizes (struct frame *frm)
       gint width_inc = 10;
       gint height_inc = 10;
 
-      default_face_height_and_width (make_frame (frm), &height_inc, &width_inc);
+      default_face_height_and_width (wrap_frame (frm), &height_inc, &width_inc);
       geometry_mask = GDK_HINT_RESIZE_INC;
       geometry.width_inc = width_inc;
       geometry.height_inc = height_inc;
@@ -1313,7 +1312,7 @@ gtk_update_frame_external_traits (struct frame* frm, Lisp_Object name)
 {
   Lisp_Object frame = Qnil;
 
-  XSETFRAME(frame, frm);
+  frame = wrap_frame (frm);
 
   if (EQ (name, Qforeground))
    {
