@@ -1,6 +1,6 @@
 /* Functions for the X window system.
    Copyright (C) 1989, 1992-5, 1997 Free Software Foundation, Inc.
-   Copyright (C) 1995, 1996 Ben Wing.
+   Copyright (C) 1995, 1996, 2001 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -28,7 +28,20 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
+#include "buffer.h"
+#include "device.h"
+#include "events.h"
+#include "extents.h"
+#include "faces.h"
+#include "frame.h"
+#include "window.h"
+#include "gutter.h"
+
 #include "console-x.h"
+#include "glyphs-x.h"
+#include "objects-x.h"
+#include "scrollbar-x.h"
+
 #include "xintrinsicp.h"	/* CoreP.h needs this */
 #include <X11/CoreP.h>		/* Numerous places access the fields of
 				   a core widget directly.  We could
@@ -42,17 +55,6 @@ Boston, MA 02111-1307, USA.  */
 #ifdef EXTERNAL_WIDGET
 #include "ExternalShell.h"
 #endif
-#include "glyphs-x.h"
-#include "objects-x.h"
-#include "scrollbar-x.h"
-
-#include "buffer.h"
-#include "events.h"
-#include "extents.h"
-#include "faces.h"
-#include "frame.h"
-#include "window.h"
-#include "gutter.h"
 
 #ifdef HAVE_DRAGNDROP
 #include "dragdrop.h"
@@ -188,7 +190,7 @@ struct frame *
 decode_x_frame (Lisp_Object frame)
 {
   if (NILP (frame))
-    XSETFRAME (frame, selected_frame ());
+    frame = wrap_frame (selected_frame ());
   CHECK_LIVE_FRAME (frame);
   /* this will also catch dead frames, but putting in the above check
      results in a more useful error */
@@ -360,7 +362,7 @@ x_wm_maybe_store_wm_command (struct frame *f)
       make_argc_argv (Vcommand_line_args, &argc, &argv);
       XSetCommand (XtDisplay (w), XtWindow (w), argv, argc);
       free_argc_argv (argv);
-      XSETFRAME (DEVICE_X_WM_COMMAND_FRAME (d), f);
+      DEVICE_X_WM_COMMAND_FRAME (d) = wrap_frame (f);
     }
 }
 
@@ -436,7 +438,7 @@ x_frame_iconified_p (struct frame *f)
 
 /* Connect the frame-property names (symbols) to the corresponding
    X Resource Manager names.  The name of a property, as a Lisp symbol,
-   has an `x-resource-name' property which is a Lisp_String. */
+   has an `x-resource-name' property which is a Lisp string. */
 
 static void
 init_x_prop_symbols (void)
@@ -492,13 +494,13 @@ init_x_prop_symbols (void)
 static Lisp_Object
 color_to_string (Widget w, unsigned long pixel)
 {
-  char buf[255];
+  Intbyte buf[255];
 
   XColor color;
   color.pixel = pixel;
   XQueryColor (XtDisplay (w), w->core.colormap, &color);
-  sprintf (buf, "#%04x%04x%04x", color.red, color.green, color.blue);
-  return build_string (buf);
+  qxesprintf (buf, "#%04x%04x%04x", color.red, color.green, color.blue);
+  return build_intstring (buf);
 }
 
 static void
@@ -578,7 +580,7 @@ x_frame_property (struct frame *f, Lisp_Object property)
   if (EQ (Qinter_line_space, property))
     return make_int (w->emacs_frame.interline);
   if (EQ (Qwindow_id, property))
-    return Fx_window_id (make_frame (f));
+    return Fx_window_id (wrap_frame (f));
 
   return Qunbound;
 }
@@ -611,7 +613,7 @@ x_frame_properties (struct frame *f)
   Widget gw = (Widget) w;
   Position x, y;
 
-  props = cons3 (Qwindow_id, Fx_window_id (make_frame (f)), props);
+  props = cons3 (Qwindow_id, Fx_window_id (wrap_frame (f)), props);
   props = cons3 (Qinter_line_space, make_int (w->emacs_frame.interline), props);
 
 #ifdef HAVE_TOOLBARS
@@ -802,7 +804,7 @@ x_set_frame_properties (struct frame *f, Lisp_Object plist)
 		    {
 		      Lisp_Object frm, font_spec;
 
-		      XSETFRAME (frm, f);
+		      frm = wrap_frame (f);
 		      font_spec = Fget (Fget_face (Qdefault), Qfont, Qnil);
 
 		      Fadd_spec_to_specifier (font_spec, val, frm, Qnil, Qnil);
@@ -933,15 +935,15 @@ x_set_frame_properties (struct frame *f, Lisp_Object plist)
       {
 	if (size_specified_p || internal_border_width_specified)
 	  {
-	    Lisp_Object frame;
-	    XSETFRAME (frame, f);
+	    Lisp_Object frame = wrap_frame (f);
+
 	    Fset_frame_size (frame, make_int (width),
 			      make_int (height), Qnil);
 	  }
 	if (position_specified_p)
 	  {
-	    Lisp_Object frame;
-	    XSETFRAME (frame, f);
+	    Lisp_Object frame = wrap_frame (f);
+
 	    Fset_frame_position (frame, make_int (x), make_int (y));
 	  }
       }
@@ -1253,7 +1255,7 @@ x_cde_transfer_callback (Widget widget, XtPointer clientData,
 
   GCPRO3 (frame, l_type, l_data);
 
-  frame = make_frame ((struct frame *) clientData);
+  frame = wrap_frame ((struct frame *) clientData);
 
   if (transferInfo->dropData->protocol == DtDND_FILENAME_TRANSFER)
     {
@@ -1295,7 +1297,7 @@ x_cde_transfer_callback (Widget widget, XtPointer clientData,
 			   l_data );
  	}
       drag_not_done = 0;
-      unbind_to(speccount, Qnil);
+      unbind_to (speccount);
     }
   else /* the other cases: NOOP_TRANSFER */
     enqueue=0;
@@ -2155,7 +2157,8 @@ allocate_x_frame_struct (struct frame *f)
 /************************************************************************/
 
 static void
-x_init_frame_1 (struct frame *f, Lisp_Object props)
+x_init_frame_1 (struct frame *f, Lisp_Object props,
+		int frame_name_is_defaulted)
 {
   /* This function can GC */
   Lisp_Object device = FRAME_DEVICE (f);
@@ -2286,11 +2289,11 @@ a string.
 */
        (frame))
 {
-  char str[255];
+  Intbyte str[255];
   struct frame *f = decode_x_frame (frame);
 
-  sprintf (str, "%lu", XtWindow (FRAME_X_TEXT_WIDGET (f)));
-  return build_string (str);
+  qxesprintf (str, "%lu", XtWindow (FRAME_X_TEXT_WIDGET (f)));
+  return build_intstring (str);
 }
 
 
@@ -2416,7 +2419,7 @@ x_get_mouse_position (struct device *d, Lisp_Object *frame, int *x, int *y)
   f = x_any_window_to_frame (d, win);
   if (!f)
     return 0;
-  XSETFRAME (*frame, f);
+  *frame = wrap_frame (f);
 
   if (XTranslateCoordinates (display, win,
 			     XtWindow (FRAME_X_TEXT_WIDGET (f)),
@@ -2715,9 +2718,8 @@ x_update_frame_external_traits (struct frame* frm, Lisp_Object name)
 {
   Arg al[10];
   int ac = 0;
-  Lisp_Object frame;
+  Lisp_Object frame = wrap_frame (frm);
 
-  XSETFRAME(frame, frm);
 
   if (EQ (name, Qforeground))
    {
