@@ -23,17 +23,8 @@ Boston, MA 02111-1307, USA.  */
 #ifndef INCLUDED_frame_h_
 #define INCLUDED_frame_h_
 
-#ifdef HAVE_SCROLLBARS
-#include "scrollbar.h"
-#endif
-
-#ifdef HAVE_TOOLBARS
-#include "toolbar.h"
-#endif
-
-#include "device.h"
-#include "glyphs.h"
 #include "redisplay.h"
+#include "console.h" /* for error_check_frame_type */
 
 #define FRAME_TYPE_NAME(f) ((f)->framemeths->name)
 #define FRAME_TYPE(f) ((f)->framemeths->symbol)
@@ -236,7 +227,6 @@ extern Lisp_Object Vmouse_motion_handler;
 
 DECLARE_LRECORD (frame, struct frame);
 #define XFRAME(x) XRECORD (x, frame, struct frame)
-#define XSETFRAME(x, p) XSETRECORD (x, p, frame)
 #define wrap_frame(p) wrap_record (p, frame)
 #define FRAMEP(x) RECORDP (x, frame)
 #define CHECK_FRAME(x) CHECK_RECORD (x, frame)
@@ -255,7 +245,7 @@ DECLARE_LRECORD (frame, struct frame);
 
 #define FRAME_TYPE_P(f, type)	EQ (FRAME_TYPE (f), Q##type)
 
-#ifdef ERROR_CHECK_TYPECHECK
+#ifdef ERROR_CHECK_TYPES
 INLINE_HEADER struct frame *
 error_check_frame_type (struct frame * f, Lisp_Object sym);
 INLINE_HEADER struct frame *
@@ -540,6 +530,7 @@ extern int frame_changed;
 #define FRAME_ROOT_WINDOW(f) ((f)->root_window)
 /* Catch people attempting to set this. */
 #define FRAME_SELECTED_WINDOW(f) NON_LVALUE ((f)->selected_window)
+#define FRAME_SELECTED_XWINDOW(f) XWINDOW (FRAME_SELECTED_WINDOW (f))
 #define FRAME_LAST_NONMINIBUF_WINDOW(f) \
   NON_LVALUE ((f)->last_nonminibuf_window)
 #define FRAME_SB_VCACHE(f) ((f)->sb_vcache)
