@@ -1,6 +1,6 @@
 /* Commonly-used symbols -- include file
    Copyright (C) 1995 Sun Microsystems.
-   Copyright (C) 1995, 1996, 2000 Ben Wing.
+   Copyright (C) 1995, 1996, 2000, 2001, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -50,6 +50,7 @@ SYMBOL (Qafter);
 SYMBOL (Qall);
 SYMBOL (Qand);
 SYMBOL (Qappend);
+SYMBOL (Qascii);
 SYMBOL (Qassoc);
 SYMBOL (Qat);
 SYMBOL (Qautodetect);
@@ -70,24 +71,31 @@ SYMBOL_KEYWORD (Q_callback_ex);
 SYMBOL (Qcancel);
 SYMBOL (Qcategory);
 SYMBOL (Qcenter);
+SYMBOL (Qchain);
 SYMBOL (Qchannel);
 SYMBOL (Qchar);
 SYMBOL (Qcharacter);
 SYMBOL (Qchars);
+SYMBOL (Qcode_page);
+SYMBOL (Qcoding_system);
 SYMBOL (Qcolor);
 SYMBOL (Qcolumns);
 SYMBOL (Qcommand);
 SYMBOL_KEYWORD (Q_config);
 SYMBOL (Qconsole);
+SYMBOL (Qcontrol_1);
 SYMBOL (Qcopies);
 SYMBOL (Qcritical);
 SYMBOL (Qctext);
+SYMBOL (Qcurrent);
 SYMBOL (Qcursor);
 SYMBOL (Qdata);
 SYMBOL (Qdead);
+SYMBOL (Qdebug);
 SYMBOL (Qdefault);
 SYMBOL (Qdelete);
 SYMBOL (Qdelq);
+SYMBOL (Qdescription);
 SYMBOL_KEYWORD (Q_descriptor);
 SYMBOL (Qdevice);
 SYMBOL_KEYWORD (Q_device);
@@ -96,8 +104,10 @@ SYMBOL (Qdirectory);
 SYMBOL (Qdimension);
 SYMBOL (Qdisplay);
 SYMBOL (Qdoc_string);
+SYMBOL (Qdocumentation);
 SYMBOL (Qduplex);
 SYMBOL (Qdynarr_overhead);
+SYMBOL (Qemergency);
 SYMBOL (Qempty);
 SYMBOL (Qeq);
 SYMBOL (Qeql);
@@ -125,6 +135,7 @@ SYMBOL (Qgutter);
 SYMBOL (Qheight);
 SYMBOL_KEYWORD (Q_height);
 SYMBOL (Qhelp);
+SYMBOL (Qhigh);
 SYMBOL (Qhighlight);
 SYMBOL (Qhorizontal);
 SYMBOL (Qicon);
@@ -152,7 +163,11 @@ SYMBOL (Qlast_command);
 SYMBOL (Qleft);
 SYMBOL (Qleft_margin);
 SYMBOL (Qlet);
+SYMBOL (Qlevel);
 SYMBOL (Qlist);
+SYMBOL (Qlittle_endian);
+SYMBOL (Qlocale);
+SYMBOL (Qlow);
 SYMBOL (Qmagic);
 SYMBOL (Qmalloc_overhead);
 SYMBOL_KEYWORD (Q_margin_width);
@@ -167,6 +182,7 @@ SYMBOL (Qmotion);
 SYMBOL (Qmsprinter);
 SYMBOL (Qmswindows);
 SYMBOL (Qname);
+SYMBOL (Qnative);
 SYMBOL (Qnatnum);
 SYMBOL (Qno);
 SYMBOL (Qnone);
@@ -227,8 +243,10 @@ SYMBOL (Qstream);
 SYMBOL (Qstring);
 SYMBOL_KEYWORD (Q_style);
 SYMBOL_KEYWORD (Q_suffix);
+SYMBOL (Qsubtype);
 SYMBOL (Qsymbol);
 SYMBOL (Qsyntax);
+SYMBOL (Qsystem_default);
 SYMBOL (Qterminal);
 SYMBOL (Qtest);
 SYMBOL (Qtext);
@@ -247,6 +265,7 @@ SYMBOL (Qtype);
 SYMBOL (Qundecided);
 SYMBOL (Qundefined);
 SYMBOL (Qunimplemented);
+SYMBOL (Quser_default);
 SYMBOL_KEYWORD (Q_value);
 SYMBOL (Qvalue_assoc);
 SYMBOL (Qvertical);
