@@ -24,18 +24,19 @@ Boston, MA 02111-1307, USA.  */
 
 #include <config.h>
 #include "lisp.h"
-#include "lstream.h"
+
+#include "bytecode.h"
 #include "console.h"
 #include "device.h"
 #include "faces.h"
-#include "glyphs.h"
-#include "objects.h"
-#include "bytecode.h"
-#include "window.h"
-#include "buffer.h"
 #include "frame.h"
+#include "glyphs.h"
+#include "gui.h"
 #include "insdel.h"
+#include "lstream.h"
+#include "objects.h"
 #include "opaque.h"
+#include "window.h"
 
 DEFINE_IMAGE_INSTANTIATOR_FORMAT (button);
 DEFINE_IMAGE_INSTANTIATOR_FORMAT (combo_box);
@@ -167,7 +168,7 @@ check_valid_string_or_vector (Lisp_Object data)
 	invalid_argument (":descriptor must be a string or a vector", data);
 }
 
-void
+static void
 check_valid_item_list (Lisp_Object items)
 {
   Lisp_Object rest;
@@ -435,6 +436,20 @@ redisplay_widget (Lisp_Object widget)
     }
 }
 
+static void
+widget_query_string_geometry (Lisp_Object string, Lisp_Object face,
+			      int *width, int *height, Lisp_Object domain)
+{
+  struct device *d = DOMAIN_XDEVICE (domain);
+
+  if (HAS_DEVMETH_P (d, widget_query_string_geometry))
+    DEVMETH (d, widget_query_string_geometry,
+	     (string, face, width, height, domain));
+  else
+    query_string_geometry (string, face, width, height, 0, domain);
+    
+}
+
 /* Query for a widgets desired geometry. If no type specific method is
    provided then use the widget text to calculate sizes. */
 static void
@@ -478,9 +493,9 @@ widget_query_geometry (Lisp_Object image_instance,
 
 	      /* Then if we are allowed to resize the widget, make the
 		 size the same as the text dimensions. */
-	      query_string_geometry (IMAGE_INSTANCE_WIDGET_TEXT (ii),
-				     IMAGE_INSTANCE_WIDGET_FACE (ii),
-				     &w, &h, 0, domain);
+	      widget_query_string_geometry (IMAGE_INSTANCE_WIDGET_TEXT (ii),
+					    IMAGE_INSTANCE_WIDGET_FACE (ii),
+					    &w, &h, domain);
 	      /* Adjust the size for borders. */
 	      if (IMAGE_INSTANCE_SUBWINDOW_H_RESIZEP (ii))
 		*width = w + 2 * WIDGET_BORDER_WIDTH;
@@ -773,9 +788,9 @@ button_query_geometry (Lisp_Object image_instance,
 {
   Lisp_Image_Instance *ii = XIMAGE_INSTANCE (image_instance);
   int w, h;
-  query_string_geometry (IMAGE_INSTANCE_WIDGET_TEXT (ii),
-			 IMAGE_INSTANCE_WIDGET_FACE (ii),
-			 &w, &h, 0, domain);
+  widget_query_string_geometry (IMAGE_INSTANCE_WIDGET_TEXT (ii),
+				IMAGE_INSTANCE_WIDGET_FACE (ii),
+				&w, &h, domain);
   /* Adjust the size for borders. */
   if (IMAGE_INSTANCE_SUBWINDOW_H_RESIZEP (ii))
     {
@@ -804,9 +819,9 @@ tree_view_query_geometry (Lisp_Object image_instance,
   if (*width)
     {
       /* #### what should this be. reconsider when X has tree views. */
-      query_string_geometry (IMAGE_INSTANCE_WIDGET_TEXT (ii),
-			     IMAGE_INSTANCE_WIDGET_FACE (ii),
-			     width, 0, 0, domain);
+      widget_query_string_geometry (IMAGE_INSTANCE_WIDGET_TEXT (ii),
+				    IMAGE_INSTANCE_WIDGET_FACE (ii),
+				    width, 0, domain);
     }
   if (*height)
     {
@@ -833,9 +848,9 @@ tab_control_query_geometry (Lisp_Object image_instance,
     {
       int h, w;
 
-      query_string_geometry (XGUI_ITEM (XCAR (rest))->name,
-			     IMAGE_INSTANCE_WIDGET_FACE (ii),
-			     &w, &h, 0, domain);
+      widget_query_string_geometry (XGUI_ITEM (XCAR (rest))->name,
+				    IMAGE_INSTANCE_WIDGET_FACE (ii),
+				    &w, &h, domain);
       tw += 5 * WIDGET_BORDER_WIDTH; /* some bias */
       tw += w;
       th = max (th, h + 2 * WIDGET_BORDER_HEIGHT);
