@@ -1,4 +1,4 @@
-/* Generic glyph data structures + display tables
+ /* Generic glyph data structures + display tables
    Copyright (C) 1994 Board of Trustees, University of Illinois.
    Copyright (C) 1995, 1996 Ben Wing
 
@@ -25,7 +25,7 @@ Boston, MA 02111-1307, USA.  */
 #define INCLUDED_glyphs_h_
 
 #include "specifier.h"
-#include "gui.h"
+#include "window.h" /* need for GLYPH_CACHEL_WIDTH */
 
 /************************************************************************/
 /*			Image Instantiators				*/
@@ -355,7 +355,6 @@ void check_valid_string (Lisp_Object data);
 void check_valid_int (Lisp_Object data);
 void check_valid_face (Lisp_Object data);
 void check_valid_vector (Lisp_Object data);
-void check_valid_item_list (Lisp_Object items);
 
 void initialize_subwindow_image_instance (Lisp_Image_Instance*);
 void subwindow_instantiate (Lisp_Object image_instance, Lisp_Object instantiator,
@@ -398,7 +397,6 @@ DECLARE_DOESNT_RETURN (signal_double_image_error_2 (const char *string1,
 
 DECLARE_SPECIFIER_TYPE (image);
 #define XIMAGE_SPECIFIER(x) XSPECIFIER_TYPE (x, image)
-#define XSETIMAGE_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, image)
 #define IMAGE_SPECIFIERP(x) SPECIFIER_TYPEP (x, image)
 #define CHECK_IMAGE_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, image)
 #define CONCHECK_IMAGE_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, image)
@@ -428,7 +426,6 @@ struct image_specifier
 
 DECLARE_LRECORD (image_instance, Lisp_Image_Instance);
 #define XIMAGE_INSTANCE(x) XRECORD (x, image_instance, Lisp_Image_Instance)
-#define XSETIMAGE_INSTANCE(x, p) XSETRECORD (x, p, image_instance)
 #define wrap_image_instance(p) wrap_record (p, image_instance)
 #define IMAGE_INSTANCEP(x) RECORDP (x, image_instance)
 #define CHECK_IMAGE_INSTANCE(x) CHECK_RECORD (x, image_instance)
@@ -894,8 +891,8 @@ Lisp_Object pixmap_to_lisp_data (Lisp_Object name, int ok_if_data_invalid);
 #ifdef HAVE_WINDOW_SYSTEM
 Lisp_Object bitmap_to_lisp_data (Lisp_Object name, int *xhot, int *yhot,
 				 int ok_if_data_invalid);
-int read_bitmap_data_from_file (const char *filename, int *width,
-				int *height, unsigned char **datap,
+int read_bitmap_data_from_file (Lisp_Object filename, int *width,
+				int *height, UChar_Binary **datap,
 				int *x_hot, int *y_hot);
 Lisp_Object xbm_mask_file_munging (Lisp_Object alist, Lisp_Object file,
 				   Lisp_Object mask_file,
@@ -938,7 +935,6 @@ typedef struct Lisp_Glyph Lisp_Glyph;
 
 DECLARE_LRECORD (glyph, Lisp_Glyph);
 #define XGLYPH(x) XRECORD (x, glyph, Lisp_Glyph)
-#define XSETGLYPH(x, p) XSETRECORD (x, p, glyph)
 #define wrap_glyph(p) wrap_record (p, glyph)
 #define GLYPHP(x) RECORDP (x, glyph)
 #define CHECK_GLYPH(x) CHECK_RECORD (x, glyph)
@@ -1063,8 +1059,6 @@ struct glyph_cachel
 
 #ifdef ERROR_CHECK_GLYPHS
 
-#include "window.h"
-
 INLINE_HEADER int
 GLYPH_CACHEL_WIDTH (struct window *window, int ind);
 INLINE_HEADER int
@@ -1095,7 +1089,7 @@ GLYPH_CACHEL_DESCENT (struct window *window, int ind)
 
 #else /* not ERROR_CHECK_GLYPHS */
 
-#define GLYPH_CACHEL_WIDTH(window, ind)		\
+#define GLYPH_CACHEL_WIDTH(window, ind)			\
   Dynarr_atp (window->glyph_cachels, ind)->width
 #define GLYPH_CACHEL_ASCENT(window, ind)		\
   Dynarr_atp (window->glyph_cachels, ind)->ascent
@@ -1106,7 +1100,7 @@ GLYPH_CACHEL_DESCENT (struct window *window, int ind)
 
 #define GLYPH_CACHEL(window, ind)			\
   Dynarr_atp (window->glyph_cachels, ind)
-#define GLYPH_CACHEL_GLYPH(window, ind)		\
+#define GLYPH_CACHEL_GLYPH(window, ind)			\
   Dynarr_atp (window->glyph_cachels, ind)->glyph
 #define GLYPH_CACHEL_DIRTYP(window, ind)		\
   Dynarr_atp (window->glyph_cachels, ind)->dirty
