@@ -20,6 +20,9 @@ Boston, MA 02111-1307, USA.  */
 
 /* Synched up with: Not in FSF. */
 
+/* This file essentially Mule-ized (except perhaps some Unicode splitting).
+   5-2000. */
+
 #include <config.h>
 #include "lisp.h"
 #include "console-msw.h"
@@ -38,12 +41,12 @@ Boston, MA 02111-1307, USA.  */
  * command if we return nil
  */
 Lisp_Object
-mswindows_handle_gui_wm_command (struct frame* f, HWND ctrl, LPARAM id)
+mswindows_handle_gui_wm_command (struct frame *f, HWND ctrl, LPARAM id)
 {
   /* Try to map the command id through the proper hash table */
   Lisp_Object callback, callback_ex, image_instance, frame, event;
 
-  XSETFRAME (frame, f);
+  frame = wrap_frame (f);
 
   /* #### make_int should assert that --kkm */
   assert (XINT (make_int (id)) == id);
