@@ -1,6 +1,6 @@
 /* General GUI code -- X-specific. (menubars, scrollbars, toolbars, dialogs)
    Copyright (C) 1995 Board of Trustees, University of Illinois.
-   Copyright (C) 1995, 1996, 2000 Ben Wing.
+   Copyright (C) 1995, 1996, 2000, 2001, 2002 Ben Wing.
    Copyright (C) 1995 Sun Microsystems, Inc.
    Copyright (C) 1998 Free Software Foundation, Inc.
 
@@ -113,7 +113,7 @@ gcpro_popup_callbacks (LWLIB_ID id)
   pdata->id = id;
   pdata->last_menubar_buffer = Qnil;
   pdata->menubar_contents_up_to_date = 0;
-  XSETPOPUP_DATA (lpdata, pdata);
+  lpdata = wrap_popup_data (pdata);
   Vpopup_callbacks = Fcons (Fcons (lid, lpdata), Vpopup_callbacks);
 }
 
@@ -228,7 +228,7 @@ popup_selection_callback (Widget widget, LWLIB_ID ignored_id,
   if (((EMACS_INT) client_data) == 0)
     return;
   VOID_TO_LISP (data, client_data);
-  XSETFRAME (frame, f);
+  frame = wrap_frame (f);
 
 #if 0
   /* #### What the hell?  I can't understand why this call is here,
@@ -382,16 +382,16 @@ add_accel_and_to_external (Lisp_Object string)
   return retval;
 }
 
-/* This does the dirty work.  gc_currently_forbidden is 1 when this is called.
- */
+/* This does the dirty work.  begin_gc_forbidden() is active when this is
+ called. */
 int
 button_item_to_widget_value (Lisp_Object gui_object_instance,
 			     Lisp_Object gui_item, widget_value *wv,
 			     int allow_text_field_p, int no_keys_p,
 			     int menu_entry_p, int accel_p)
 {
-  /* This function cannot GC because gc_currently_forbidden is set when
-     it's called */
+  /* This function cannot GC because begin_gc_forbidden() is active when
+     it's called. */
   Lisp_Gui_Item* pgui = 0;
 
   /* degenerate case */
@@ -476,13 +476,14 @@ button_item_to_widget_value (Lisp_Object gui_object_instance,
     }
   else if (SYMBOLP (pgui->callback))	/* Show the binding of this command. */
     {
-      char buf[1024]; /* #### */
+      DECLARE_EISTRING_MALLOC (buf);
       /* #### Warning, dependency here on current_buffer and point */
       where_is_to_char (pgui->callback, buf);
-      if (buf [0])
-	C_STRING_TO_EXTERNAL_MALLOC (buf, wv->key, Qlwlib_encoding);
+      if (eilen (buf) > 0)
+	C_STRING_TO_EXTERNAL_MALLOC (eidata (buf), wv->key, Qlwlib_encoding);
       else
 	wv->key = 0;
+      eifree (buf);
     }
 
   CHECK_SYMBOL (pgui->style);
@@ -627,7 +628,7 @@ gui_items_to_widget_values (Lisp_Object gui_object_instance, Lisp_Object items,
 {
   /* This function can GC */
   widget_value *control = 0, *tmp = 0;
-  int count = specpdl_depth ();
+  int count;
   Lisp_Object wv_closure;
 
   if (NILP (items))
@@ -636,9 +637,7 @@ gui_items_to_widget_values (Lisp_Object gui_object_instance, Lisp_Object items,
   /* Inhibit GC during this conversion.  The reasons for this are
      the same as in menu_item_descriptor_to_widget_value(); see
      the large comment above that function. */
-  record_unwind_protect (restore_gc_inhibit,
-			 make_int (gc_currently_forbidden));
-  gc_currently_forbidden = 1;
+  count = begin_gc_forbidden ();
 
   /* Also make sure that we free the partially-created widget_value
      tree on Lisp error. */
@@ -658,7 +657,7 @@ gui_items_to_widget_values (Lisp_Object gui_object_instance, Lisp_Object items,
 
   /* No more need to free the half-filled-in structures. */
   set_opaque_ptr (wv_closure, 0);
-  unbind_to (count, Qnil);
+  unbind_to (count);
 
   return control;
 }
