@@ -1,6 +1,6 @@
 /* Generic GUI code. (menubars, scrollbars, toolbars, dialogs)
    Copyright (C) 1995 Board of Trustees, University of Illinois.
-   Copyright (C) 1995, 1996 Ben Wing.
+   Copyright (C) 1995, 1996, 2000, 2001, 2002 Ben Wing.
    Copyright (C) 1995 Sun Microsystems, Inc.
    Copyright (C) 1998 Free Software Foundation, Inc.
 
@@ -23,8 +23,7 @@ Boston, MA 02111-1307, USA.  */
 
 /* Synched up with: Not in FSF. */
 
-/* This file not quite Mule-ized yet but will be when merged with my
-   Mule workspace. --ben */
+/* This file Mule-ized by Ben Wing, 3-24-02. */
 
 #include <config.h>
 #include "lisp.h"
@@ -103,7 +102,7 @@ get_gui_callback (Lisp_Object data, Lisp_Object *fn, Lisp_Object *arg)
       *fn = Qeval;
       *arg = list3 (Qsignal,
 		    list2 (Qquote, Qerror),
-		    list2 (Qquote, list2 (build_translated_string
+		    list2 (Qquote, list2 (build_msg_string
 					  ("illegal callback"),
 					  data)));
     }
@@ -133,13 +132,13 @@ gui_item_add_keyval_pair (Lisp_Object gui_item,
 	  pgui_item->name   = val;
 	}
     }
-#define FROB(slot) \
+#define FROB(slot)				\
   else if (EQ (key, Q_##slot))			\
   {						\
-    if (!EQ (pgui_item->slot, val))			\
+    if (!EQ (pgui_item->slot, val))		\
       {						\
 	retval = 1;				\
-	pgui_item->slot   = val;			\
+	pgui_item->slot   = val;		\
       }						\
   }
   FROB (suffix)
@@ -168,8 +167,7 @@ gui_item_add_keyval_pair (Lisp_Object gui_item,
 	}
     }
   else if (ERRB_EQ (errb, ERROR_ME))
-    invalid_argument_2 ("Unknown keyword in gui item", key,
-			   pgui_item->name);
+    invalid_argument_2 ("Unknown keyword in gui item", key, pgui_item->name);
   return retval;
 }
 
@@ -200,7 +198,7 @@ allocate_gui_item (void)
   Lisp_Object val;
 
   zero_lcrecord (lp);
-  XSETGUI_ITEM (val, lp);
+  val = wrap_gui_item (lp);
 
   gui_item_init (val);
 
@@ -263,7 +261,7 @@ make_gui_item_from_keywords_internal (Lisp_Object item,
       if ((length - start) & 1)
 	sferror (
 		"GUI item descriptor has an odd number of keywords and values",
-			     item);
+		 item);
 
       for (i = start; i < length;)
 	{
@@ -332,7 +330,7 @@ update_gui_item_keywords (Lisp_Object gui_item, Lisp_Object item)
     {
       Lisp_Object key = contents [i++];
       Lisp_Object val = contents [i++];
-      if (gui_item_add_keyval_pair (gui_item, key, val, ERROR_ME_NOT))
+      if (gui_item_add_keyval_pair (gui_item, key, val, ERROR_ME_DEBUG_WARN))
 	retval = 1;
     }
   return retval;
@@ -347,7 +345,7 @@ gui_parse_item_keywords (Lisp_Object item)
 Lisp_Object
 gui_parse_item_keywords_no_errors (Lisp_Object item)
 {
-  return make_gui_item_from_keywords_internal (item, ERROR_ME_NOT);
+  return make_gui_item_from_keywords_internal (item, ERROR_ME_DEBUG_WARN);
 }
 
 /* convert a gui item into plist properties */
@@ -424,14 +422,12 @@ gui_name_accelerator (Lisp_Object nm)
 	  if (*name == '_' && *(name + 1))
 	    {
 	      Emchar accelerator = charptr_emchar (name + 1);
-	      /* #### bogus current_buffer dependency */
-	      return make_char (DOWNCASE (current_buffer, accelerator));
+	      return make_char (DOWNCASE (0, accelerator));
 	    }
 	}
 	INC_CHARPTR (name);
     }
-  return make_char (DOWNCASE (current_buffer,
-			      charptr_emchar (XSTRING_DATA (nm))));
+  return make_char (DOWNCASE (0, charptr_emchar (XSTRING_DATA (nm))));
 }
 
 /*
@@ -485,41 +481,19 @@ gui_item_included_p (Lisp_Object gui_item, Lisp_Object conflist)
   return 1;
 }
 
-static DOESNT_RETURN
-signal_too_long_error (Lisp_Object name)
-{
-  invalid_argument ("GUI item produces too long displayable string", name);
-}
-
-#ifdef HAVE_WINDOW_SYSTEM
 /*
- * Format "left flush" display portion of an item into BUF, guarded by
- * maximum buffer size BUF_LEN. BUF_LEN does not count for terminating
- * null character, so actual maximum size of buffer consumed is
- * BUF_LEN + 1 bytes. If buffer is not big enough, then error is
- * signaled.
- * Return value is the offset to the terminating null character into the
- * buffer.
+ * Format "left flush" display portion of an item.
  */
-Bytecount
-gui_item_display_flush_left (Lisp_Object gui_item,
-			     char *buf, Bytecount buf_len)
+Lisp_Object
+gui_item_display_flush_left (Lisp_Object gui_item)
 {
   /* This function can call lisp */
-  char *p = buf;
-  Bytecount len;
   Lisp_Gui_Item *pgui_item = XGUI_ITEM (gui_item);
+  Lisp_Object retval;
 
-  /* Copy item name first */
   CHECK_STRING (pgui_item->name);
-  len = XSTRING_LENGTH (pgui_item->name);
-  if (len > buf_len)
-    signal_too_long_error (pgui_item->name);
-  memcpy (p, XSTRING_DATA (pgui_item->name), len);
-  p += len;
+  retval = pgui_item->name;
 
-  /* Add space and suffix, if there is a suffix.
-   * If suffix is not string evaluate it */
   if (!NILP (pgui_item->suffix))
     {
       Lisp_Object suffix = pgui_item->suffix;
@@ -530,68 +504,48 @@ gui_item_display_flush_left (Lisp_Object gui_item,
 	  CHECK_STRING (suffix);
 	}
 
-      len = XSTRING_LENGTH (suffix);
-      if (p + len + 1 > buf + buf_len)
-	signal_too_long_error (pgui_item->name);
-      *(p++) = ' ';
-      memcpy (p, XSTRING_DATA (suffix), len);
-      p += len;
+      retval = concat3 (pgui_item->name, build_string (" "), suffix);
     }
-  *p = '\0';
-  return p - buf;
+
+  return retval;
 }
 
 /*
- * Format "right flush" display portion of an item into BUF, guarded by
- * maximum buffer size BUF_LEN. BUF_LEN does not count for terminating
- * null character, so actual maximum size of buffer consumed is
- * BUF_LEN + 1 bytes. If buffer is not big enough, then error is
- * signaled.
- * Return value is the offset to the terminating null character into the
- * buffer.
+ * Format "right flush" display portion of an item into BUF.
  */
-Bytecount
-gui_item_display_flush_right (Lisp_Object gui_item,
-			      char *buf, Bytecount buf_len)
+Lisp_Object
+gui_item_display_flush_right (Lisp_Object gui_item)
 {
   Lisp_Gui_Item *pgui_item = XGUI_ITEM (gui_item);
-  *buf = 0;
 
 #ifdef HAVE_MENUBARS
   /* Have keys? */
   if (!menubar_show_keybindings)
-    return 0;
+    return Qnil;
 #endif
 
   /* Try :keys first */
   if (!NILP (pgui_item->keys))
     {
       CHECK_STRING (pgui_item->keys);
-      if (XSTRING_LENGTH (pgui_item->keys) + 1 > buf_len)
-	signal_too_long_error (pgui_item->name);
-      memcpy (buf, XSTRING_DATA (pgui_item->keys),
-	      XSTRING_LENGTH (pgui_item->keys) + 1);
-      return XSTRING_LENGTH (pgui_item->keys);
+      return pgui_item->keys;
     }
 
   /* See if we can derive keys out of callback symbol */
   if (SYMBOLP (pgui_item->callback))
     {
-      char buf2[1024]; /* #### */
-      Bytecount len;
-
-      where_is_to_char (pgui_item->callback, buf2);
-      len = strlen (buf2);
-      if (len > buf_len)
-	signal_too_long_error (pgui_item->name);
-      strcpy (buf, buf2);
-      return len;
+      DECLARE_EISTRING_MALLOC (buf);
+      Lisp_Object str;
+      
+      where_is_to_char (pgui_item->callback, buf);
+      str = eimake_string (buf);
+      eifree (buf);
+      return str;
     }
 
   /* No keys - no right flush display */
-  return 0;
+  return Qnil;
 }
-#endif /* HAVE_WINDOW_SYSTEM */
 
 static Lisp_Object
 mark_gui_item (Lisp_Object obj)
@@ -698,14 +652,11 @@ static void
 print_gui_item (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   Lisp_Gui_Item *g = XGUI_ITEM (obj);
-  char buf[20];
 
   if (print_readably)
     printing_unreadable_object ("#<gui-item 0x%x>", g->header.uid);
 
-  write_c_string ("#<gui-item ", printcharfun);
-  sprintf (buf, "0x%x>", g->header.uid);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, "#<gui-item 0x%x>", g->header.uid);
 }
 
 Lisp_Object
@@ -814,7 +765,7 @@ parse_gui_item_tree_list (Lisp_Object list)
 }
 
 static void
-finalize_gui_item (void* header, int for_disksave)
+finalize_gui_item (void *header, int for_disksave)
 {
 }
 
@@ -827,13 +778,13 @@ DEFINE_LRECORD_IMPLEMENTATION ("gui-item", gui_item,
 
 
 DOESNT_RETURN
-gui_error (const char *reason, Lisp_Object frob)
+gui_error (const Char_ASCII *reason, Lisp_Object frob)
 {
   signal_error (Qgui_error, reason, frob);
 }
 
 DOESNT_RETURN
-gui_error_2 (const char *reason, Lisp_Object frob0, Lisp_Object frob1)
+gui_error_2 (const Char_ASCII *reason, Lisp_Object frob0, Lisp_Object frob1)
 {
   signal_error_2 (Qgui_error, reason, frob0, frob1);
 }
