@@ -62,7 +62,6 @@ struct Lisp_Gui_Item
 
 DECLARE_LRECORD (gui_item, Lisp_Gui_Item);
 #define XGUI_ITEM(x) XRECORD (x, gui_item, Lisp_Gui_Item)
-#define XSETGUI_ITEM(x, p) XSETRECORD (x, p, gui_item)
 #define wrap_gui_item(p) wrap_record (p, gui_item)
 #define GUI_ITEMP(x) RECORDP (x, gui_item)
 #define CHECK_GUI_ITEM(x) CHECK_RECORD (x, gui_item)
@@ -84,11 +83,8 @@ int  gui_item_included_p (Lisp_Object, Lisp_Object into);
 Lisp_Object gui_item_accelerator (Lisp_Object gui_item);
 Lisp_Object gui_name_accelerator (Lisp_Object name);
 int  gui_item_id_hash (Lisp_Object, Lisp_Object gui_item, int);
-Bytecount gui_item_display_flush_left  (Lisp_Object pgui_item,
-					char* buf, Bytecount buf_len);
-Bytecount gui_item_display_flush_right (Lisp_Object gui_item,
-					char* buf, Bytecount buf_len);
-
+Lisp_Object gui_item_display_flush_left (Lisp_Object gui_item);
+Lisp_Object gui_item_display_flush_right (Lisp_Object gui_item);
 Lisp_Object allocate_gui_item (void);
 void gui_item_init (Lisp_Object gui_item);
 Lisp_Object parse_gui_item_tree_children (Lisp_Object list);
