@@ -349,10 +349,7 @@ calculate_gutter_size (struct window *w, enum gutter_pos pos)
 
   /* Redisplay code that we use relies on GC not happening. Make it
      so. */
-  count = specpdl_depth ();
-  record_unwind_protect (restore_gc_inhibit,
-			 make_int (gc_currently_forbidden));
-  gc_currently_forbidden = 1;
+  count = begin_gc_forbidden ();
 
   ddla = Dynarr_new (display_line);
   /* generate some display lines */
@@ -366,7 +363,7 @@ calculate_gutter_size (struct window *w, enum gutter_pos pos)
 			     ddla, 0, 0);
 
   /* Let GC happen again. */
-  unbind_to (count, Qnil);
+  unbind_to (count);
 
   ret = make_int (calculate_gutter_size_from_display_lines (pos, ddla));
   free_display_lines (ddla);
@@ -403,7 +400,7 @@ output_gutter (struct frame *f, enum gutter_pos pos, int force)
   cdla = f->current_display_lines[pos];
   cdla_len = Dynarr_length (cdla);
 
-  XSETFRAME (frame, f);
+  frame = wrap_frame (f);
 
   get_gutter_coords (f, pos, &x, &y, &width, &height);
   /* generate some display lines */
@@ -733,7 +730,7 @@ decode_gutter_position (Lisp_Object position)
   if (EQ (position, Qright))  return RIGHT_GUTTER;
   invalid_constant ("Invalid gutter position", position);
 
-  return TOP_GUTTER; /* not reached */
+  RETURN_NOT_REACHED (TOP_GUTTER)
 }
 
 DEFUN ("set-default-gutter-position", Fset_default_gutter_position, 1, 1, 0, /*
