@@ -27,25 +27,25 @@ typedef struct
 } hentry;
 
 typedef int           (*hash_table_test_function) (const void *, const void *);
-typedef Hash_Code     (*hash_table_hash_function) (const void *);
+typedef Hashcode     (*hash_table_hash_function) (const void *);
 
 struct hash_table
 {
   hentry	*harray;
   long		zero_set;
   void		*zero_entry;
-  Element_Count	size;		/* size of the hasharray */
-  Element_Count	fullness;	/* number of entries in the hash table */
+  Elemcount	size;		/* size of the hasharray */
+  Elemcount	fullness;	/* number of entries in the hash table */
   hash_table_hash_function hash_function;
   hash_table_test_function test_function;
 };
 
 /* SIZE is the number of initial entries. The hash table will be grown
    automatically if the number of entries approaches the size */
-struct hash_table *make_hash_table (Element_Count size);
+struct hash_table *make_hash_table (Elemcount size);
 
 struct hash_table *
-make_general_hash_table (Element_Count size,
+make_general_hash_table (Elemcount size,
 			 hash_table_hash_function hash_function,
 			 hash_table_test_function test_function);
 
