@@ -1,6 +1,7 @@
 /* Fundamental definitions for XEmacs Lisp interpreter -- union objects.
    Copyright (C) 1985, 1986, 1987, 1992, 1993, 1994
    Free Software Foundation, Inc.
+   Copyright (C) 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -81,23 +82,6 @@ union Lisp_Object
 Lisp_Object;
 
 #define XCHARVAL(x) ((x).gu.val)
-
-# define XSETINT(var, value) do {	\
-  EMACS_INT xset_value = (value);	\
-  Lisp_Object *xset_var = &(var);	\
-  xset_var->s.bits = 1;			\
-  xset_var->s.val = xset_value;		\
-} while (0)
-# define XSETCHAR(var, value) do {	\
-  Emchar xset_value = (value);		\
-  Lisp_Object *xset_var = &(var);	\
-  xset_var->gu.type = Lisp_Type_Char;	\
-  xset_var->gu.val = xset_value;	\
-} while (0)
-# define XSETOBJ(var, value) do {		\
-  EMACS_UINT xset_value = (EMACS_UINT) (value);	\
-  (var).ui = xset_value;			\
-} while (0)
 # define XPNTRVAL(x) ((x).ui)
 
 INLINE_HEADER Lisp_Object make_int (EMACS_INT val);
@@ -105,25 +89,55 @@ INLINE_HEADER Lisp_Object
 make_int (EMACS_INT val)
 {
   Lisp_Object obj;
-  XSETINT (obj, val);
+  obj.s.bits = 1;
+  obj.s.val = val;
   return obj;
 }
 
+
+#ifdef __cplusplus
+
+#define volatile_make_int(val) make_int (val)
+
+#else
+
+/* Ugh, need different definition to avoid compiler complaint in
+   unix_send_process().  Furthermore, there's no way under C++, it seems,
+   to declare something volatile and then return it.  Perhaps I'd have to
+   assign to something else instead?  But in any case, the warnings about
+   volatile clobbering doesn't occur in C++.  I bet the thing is that C++
+   already has a built-in system for dealing with non-local exits and such,
+   in a smart way that doesn't clobber registers, and incorporates
+   longjmp() into that.  */
+INLINE_HEADER Lisp_Object volatile_make_int (EMACS_INT val);
+INLINE_HEADER Lisp_Object
+volatile_make_int (EMACS_INT val)
+{
+  volatile Lisp_Object obj;
+  obj.s.bits = 1;
+  obj.s.val = val;
+  return obj;
+}
+
+#endif /* __cplusplus */
+
+
 INLINE_HEADER Lisp_Object make_char (Emchar val);
 INLINE_HEADER Lisp_Object
 make_char (Emchar val)
 {
   Lisp_Object obj;
-  XSETCHAR (obj, val);
+  obj.gu.type = Lisp_Type_Char;
+  obj.gu.val = val;
   return obj;
 }
 
-INLINE_HEADER Lisp_Object wrap_pointer_1 (void *ptr);
+INLINE_HEADER Lisp_Object wrap_pointer_1 (const void *ptr);
 INLINE_HEADER Lisp_Object
-wrap_pointer_1 (void *ptr)
+wrap_pointer_1 (const void *ptr)
 {
   Lisp_Object obj;
-  XSETOBJ (obj, ptr);
+  obj.ui = (EMACS_UINT) ptr;
   return obj;
 }
 
