@@ -24,9 +24,7 @@ Boston, MA 02111-1307, USA.  */
    operating system this machine is likely to run.
    USUAL-OPSYS="usg5-2"  */
 
-#define EXPLICIT_SIGN_EXTEND
-
-#define LIB_STANDARD -lc
+#define LIB_STANDARD "-lc"
 
 #ifdef __GNUC__
 #define alloca __builtin_alloca
@@ -44,11 +42,9 @@ Boston, MA 02111-1307, USA.  */
 #undef TERMINFO
 
 #undef CANNOT_DUMP
-#undef SHORTNAMES
 #define TERMCAP
 
-#define LIBS_TERMCAP -ltermlib
-#define SYSTEM_PURESIZE_EXTRA 100000
+#define LIBS_TERMCAP "-ltermlib"
 #define ALTOS
 
 #ifdef __GNUC__
