@@ -6,7 +6,7 @@
 #define COFF
 #define SYSTEM_MALLOC
 #define TEXT_START 0x10000
-#define LIB_STANDARD -lc -lic -lmach
+#define LIB_STANDARD "-lc -lic -lmach"
 #define KEEP_OLD_TEXT_SCNPTR
 #define KEEP_OLD_PADDR
 #define drem fmod
