@@ -1,5 +1,6 @@
 /* Markers: examining, setting and killing.
    Copyright (C) 1985, 1992, 1993, 1994, 1995 Free Software Foundation, Inc.
+   Copyright (C) 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -48,7 +49,7 @@ mark_marker (Lisp_Object obj)
   if (!marker->buffer)
     return (Qnil);
 
-  XSETBUFFER (buf, marker->buffer);
+  buf = wrap_buffer (marker->buffer);
   return (buf);
 }
 
@@ -56,7 +57,6 @@ static void
 print_marker (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   Lisp_Marker *marker = XMARKER (obj);
-  char buf[200];
 
   if (print_readably)
     printing_unreadable_object ("#<marker 0x%lx>", (long) marker);
@@ -66,12 +66,10 @@ print_marker (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
     write_c_string (GETTEXT ("in no buffer"), printcharfun);
   else
     {
-      sprintf (buf, "at %ld in ", (long) marker_position (obj));
-      write_c_string (buf, printcharfun);
+      write_fmt_string (printcharfun, "at %ld in ", (long) marker_position (obj));
       print_internal (marker->buffer->name, printcharfun, 0);
     }
-  sprintf (buf, " 0x%lx>", (long) marker);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, " 0x%lx>", (long) marker);
 }
 
 static int
@@ -120,9 +118,7 @@ Return nil if MARKER points into a dead buffer or doesn't point anywhere.
   /* Return marker's buffer only if it is not dead.  */
   if ((buf = XMARKER (marker)->buffer) && BUFFER_LIVE_P (buf))
     {
-      Lisp_Object buffer;
-      XSETBUFFER (buffer, buf);
-      return buffer;
+      return wrap_buffer (buf);
     }
   return Qnil;
 }
@@ -301,7 +297,7 @@ unchain_marker (Lisp_Object m)
   if (b == 0)
     return;
 
-#ifdef ERROR_CHECK_GC
+#ifdef ERROR_CHECK_STRUCTURES
   assert (BUFFER_LIVE_P (b));
 #endif
 
@@ -312,7 +308,7 @@ unchain_marker (Lisp_Object m)
   else
     BUF_MARKERS (b) = marker_next (marker);
 
-#ifdef ERROR_CHECK_GC
+#ifdef ERROR_CHECK_STRUCTURES
   assert (marker != XMARKER (b->point_marker));
 #endif
 
@@ -337,7 +333,7 @@ bi_marker_position (Lisp_Object marker)
      positions. */
   pos = membpos_to_bytebpos (buf, m->membpos);
 
-#ifdef ERROR_CHECK_CHARBPOS
+#ifdef ERROR_CHECK_TEXT
   if (pos < BI_BUF_BEG (buf) || pos > BI_BUF_Z (buf))
     abort ();
 #endif
@@ -365,7 +361,7 @@ set_bi_marker_position (Lisp_Object marker, Bytebpos pos)
   if (!buf)
     invalid_argument ("Marker does not point anywhere", Qunbound);
 
-#ifdef ERROR_CHECK_CHARBPOS
+#ifdef ERROR_CHECK_TEXT
   if (pos < BI_BUF_BEG (buf) || pos > BI_BUF_Z (buf))
     abort ();
 #endif
@@ -531,9 +527,8 @@ syms_of_marker (void)
 void
 init_buffer_markers (struct buffer *b)
 {
-  Lisp_Object buf;
+  Lisp_Object buf = wrap_buffer (b);
 
-  XSETBUFFER (buf, b);
   b->mark = Fmake_marker ();
   BUF_MARKERS (b) = 0;
   b->point_marker = Fmake_marker ();
