@@ -563,7 +563,7 @@ __generic_button_callback (GtkMenuItem *item, gpointer user_data)
 {
   Lisp_Object callback, function, data, channel;
 
-  XSETFRAME (channel, __get_channel (GTK_WIDGET (item)));
+  channel = wrap_frame (__get_channel (GTK_WIDGET (item)));
 
   VOID_TO_LISP (callback, user_data);
 
@@ -748,12 +748,13 @@ static GtkWidget *menu_descriptor_to_widget_1 (Lisp_Object descr)
 	    }
 	  else if (SYMBOLP (callback))
 	    {
-	      char buf[1024];
+	      DECLARE_EISTRING_MALLOC (buf);
 
 	      /* #### Warning, dependency here on current_buffer and point */
 	      where_is_to_char (callback, buf);
 
-	      keys = build_string (buf);
+	      keys = eimake_string (buf);
+	      eifree (buf);
 	    }
 	}
 
@@ -865,16 +866,12 @@ static GtkWidget *menu_descriptor_to_widget_1 (Lisp_Object descr)
 
 static GtkWidget *menu_descriptor_to_widget (Lisp_Object descr)
 {
-  int count = specpdl_depth ();
   GtkWidget *rval = NULL;
-
-  record_unwind_protect (restore_gc_inhibit, make_int (gc_currently_forbidden));
-
-  gc_currently_forbidden = 1;
+  int count = begin_gc_forbidden ();
 
   /* Cannot GC from here on out... */
   rval = menu_descriptor_to_widget_1 (descr);
-  unbind_to (count, Qnil);
+  unbind_to (count);
   return (rval);
   
 }
@@ -1108,7 +1105,7 @@ set_frame_menubar (struct frame *f, int first_time_p)
     menu_create_menubar (f, menubar);
 
     Fset_buffer (old_buffer);
-    unbind_to (count, Qnil);
+    unbind_to (count);
   }
 
   FRAME_MENUBAR_DATA (f) = Fcons (XWINDOW (FRAME_LAST_NONMINIBUF_WINDOW (f))->buffer, Qt);
