@@ -1,7 +1,7 @@
 /* Implements an elisp-programmable menubar -- X interface.
    Copyright (C) 1993, 1994 Free Software Foundation, Inc.
    Copyright (C) 1995 Tinker Systems and INS Engineering Corp.
-   Copyright (C) 2000 Ben Wing.
+   Copyright (C) 2000, 2001 ,2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -36,13 +36,9 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
-#include "console-x.h"
-#include "EmacsFrame.h"
-#include "gui-x.h"
-#include "../lwlib/lwlib.h"
-
 #include "buffer.h"
 #include "commands.h"           /* zmacs_regions */
+#include "device.h"
 #include "events.h"
 #include "frame.h"
 #include "gui.h"
@@ -51,6 +47,12 @@ Boston, MA 02111-1307, USA.  */
 #include "opaque.h"
 #include "window.h"
 
+#include "console-x.h"
+#include "gui-x.h"
+
+#include "EmacsFrame.h"
+#include "../lwlib/lwlib.h"
+
 static int set_frame_menubar (struct frame *f,
 			      int deep_p,
 			      int first_time_p);
@@ -335,7 +337,7 @@ menu_item_descriptor_to_widget_value_1 (Lisp_Object desc,
       set_opaque_ptr (wv_closure, 0);
     }
 
-  unbind_to (count, Qnil);
+  unbind_to (count);
   return wv;
 }
 
@@ -348,14 +350,11 @@ menu_item_descriptor_to_widget_value (Lisp_Object desc,
 							should run now */
 {
   widget_value *wv;
-  int count = specpdl_depth ();
-  record_unwind_protect (restore_gc_inhibit,
-			 make_int (gc_currently_forbidden));
-  gc_currently_forbidden = 1;
+  int count = begin_gc_forbidden ();
   /* Can't GC! */
   wv = menu_item_descriptor_to_widget_value_1 (desc, menu_type, deep_p,
 					       filter_p, 0);
-  unbind_to (count, Qnil);
+  unbind_to (count);
   return wv;
 }
 
@@ -438,7 +437,7 @@ pre_activate_callback (Widget widget, LWLIB_ID id, XtPointer client_data)
     return;
 
   /* make sure f is the selected frame */
-  XSETFRAME (frame, f);
+  frame = wrap_frame (f);
   Fselect_frame (frame);
 
   if (client_data)
@@ -464,7 +463,7 @@ pre_activate_callback (Widget widget, LWLIB_ID id, XtPointer client_data)
       in_menu_callback = 1;
       wv = menu_item_descriptor_to_widget_value (submenu_desc, SUBMENU_TYPE,
 						 1, 0);
-      unbind_to (count, Qnil);
+      unbind_to (count);
 
       if (!wv)
 	{
@@ -528,7 +527,7 @@ compute_menubar_data (struct frame *f, Lisp_Object menubar, int deep_p)
       Fset_buffer (XWINDOW (FRAME_SELECTED_WINDOW (f))->buffer);
       data = menu_item_descriptor_to_widget_value (menubar, MENUBAR_TYPE,
 						   deep_p, 0);
-      unbind_to (count, Qnil);
+      unbind_to (count);
 
       return data;
     }
@@ -582,7 +581,7 @@ set_frame_menubar (struct frame *f, int deep_p, int first_time_p)
       mdata->id = new_lwlib_id ();
       mdata->last_menubar_buffer = Qnil;
       mdata->menubar_contents_up_to_date = 0;
-      XSETPOPUP_DATA (FRAME_MENUBAR_DATA (f), mdata);
+      FRAME_MENUBAR_DATA (f) = wrap_popup_data (mdata);
     }
 
   /***** now store into the menubar widget, creating it if necessary *****/
@@ -783,9 +782,8 @@ x_popup_menu (Lisp_Object menu_desc, Lisp_Object event)
   Widget menu;
   Lisp_Event *eev = NULL;
   XEvent xev;
-  Lisp_Object frame;
+  Lisp_Object frame = wrap_frame (f);
 
-  XSETFRAME (frame, f);
   CHECK_X_FRAME (frame);
   parent = FRAME_X_SHELL_WIDGET (f);
 
@@ -988,15 +986,13 @@ command_builder_operate_menu_accelerator (struct command_builder *builder)
   {
     int i;
     Lisp_Object t;
-    char buf[50];
 
     t = builder->current_events;
     i = 0;
     while (!NILP (t))
       {
 	i++;
-	sprintf (buf,"OPERATE (%d): ",i);
-	write_c_string (buf, Qexternal_debugging_output);
+	write_fmt_string (Qexternal_debugging_output, "OPERATE (%d): ",i);
 	print_internal (t, Qexternal_debugging_output, 1);
 	write_c_string ("\n", Qexternal_debugging_output);
 	t = XEVENT_NEXT (t);
@@ -1155,17 +1151,15 @@ menu_accelerator_junk_on_error (Lisp_Object errordata, Lisp_Object ignored)
   Vmenu_accelerator_enabled   = Qnil;
   if (!NILP (errordata))
     {
-      Lisp_Object args[2];
-
-      args[0] = build_string ("Error in menu accelerators (setting to nil)");
       /* #### This should call
 	 (with-output-to-string (display-error errordata))
 	 but that stuff is all in Lisp currently. */
-      args[1] = errordata;
       warn_when_safe_lispobj
 	(Qerror, Qwarning,
-	 emacs_doprnt_string_lisp ((const Intbyte *) "%s: %s",
-				   Qnil, -1, 2, args));
+	 emacs_sprintf_string_lisp
+	 ("%s: %s", Qnil, 2,
+	  build_msg_string ("Error in menu accelerators (setting to nil)"),
+	  errordata));
     }
 
   return Qnil;
