@@ -1,6 +1,7 @@
 /* Implements an elisp-programmable menubar.
    Copyright (C) 1993, 1994 Free Software Foundation, Inc.
    Copyright (C) 1995 Tinker Systems and INS Engineering Corp.
+   Copyright (C) 2001 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -39,6 +40,7 @@ Boston, MA 02111-1307, USA.  */
 #include "keymap.h"
 #include "menubar.h"
 #include "redisplay.h"
+#include "specifier.h"
 #include "window.h"
 
 int menubar_show_keybindings;
@@ -327,7 +329,6 @@ This removes %_'s (accelerator indications) and converts %% to %.
        (name, buffer))
 {
   struct buffer *buf = decode_buffer (buffer, 0);
-  Lisp_String *n;
   Charcount end;
   int i;
   Intbyte *name_data;
@@ -338,9 +339,8 @@ This removes %_'s (accelerator indications) and converts %% to %.
 
   CHECK_STRING (name);
 
-  n = XSTRING (name);
-  end = string_char_length (n);
-  name_data = string_data (n);
+  end = XSTRING_CHAR_LENGTH (name);
+  name_data = XSTRING_DATA (name);
 
   string_result = (Intbyte *) alloca (end * MAX_EMCHAR_LEN);
   string_result_ptr = string_result;
@@ -414,7 +414,7 @@ vars_of_menubar (void)
    * and this works fine.
    */
 
-  Vblank_menubar = list1 (list2 (build_string ("No active menubar"),
+  Vblank_menubar = list1 (list2 (build_msg_string ("No active menubar"),
 				 vector3 (build_string (""), Qnil, Qnil)));
   staticpro (&Vblank_menubar);
 
