@@ -1,6 +1,7 @@
 /* Minibuffer input and completion.
    Copyright (C) 1985, 1986, 1992-1995 Free Software Foundation, Inc.
    Copyright (C) 1995 Sun Microsystems, Inc.
+   Copyright (C) 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -187,7 +188,7 @@ Lowest-level interface to minibuffers.  Don't call this.
 
   val = call_command_loop (Qt);
 
-  return unbind_to (speccount, val);
+  return unbind_to_1 (speccount, val);
 }
 
 
@@ -214,8 +215,8 @@ scmp_1 (const Intbyte *s1, const Intbyte *s2, Charcount len,
     {
       while (l)
         {
-          Emchar c1 = DOWNCASE (current_buffer, charptr_emchar (s1));
-          Emchar c2 = DOWNCASE (current_buffer, charptr_emchar (s2));
+          Emchar c1 = DOWNCASE (0, charptr_emchar (s1));
+          Emchar c2 = DOWNCASE (0, charptr_emchar (s2));
 
           if (c1 == c2)
             {
@@ -400,7 +401,7 @@ or the symbol from the obarray.
 	      elt = bucket;
 	      eltstring = Fsymbol_name (elt);
               if (next)
-		XSETSYMBOL (bucket, next);
+		bucket = wrap_symbol (next);
 	      else
 		bucket = Qzero;
 	    }
@@ -601,7 +602,7 @@ or the symbol from the obarray.
 	      elt = bucket;
 	      eltstring = Fsymbol_name (elt);
               if (next)
-		XSETSYMBOL (bucket, next);
+		bucket = wrap_symbol (next);
 	      else
 		bucket = Qzero;
             }
@@ -621,8 +622,8 @@ or the symbol from the obarray.
           /* Reject alternatives that start with space
 	     unless the input starts with space.  */
 	  && ((XSTRING_CHAR_LENGTH (string) > 0 &&
-	       string_char (XSTRING (string), 0) == ' ')
-	      || string_char (XSTRING (eltstring), 0) != ' ')
+	       XSTRING_CHAR (string, 0) == ' ')
+	      || XSTRING_CHAR (eltstring, 0) != ' ')
 	  && (0 > scmp (XSTRING_DATA (eltstring),
                         XSTRING_DATA (string),
                         slength)))
@@ -680,16 +681,14 @@ clear_echo_area_internal (struct frame *f, Lisp_Object label, int from_print,
   /* This function can call lisp */
   if (!NILP (Ffboundp (Qclear_message)))
     {
-      Lisp_Object frame;
+      Lisp_Object frame = wrap_frame (f);
 
-      XSETFRAME (frame, f);
       return call4 (Qclear_message, label, frame, from_print ? Qt : Qnil,
 		    no_restore ? Qt : Qnil);
     }
   else
     {
-      write_string_to_stdio_stream (stderr, 0, (const Intbyte *) "\n", 0, 1,
-				    Qterminal, 0);
+      stderr_out ("\n");
       return Qnil;
     }
 }
@@ -747,7 +746,7 @@ echo_area_append (struct frame *f, const Intbyte *nonreloc, Lisp_Object reloc,
 	  obj = make_string (nonreloc + offset, length);
 	}
 
-      XSETFRAME (frame, f);
+      frame = wrap_frame (f);
       GCPRO1 (obj);
       call4 (Qappend_message, label, obj, frame,
 	     EQ (label, Qprint) ? Qt : Qnil);
@@ -757,8 +756,7 @@ echo_area_append (struct frame *f, const Intbyte *nonreloc, Lisp_Object reloc,
     {
       if (STRINGP (reloc))
 	nonreloc = XSTRING_DATA (reloc);
-      write_string_to_stdio_stream (stderr, 0, nonreloc, offset, length,
-				    Qterminal, 0);
+      write_string_1 (nonreloc + offset, length, Qexternal_debugging_output);
     }
 }
 
@@ -788,9 +786,8 @@ echo_area_status (struct frame *f)
   /* This function can call lisp */
   if (!NILP (Ffboundp (Qcurrent_message_label)))
     {
-      Lisp_Object frame;
+      Lisp_Object frame = wrap_frame (f);
 
-      XSETFRAME (frame, f);
       return call1 (Qcurrent_message_label, frame);
     }
   else
@@ -832,7 +829,7 @@ message_append_internal (const Intbyte *nonreloc, Lisp_Object reloc,
    on the format string; message_no_translate() does not. */
 
 static void
-message_1 (const char *fmt, va_list args)
+message_1 (const CIntbyte *fmt, va_list args)
 {
   /* This function can call lisp */
   if (fmt)
@@ -840,8 +837,7 @@ message_1 (const char *fmt, va_list args)
       struct gcpro gcpro1;
       /* message_internal() might GC, e.g. if there are after-change-hooks
 	 on the echo area buffer */
-      Lisp_Object obj = emacs_doprnt_string_va ((const Intbyte *) fmt, Qnil,
-						-1, args);
+      Lisp_Object obj = emacs_vsprintf_string (fmt, args);
       GCPRO1 (obj);
       message_internal (0, obj, 0, -1);
       UNGCPRO;
@@ -851,7 +847,7 @@ message_1 (const char *fmt, va_list args)
 }
 
 static void
-message_append_1 (const char *fmt, va_list args)
+message_append_1 (const CIntbyte *fmt, va_list args)
 {
   /* This function can call lisp */
   if (fmt)
@@ -859,8 +855,7 @@ message_append_1 (const char *fmt, va_list args)
       struct gcpro gcpro1;
       /* message_internal() might GC, e.g. if there are after-change-hooks
 	 on the echo area buffer */
-      Lisp_Object obj = emacs_doprnt_string_va ((const Intbyte *) fmt, Qnil,
-						-1, args);
+      Lisp_Object obj = emacs_vsprintf_string (fmt, args);
       GCPRO1 (obj);
       message_append_internal (0, obj, 0, -1);
       UNGCPRO;
