@@ -271,7 +271,7 @@ Boston, MA 02111-1307, USA.  */
 #include "sysdep.h"
 
 #include "wnn/commonhd.h"
-#include "mule-charset.h"
+#include "charset.h"
 #include "wnn/jllib.h"
 #include "wnn/cplib.h"
 
@@ -765,8 +765,7 @@ Get bunsetsu information specified by BUN-NUMBER.
   val = Fcons (make_string (cbuf, strlen ((char *) cbuf)), val);
   jl_get_kanji (wnnfns_buf[snum], bun_no, bun_no + 1, wbuf);
   w2m (wbuf, cbuf, lb);
-  val = Fcons (make_string (cbuf, strlen ((char *) cbuf)), val);
-  return val;
+  return Fcons (make_string (cbuf, strlen ((char *) cbuf)), val);
 }
 
 
@@ -1915,7 +1914,7 @@ Leading character for Sisheng.
 
   Vwnn_uniq_level = Qwnn_uniq;
 
-  Fprovide(intern("wnn"));
+  Fprovide (intern ("wnn"));
 }
 
 void
