@@ -1,7 +1,7 @@
 /* Generic Objects and Functions.
    Copyright (C) 1995 Free Software Foundation, Inc.
    Copyright (C) 1995 Board of Trustees, University of Illinois.
-   Copyright (C) 1995, 1996 Ben Wing.
+   Copyright (C) 1995, 1996, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -25,10 +25,12 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
+#include "buffer.h"
 #include "device.h"
 #include "elhash.h"
 #include "faces.h"
 #include "frame.h"
+#include "glyphs.h"
 #include "objects.h"
 #include "specifier.h"
 #include "window.h"
@@ -42,8 +44,8 @@ Lisp_Object Vthe_null_color_instance, Vthe_null_font_instance;
 void
 finalose (void *ptr)
 {
-  Lisp_Object obj;
-  XSETOBJ (obj, ptr);
+  Lisp_Object obj = wrap_pointer_1 (ptr);
+
 
   invalid_operation
     ("Can't dump an emacs containing window system objects", obj);
@@ -71,20 +73,16 @@ static void
 print_color_instance (Lisp_Object obj, Lisp_Object printcharfun,
 		      int escapeflag)
 {
-  char buf[100];
   Lisp_Color_Instance *c = XCOLOR_INSTANCE (obj);
   if (print_readably)
     printing_unreadable_object ("#<color-instance 0x%x>",
            c->header.uid);
-  write_c_string ("#<color-instance ", printcharfun);
-  print_internal (c->name, printcharfun, 0);
-  write_c_string (" on ", printcharfun);
-  print_internal (c->device, printcharfun, 0);
+  write_fmt_string_lisp (printcharfun, "#<color-instance %s", 1, c->name);
+  write_fmt_string_lisp (printcharfun, " on %s", 1, c->device);
   if (!NILP (c->device)) /* Vthe_null_color_instance */
     MAYBE_DEVMETH (XDEVICE (c->device), print_color_instance,
 		   (c, printcharfun, escapeflag));
-  sprintf (buf, " 0x%x>", c->header.uid);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, " 0x%x>", c->header.uid);
 }
 
 static void
@@ -150,11 +148,10 @@ is deallocated as well.
        (name, device, noerror))
 {
   Lisp_Color_Instance *c;
-  Lisp_Object val;
   int retval;
 
   CHECK_STRING (name);
-  XSETDEVICE (device, decode_device (device));
+  device = wrap_device (decode_device (device));
 
   c = alloc_lcrecord_type (Lisp_Color_Instance, &lrecord_color_instance);
   c->name = name;
@@ -167,8 +164,7 @@ is deallocated as well.
   if (!retval)
     return Qnil;
 
-  XSETCOLOR_INSTANCE (val, c);
-  return val;
+  return wrap_color_instance (c);
 }
 
 DEFUN ("color-instance-p", Fcolor_instance_p, 1, 1, 0, /*
@@ -251,19 +247,15 @@ mark_font_instance (Lisp_Object obj)
 static void
 print_font_instance (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
-  char buf[200];
   Lisp_Font_Instance *f = XFONT_INSTANCE (obj);
   if (print_readably)
     printing_unreadable_object ("#<font-instance 0x%x>", f->header.uid);
-  write_c_string ("#<font-instance ", printcharfun);
-  print_internal (f->name, printcharfun, 1);
-  write_c_string (" on ", printcharfun);
-  print_internal (f->device, printcharfun, 0);
+  write_fmt_string_lisp (printcharfun, "#<font-instance %S", 1, f->name);
+  write_fmt_string_lisp (printcharfun, " on %s", 1, f->device);
   if (!NILP (f->device))
     MAYBE_DEVMETH (XDEVICE (f->device), print_font_instance,
 		   (f, printcharfun, escapeflag));
-  sprintf (buf, " 0x%x>", f->header.uid);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, " 0x%x>", f->header.uid);
 }
 
 static void
@@ -286,15 +278,18 @@ static int
 font_instance_equal (Lisp_Object obj1, Lisp_Object obj2, int depth)
 {
   /* #### should this be moved into a device method? */
-  return internal_equal (font_instance_truename_internal (obj1, ERROR_ME_NOT),
-			 font_instance_truename_internal (obj2, ERROR_ME_NOT),
+  return internal_equal (font_instance_truename_internal
+			 (obj1, ERROR_ME_DEBUG_WARN),
+			 font_instance_truename_internal
+			 (obj2, ERROR_ME_DEBUG_WARN),
 			 depth + 1);
 }
 
 static unsigned long
 font_instance_hash (Lisp_Object obj, int depth)
 {
-  return internal_hash (font_instance_truename_internal (obj, ERROR_ME_NOT),
+  return internal_hash (font_instance_truename_internal
+			(obj, ERROR_ME_DEBUG_WARN),
 			depth + 1);
 }
 
@@ -318,7 +313,6 @@ these objects are GCed, the underlying X data is deallocated as well.
        (name, device, noerror))
 {
   Lisp_Font_Instance *f;
-  Lisp_Object val;
   int retval = 0;
   Error_Behavior errb = decode_error_behavior_flag (noerror);
 
@@ -327,7 +321,7 @@ these objects are GCed, the underlying X data is deallocated as well.
   else if (!STRINGP (name))
     return Qnil;
 
-  XSETDEVICE (device, decode_device (device));
+  device = wrap_device (decode_device (device));
 
   f = alloc_lcrecord_type (Lisp_Font_Instance, &lrecord_font_instance);
   f->name = name;
@@ -347,8 +341,7 @@ these objects are GCed, the underlying X data is deallocated as well.
   if (!retval)
     return Qnil;
 
-  XSETFONT_INSTANCE (val, f);
-  return val;
+  return wrap_font_instance (f);
 }
 
 DEFUN ("font-instance-p", Ffont_instance_p, 1, 1, 0, /*
@@ -467,7 +460,7 @@ currently selected device.
        (pattern, device))
 {
   CHECK_STRING (pattern);
-  XSETDEVICE (device, decode_device (device));
+  device = wrap_device (decode_device (device));
 
   return MAYBE_LISP_DEVMETH (XDEVICE (device), list_fonts, (pattern, device));
 }
@@ -1093,7 +1086,7 @@ reinit_vars_of_objects (void)
     c->device = Qnil;
     c->data = 0;
 
-    XSETCOLOR_INSTANCE (Vthe_null_color_instance, c);
+    Vthe_null_color_instance = wrap_color_instance (c);
   }
 
   staticpro_nodump (&Vthe_null_font_instance);
@@ -1109,7 +1102,7 @@ reinit_vars_of_objects (void)
     f->width = 0;
     f->proportional_p = 0;
 
-    XSETFONT_INSTANCE (Vthe_null_font_instance, f);
+    Vthe_null_font_instance = wrap_font_instance (f);
   }
 }
 
