@@ -1,6 +1,6 @@
 /* Lisp object printing and output streams.
    Copyright (C) 1985, 1986, 1988, 1992-1995 Free Software Foundation, Inc.
-   Copyright (C) 1995, 1996, 2000 Ben Wing.
+   Copyright (C) 1995, 1996, 2000, 2001, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -23,7 +23,14 @@ Boston, MA 02111-1307, USA.  */
 
 /* This file has been Mule-ized. */
 
-/* Seriously hacked on by Ben Wing for Mule. */
+/* Seriously divergent from FSF by this point.
+
+   Seriously hacked on by Ben Wing for Mule.  All stdio code also by Ben,
+   as well as the debugging code (initial version of debug_print(), though,
+   by Jamie Zawinski) and the _fmt interfaces.  Also a fair amount of work
+   by Hrvoje, e.g. floating-point code and rewriting to avoid O(N^2)
+   consing when outputting to the echo area.  Print-circularity code by
+   Martin? */
 
 #include <config.h>
 #include "lisp.h"
@@ -31,17 +38,21 @@ Boston, MA 02111-1307, USA.  */
 #include "backtrace.h"
 #include "buffer.h"
 #include "bytecode.h"
-#include "console-tty.h"
-#include "console-stream.h"
+#include "device.h"
 #include "extents.h"
 #include "frame.h"
 #include "insdel.h"
 #include "lstream.h"
-#include "sysfile.h"
+#include "opaque.h"
+
+#include "console-tty.h"
+#include "console-stream.h"
 #ifdef WIN32_NATIVE
 #include "console-msw.h"
 #endif
 
+#include "sysfile.h"
+
 #include <float.h>
 /* Define if not in float.h */
 #ifndef DBL_DIG
@@ -105,18 +116,39 @@ int print_unbuffered;
 
 FILE *termscript;	/* Stdio stream being used for copy of all output.  */
 
+static void write_string_to_alternate_debugging_output (Intbyte *str,
+							Bytecount len);
+
 
 
 int stdout_needs_newline;
 
+/* Basic function to actually write to a stdio stream or TTY console. */
+
 static void
-std_handle_out_external (FILE *stream, Lisp_Object lstream,
-			 const Extbyte *extptr, Bytecount extlen,
-			 /* is this really stdout/stderr?
-			    (controls termscript writing) */
-			 int output_is_std_handle,
-			 int must_flush)
+write_string_to_stdio_stream (FILE *stream, struct console *con,
+			      const Intbyte *ptr, Bytecount len,
+			      int must_flush)
 {
+  Extbyte *extptr = 0;
+  Bytecount extlen = 0;
+  int output_is_std_handle =
+    stream ? stream == stdout || stream == stderr :
+      CONSOLE_TTY_DATA (con)->is_stdio;
+
+  if (stream || output_is_std_handle)
+    {
+      if (initialized && !inhibit_non_essential_printing_operations)
+	TO_EXTERNAL_FORMAT (DATA, (ptr, len),
+			    ALLOCA, (extptr, extlen),
+			    Qterminal);
+      else
+	{
+	  extptr = (Extbyte *) ptr;
+	  extlen = (Bytecount) len;
+	}
+    }
+
   if (stream)
     {
 #ifdef WIN32_NATIVE
@@ -128,11 +160,11 @@ std_handle_out_external (FILE *stream, Lisp_Object lstream,
       /* we typically have no useful stdout/stderr under windows if we're
 	 being invoked graphically. */
       if (no_useful_stderr)
-	mswindows_output_console_string (extptr, extlen);
+	mswindows_output_console_string (ptr, len);
       else
 #endif
 	{
-	  fwrite (extptr, 1, extlen, stream);
+	  retry_fwrite (extptr, 1, extlen, stream);
 #ifdef WIN32_NATIVE
 	  /* Q122442 says that pipes are "treated as files, not as
 	     devices", and that this is a feature. Before I found that
@@ -145,21 +177,22 @@ std_handle_out_external (FILE *stream, Lisp_Object lstream,
 	}
     }
   else
-    Lstream_write (XLSTREAM (lstream), extptr, extlen);
+    /* The stream itself does conversion to external format */
+    Lstream_write (XLSTREAM (CONSOLE_TTY_DATA (con)->outstream), ptr, len);
 
   if (output_is_std_handle)
     {
       if (termscript)
 	{
-	  fwrite (extptr, 1, extlen, termscript);
+	  retry_fwrite (extptr, 1, extlen, termscript);
 	  fflush (termscript);
 	}
-      stdout_needs_newline = (extptr[extlen - 1] != '\n');
+      stdout_needs_newline = (ptr[extlen - 1] != '\n');
     }
 }
 
 /* #### The following function should be replaced a call to the
-   emacs_doprnt_*() functions.  This is the only way to ensure that
+   emacs_vsprintf_*() functions.  This is the only way to ensure that
    I18N3 works properly (many implementations of the *printf()
    functions, including the ones included in glibc, do not implement
    the %###$ argument-positioning syntax).
@@ -173,114 +206,100 @@ std_handle_out_external (FILE *stream, Lisp_Object lstream,
    2) (to be really correct) make a new lstream that outputs using
    mswindows_output_console_string().  */
 
-static int
-std_handle_out_va (FILE *stream, const char *fmt, va_list args)
+static void
+std_handle_out_va (FILE *stream, const CIntbyte *fmt, va_list args,
+		   int debug_output_as_well)
 {
   Intbyte kludge[8192];
-  Extbyte *extptr;
-  Bytecount extlen;
-  int retval;
+  Bytecount kludgelen;
 
-  retval = vsprintf ((char *) kludge, fmt, args);
-  if (initialized && !fatal_error_in_progress)
-    TO_EXTERNAL_FORMAT (DATA, (kludge, strlen ((char *) kludge)),
-			ALLOCA, (extptr, extlen),
-			Qnative);
-  else
+  if (initialized && !inhibit_non_essential_printing_operations)
+    fmt = GETTEXT (fmt);
+  vsprintf ((CIntbyte *) kludge, fmt, args);
+  kludgelen = qxestrlen (kludge);
+
+  write_string_to_stdio_stream (stream, 0, kludge, kludgelen, 1);
+  if (debug_output_as_well)
     {
-      extptr = (Extbyte *) kludge;
-      extlen = (Bytecount) strlen ((char *) kludge);
+      write_string_to_alternate_debugging_output (kludge, kludgelen);
+#ifdef WIN32_NATIVE
+      write_string_to_mswindows_debugging_output (kludge, kludgelen);
+#endif
     }
-
-  std_handle_out_external (stream, Qnil, extptr, extlen, 1, 1);
-  return retval;
 }
 
-/* Output portably to stderr or its equivalent; call GETTEXT on the
-   format string.  Automatically flush when done. */
+/* Output portably to stderr or its equivalent (i.e. may be a console
+   window under MS Windows); do external-format conversion and call GETTEXT
+   on the format string.  Automatically flush when done.
 
-int
-stderr_out (const char *fmt, ...)
+   This is safe even when not initialized or when dying -- we don't do
+   conversion in such cases. */
+
+void
+stderr_out (const CIntbyte *fmt, ...)
 {
-  int retval;
   va_list args;
   va_start (args, fmt);
-  retval =
-    std_handle_out_va
-    (stderr, initialized && !fatal_error_in_progress ? GETTEXT (fmt) : fmt,
-     args);
+  std_handle_out_va (stderr, fmt, args, 0);
   va_end (args);
-  return retval;
 }
 
-/* Output portably to stdout or its equivalent; call GETTEXT on the
-   format string.  Automatically flush when done. */
+/* Output portably to stdout or its equivalent (i.e. may be a console
+   window under MS Windows).  Works like stderr_out(). */
 
-int
-stdout_out (const char *fmt, ...)
+void
+stdout_out (const CIntbyte *fmt, ...)
 {
-  int retval;
   va_list args;
   va_start (args, fmt);
-  retval =
-    std_handle_out_va
-    (stdout, initialized && !fatal_error_in_progress ? GETTEXT (fmt) : fmt,
-     args);
+  std_handle_out_va (stdout, fmt, args, 0);
+  va_end (args);
+}
+
+/* Output portably to stderr or its equivalent (i.e. may be a console
+   window under MS Windows), as well as alternate-debugging-output and
+   (under MS Windows) the C debugging output, i.e. OutputDebugString().
+   Works like stderr_out(). */
+
+void
+debug_out (const CIntbyte *fmt, ...)
+{
+  va_list args;
+  va_start (args, fmt);
+  std_handle_out_va (stderr, fmt, args, 1);
   va_end (args);
-  return retval;
 }
 
 DOESNT_RETURN
-fatal (const char *fmt, ...)
+fatal (const CIntbyte *fmt, ...)
 {
   va_list args;
   va_start (args, fmt);
 
-  stderr_out ("\nXEmacs: ");
-  std_handle_out_va (stderr, GETTEXT (fmt), args);
+  stderr_out ("\nXEmacs: fatal error: ");
+  std_handle_out_va (stderr, fmt, args, 0);
   stderr_out ("\n");
 
   va_end (args);
   exit (1);
 }
 
-/* Write a string (in internal format) to stdio stream STREAM. */
-
-void
-write_string_to_stdio_stream (FILE *stream, struct console *con,
-			      const Intbyte *str,
-			      Bytecount offset, Bytecount len,
-			      Lisp_Object coding_system,
-			      int must_flush)
-{
-  Bytecount extlen;
-  const Extbyte *extptr;
-
-  /* #### yuck! sometimes this function is called with string data,
-     and the following call may gc. */
-  {
-    Intbyte *puta = (Intbyte *) alloca (len);
-    memcpy (puta, str + offset, len);
-    TO_EXTERNAL_FORMAT (DATA, (puta, len),
-			ALLOCA, (extptr, extlen),
-			coding_system);
-  }
-
-  if (stream)
-    std_handle_out_external (stream, Qnil, extptr, extlen,
-			     stream == stdout || stream == stderr, must_flush);
-  else
-    {
-      assert (CONSOLE_TTY_P (con));
-      std_handle_out_external (0, CONSOLE_TTY_DATA (con)->outstream,
-			       extptr, extlen,
-			       CONSOLE_TTY_DATA (con)->is_stdio, must_flush);
-    }
-}
-
 /* Write a string to the output location specified in FUNCTION.
    Arguments NONRELOC, RELOC, OFFSET, and LEN are as in
-   buffer_insert_string_1() in insdel.c. */
+   buffer_insert_string_1() in insdel.c.
+
+   FUNCTION is one of
+
+   -- an lstream
+   -- a buffer (insert at point and advance point)
+   -- a marker (insert at marker and advance marker)
+   -- a frame (append to echo area; clear echo area first if
+               `print-message-label' has changed since the last time)
+   -- t or nil (send to stdout)
+   -- a Lisp function of one argument (call to get data output)
+
+   Use Qexternal_debugging_output to get output to stderr.
+*/
 
 static void
 output_string (Lisp_Object function, const Intbyte *nonreloc,
@@ -297,7 +316,9 @@ output_string (Lisp_Object function, const Intbyte *nonreloc,
   struct gcpro gcpro1, gcpro2;
 
   /* Emacs won't print while GCing, but an external debugger might */
+#ifdef NO_PRINT_DURING_GC
   if (gc_in_progress) return;
+#endif
 
   /* Perhaps not necessary but probably safer. */
   GCPRO2 (function, reloc);
@@ -305,9 +326,12 @@ output_string (Lisp_Object function, const Intbyte *nonreloc,
   fixup_internal_substring (newnonreloc, reloc, offset, &len);
 
   if (STRINGP (reloc))
-    newnonreloc = XSTRING_DATA (reloc);
-
-  cclen = bytecount_to_charcount (newnonreloc + offset, len);
+    {
+      cclen = string_offset_byte_to_char_len (reloc, offset, len);
+      newnonreloc = XSTRING_DATA (reloc);
+    }
+  else
+    cclen = bytecount_to_charcount (newnonreloc + offset, len);
 
   if (LSTREAMP (function))
     {
@@ -325,12 +349,9 @@ output_string (Lisp_Object function, const Intbyte *nonreloc,
 	    }
 	  else
 	    {
-	      int speccount = specpdl_depth ();
-	      record_unwind_protect (restore_gc_inhibit,
-				     make_int (gc_currently_forbidden));
-	      gc_currently_forbidden = 1;
+	      int speccount = begin_gc_forbidden ();
 	      Lstream_write (XLSTREAM (function), newnonreloc + offset, len);
-	      unbind_to (speccount, Qnil);
+	      unbind_to (speccount);
 	    }
 	}
       else
@@ -368,20 +389,45 @@ output_string (Lisp_Object function, const Intbyte *nonreloc,
     }
   else if (EQ (function, Qt) || EQ (function, Qnil))
     {
-      write_string_to_stdio_stream (stdout, 0, newnonreloc, offset, len,
-				    Qterminal, print_unbuffered);
+      write_string_to_stdio_stream (stdout, 0, newnonreloc + offset, len,
+				    print_unbuffered);
+    }
+  else if (EQ (function, Qexternal_debugging_output))
+    {
+      /* This is not strictly necessary, and somewhat of a hack, but it
+	 avoids having each character passed separately to
+	 `external-debugging-output'. #### Why do we pass each character
+	 separately, anyway?
+	 */
+      write_string_to_stdio_stream (stderr, 0, newnonreloc + offset, len,
+				    print_unbuffered);
     }
   else
     {
-      Charcount ccoff = bytecount_to_charcount (newnonreloc, offset);
+      Charcount ccoff;
       Charcount iii;
 
-      for (iii = ccoff; iii < cclen + ccoff; iii++)
+      if (STRINGP (reloc))
+	ccoff = string_index_byte_to_char (reloc, offset);
+      else
+	ccoff = bytecount_to_charcount (newnonreloc, offset);
+
+      if (STRINGP (reloc))
 	{
-	  call1 (function,
-		 make_char (charptr_emchar_n (newnonreloc, iii)));
-	  if (STRINGP (reloc))
-	    newnonreloc = XSTRING_DATA (reloc);
+	  for (iii = ccoff; iii < cclen + ccoff; iii++)
+	    {
+	      call1 (function, make_char (XSTRING_CHAR (reloc, iii)));
+	      if (STRINGP (reloc))
+		newnonreloc = XSTRING_DATA (reloc);
+	    }
+	}
+      else
+	{
+	  for (iii = ccoff; iii < cclen + ccoff; iii++)
+	    {
+	      call1 (function,
+		     make_char (charptr_emchar_n (newnonreloc, iii)));
+	    }
 	}
     }
 
@@ -409,9 +455,11 @@ static Lisp_Object
 print_prepare (Lisp_Object printcharfun, Lisp_Object *frame_kludge)
 {
   /* Emacs won't print while GCing, but an external debugger might */
+#ifdef NO_PRINT_DURING_GC
   if (gc_in_progress)
     return Qnil;
-
+#endif
+  
   RESET_PRINT_GENSYM;
 
   printcharfun = canonicalize_printcharfun (printcharfun);
@@ -454,9 +502,11 @@ static void
 print_finish (Lisp_Object stream, Lisp_Object frame_kludge)
 {
   /* Emacs won't print while GCing, but an external debugger might */
+#ifdef NO_PRINT_DURING_GC
   if (gc_in_progress)
     return;
-
+#endif
+  
   RESET_PRINT_GENSYM;
 
   /* See the comment in print_prepare().  */
@@ -481,7 +531,10 @@ print_finish (Lisp_Object stream, Lisp_Object frame_kludge)
   output_string (stream, (const Intbyte *) (string_of_length_1),	\
 		 Qnil, 0, 1)
 
-/* NOTE: Do not call this with the data of a Lisp_String, as
+/* Write internal-format data to STREAM.  See output_string() for
+   interpretation of STREAM.
+
+   NOTE: Do not call this with the data of a Lisp_String, as
    printcharfun might cause a GC, which might cause the string's data
    to be relocated.  To princ a Lisp string, use:
 
@@ -494,19 +547,96 @@ void
 write_string_1 (const Intbyte *str, Bytecount size, Lisp_Object stream)
 {
   /* This function can GC */
-#ifdef ERROR_CHECK_CHARBPOS
+#ifdef ERROR_CHECK_TEXT
   assert (size >= 0);
 #endif
   output_string (stream, str, Qnil, 0, size);
 }
 
 void
-write_c_string (const char *str, Lisp_Object stream)
+write_string (const Intbyte *str, Lisp_Object stream)
+{
+  /* This function can GC */
+  write_string_1 (str, qxestrlen (str), stream);
+}
+
+void
+write_c_string (const CIntbyte *str, Lisp_Object stream)
 {
   /* This function can GC */
   write_string_1 ((const Intbyte *) str, strlen (str), stream);
 }
 
+void
+write_eistring (const Eistring *ei, Lisp_Object stream)
+{
+  write_string_1 (eidata (ei), eilen (ei), stream);
+}
+
+/* Write a printf-style string to STREAM; see output_string(). */
+
+void
+write_fmt_string (Lisp_Object stream, const CIntbyte *fmt, ...)
+{
+  va_list va;
+  Intbyte *str;
+  Bytecount len;
+  int count;
+
+  va_start (va, fmt);
+  str = emacs_vsprintf_malloc (fmt, va, &len);
+  va_end (va);
+  count = record_unwind_protect_freeing (str);
+  write_string_1 (str, len, stream);
+  unbind_to (count);
+}
+
+/* Write a printf-style string to STREAM, where the arguments are
+   Lisp objects and not C strings or integers; see output_string().
+
+   #### It shouldn't be necessary to specify the number of arguments.
+   This would require some rewriting of the doprnt() functions, though. */
+
+void
+write_fmt_string_lisp (Lisp_Object stream, const CIntbyte *fmt, int nargs, ...)
+{
+  Lisp_Object *args = alloca_array (Lisp_Object, nargs);
+  va_list va;
+  int i;
+  Intbyte *str;
+  Bytecount len;
+  int count;
+
+  va_start (va, nargs);
+  for (i = 0; i < nargs; i++)
+    args[i] = va_arg (va, Lisp_Object);
+  va_end (va);
+  str = emacs_vsprintf_malloc_lisp (fmt, Qnil, nargs, args, &len);
+  count = record_unwind_protect_freeing (str);
+  write_string_1 (str, len, stream);
+  unbind_to (count);
+}
+
+void
+stderr_out_lisp (const CIntbyte *fmt, int nargs, ...)
+{
+  Lisp_Object *args = alloca_array (Lisp_Object, nargs);
+  va_list va;
+  int i;
+  Intbyte *str;
+  Bytecount len;
+  int count;
+
+  va_start (va, nargs);
+  for (i = 0; i < nargs; i++)
+    args[i] = va_arg (va, Lisp_Object);
+  va_end (va);
+  str = emacs_vsprintf_malloc_lisp (fmt, Qnil, nargs, args, &len);
+  count = record_unwind_protect_freeing (str);
+  write_string_1 (str, len, Qexternal_debugging_output);
+  unbind_to (count);
+}
+
 
 DEFUN ("write-char", Fwrite_char, 1, 2, 0, /*
 Output character CHARACTER to stream STREAM.
@@ -541,7 +671,7 @@ temp_output_buffer_setup (Lisp_Object bufname)
   current_buffer->read_only = Qnil;
   Ferase_buffer (Qnil);
 
-  XSETBUFFER (buf, current_buffer);
+  buf = wrap_buffer (current_buffer);
   specbind (Qstandard_output, buf);
 
   set_buffer_internal (old);
@@ -567,7 +697,7 @@ internal_with_output_to_temp_buffer (Lisp_Object bufname,
   temp_output_buffer_show (buf, same_frame);
   UNGCPRO;
 
-  return unbind_to (speccount, arg);
+  return unbind_to_1 (speccount, arg);
 }
 
 DEFUN ("with-output-to-temp-buffer", Fwith_output_to_temp_buffer, 1, UNEVALLED, 0, /*
@@ -605,7 +735,7 @@ to get the buffer displayed.  It gets one argument, the buffer to display.
 
   temp_output_buffer_show (Vstandard_output, Qnil);
 
-  return unbind_to (speccount, val);
+  return unbind_to_1 (speccount, val);
 }
 
 DEFUN ("terpri", Fterpri, 0, 1, 0, /*
@@ -632,7 +762,6 @@ Output stream is STREAM, or value of `standard-output' (which see).
   struct gcpro gcpro1, gcpro2;
   GCPRO2 (object, stream);
 
-  print_depth = 0;
   stream = print_prepare (stream, &frame);
   print_internal (object, stream, 1);
   print_finish (stream, frame);
@@ -657,7 +786,6 @@ second argument NOESCAPE is non-nil.
   struct gcpro gcpro1, gcpro2, gcpro3;
   GCPRO3 (object, stream, result);
 
-  print_depth = 0;
   RESET_PRINT_GENSYM;
   print_internal (object, stream, NILP (noescape));
   RESET_PRINT_GENSYM;
@@ -683,7 +811,6 @@ Output stream is STREAM, or value of `standard-output' (which see).
 
   GCPRO2 (object, stream);
   stream = print_prepare (stream, &frame);
-  print_depth = 0;
   print_internal (object, stream, 0);
   print_finish (stream, frame);
   UNGCPRO;
@@ -704,7 +831,6 @@ Output stream is STREAM, or value of `standard-output' (which see).
 
   GCPRO2 (object, stream);
   stream = print_prepare (stream, &frame);
-  print_depth = 0;
   write_char_internal ("\n", stream);
   print_internal (object, stream, 1);
   write_char_internal ("\n", stream);
@@ -795,7 +921,7 @@ print_error_message (Lisp_Object error_object, Lisp_Object stream)
       }
     print_finish (stream, frame);
     UNGCPRO;
-    unbind_to (speccount, Qnil);
+    unbind_to (speccount);
     return;
     /* not reached */
   }
@@ -1157,12 +1283,11 @@ print_vector (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 void
 print_string (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
-  Lisp_String *s = XSTRING (obj);
   /* We distinguish between Bytecounts and Charcounts, to make
      Vprint_string_length work correctly under Mule.  */
-  Charcount size = string_char_length (s);
+  Charcount size = XSTRING_CHAR_LENGTH (obj);
   Charcount max = size;
-  Bytecount bcmax = string_length (s);
+  Bytecount bcmax = XSTRING_LENGTH (obj);
   struct gcpro gcpro1, gcpro2;
   GCPRO2 (obj, printcharfun);
 
@@ -1170,7 +1295,7 @@ print_string (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
       XINT (Vprint_string_length) < max)
     {
       max = XINT (Vprint_string_length);
-      bcmax = charcount_to_bytecount (string_data (s), max);
+      bcmax = string_index_char_to_byte (obj, max);
     }
   if (max < 0)
     {
@@ -1192,7 +1317,7 @@ print_string (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
       write_char_internal ("\"", printcharfun);
       for (i = 0; i < bcmax; i++)
 	{
-	  Intbyte ch = string_byte (s, i);
+	  Intbyte ch = XSTRING_BYTE (obj, i);
 	  if (ch == '\"' || ch == '\\'
 	      || (ch == '\n' && print_escape_newlines))
 	    {
@@ -1210,7 +1335,7 @@ print_string (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 		  write_char_internal ("\\", printcharfun);
 		  /* This is correct for Mule because the
 		     character is either \ or " */
-		  write_char_internal (string_data (s) + i, printcharfun);
+		  write_char_internal (XSTRING_DATA (obj) + i, printcharfun);
 		}
 	      last = i + 1;
 	    }
@@ -1233,7 +1358,6 @@ default_object_printer (Lisp_Object obj, Lisp_Object printcharfun,
 {
   struct lcrecord_header *header =
     (struct lcrecord_header *) XPNTR (obj);
-  char buf[200];
 
   if (print_readably)
     printing_unreadable_object
@@ -1241,32 +1365,89 @@ default_object_printer (Lisp_Object obj, Lisp_Object printcharfun,
        LHEADER_IMPLEMENTATION (&header->lheader)->name,
        header->uid);
 
-  sprintf (buf, "#<%s 0x%x>",
-	   LHEADER_IMPLEMENTATION (&header->lheader)->name,
-	   header->uid);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, "#<%s 0x%x>",
+		    LHEADER_IMPLEMENTATION (&header->lheader)->name,
+		    header->uid);
 }
 
 void
 internal_object_printer (Lisp_Object obj, Lisp_Object printcharfun,
 			 int escapeflag)
 {
-  char buf[200];
-  sprintf (buf, "#<INTERNAL OBJECT (XEmacs bug?) (%s) 0x%lx>",
-	   XRECORD_LHEADER_IMPLEMENTATION (obj)->name,
-	   (unsigned long) XPNTR (obj));
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun,
+		    "#<INTERNAL OBJECT (XEmacs bug?) (%s) 0x%lx>",
+		    XRECORD_LHEADER_IMPLEMENTATION (obj)->name,
+		    (unsigned long) XPNTR (obj));
 }
 
 void
 print_internal (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   /* This function can GC */
+  int specdepth;
 
   QUIT;
 
+#ifdef NO_PRINT_DURING_GC
   /* Emacs won't print while GCing, but an external debugger might */
   if (gc_in_progress) return;
+#endif
+
+  /* Try to check for a bogus pointer if we're in a situation where it may
+     be likely.  In such cases, crashing is counterproductive. */
+  if (inhibit_non_essential_printing_operations || print_unbuffered)
+    {
+      if (XTYPE (obj) == Lisp_Type_Record)
+	{
+	  struct lrecord_header *lheader = XRECORD_LHEADER (obj);
+      
+	  if (!debug_can_access_memory (lheader, sizeof (*lheader)))
+	    {
+	      write_fmt_string (printcharfun, "#<EMACS BUG: BAD MEMORY %p>",
+				lheader);
+	      return;
+	    }
+	  else
+	    {
+	      const struct lrecord_implementation *impl;
+
+	      if ((int) lheader->type >= lrecord_type_count)
+		{
+		  write_fmt_string (printcharfun,
+				    "#<EMACS BUG: bad type %d BAD MEMORY %p>",
+				    lheader->type, lheader);
+		  return;
+		}
+
+	      impl = LHEADER_IMPLEMENTATION (lheader);
+	      if (!debug_can_access_memory
+		  (lheader,
+		   (impl->size_in_bytes_method ?
+		    impl->size_in_bytes_method (lheader) :
+		    impl->static_size)))
+		{
+		  write_fmt_string (printcharfun,
+				    "#<EMACS BUG: type %s BAD MEMORY %p>",
+				    impl->name, lheader);
+		  return;
+		}
+
+	      if (STRINGP (obj))
+		{
+		  Lisp_String *l = (Lisp_String *) lheader;
+		  if (!debug_can_access_memory
+		      (l->data_, l->size_))
+		    {
+		      write_fmt_string
+			(printcharfun,
+			 "#<EMACS BUG: %p (BAD STRING DATA %p)>",
+			 lheader, l->data_);
+		      return;
+		    }
+		}
+	    }
+	}
+    }
 
 #ifdef I18N3
   /* #### Both input and output streams should have a flag associated
@@ -1297,7 +1478,7 @@ print_internal (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
     }
 
   being_printed[print_depth] = obj;
-  print_depth++;
+  specdepth = internal_bind_int (&print_depth, print_depth + 1);
 
   if (print_depth > PRINT_CIRCLE)
     signal_error (Qstack_overflow, "Apparently circular structure being printed", Qunbound);
@@ -1408,27 +1589,25 @@ print_internal (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 
     default:
       {
-#ifdef ERROR_CHECK_TYPECHECK
+#ifdef ERROR_CHECK_TYPES
 	abort ();
-#else  /* not ERROR_CHECK_TYPECHECK */
-	char buf[128];
+#else  /* not ERROR_CHECK_TYPES */
 	/* We're in trouble if this happens! */
 	if (print_readably)
 	  signal_error (Qinternal_error, "printing illegal data type #o%03o",
 			make_int (XTYPE (obj)));
 	write_c_string ("#<EMACS BUG: ILLEGAL DATATYPE ",
 			printcharfun);
-	sprintf (buf, "(#o%3o)", (int) XTYPE (obj));
-	write_c_string (buf, printcharfun);
+	write_fmt_string (printcharfun, "(#o%3o)", (int) XTYPE (obj));
 	write_c_string
 	  (" Save your buffers immediately and please report this bug>",
 	   printcharfun);
-#endif /* not ERROR_CHECK_TYPECHECK */
+#endif /* not ERROR_CHECK_TYPES */
 	break;
       }
     }
 
-  print_depth--;
+  unbind_to (specdepth);
 }
 
 
@@ -1449,16 +1628,14 @@ print_symbol (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
   /* This function can GC */
   /* #### Bug!! (intern "") isn't printed in some distinguished way */
   /* ####  (the reader also loses on it) */
-  Lisp_String *name = symbol_name (XSYMBOL (obj));
-  Bytecount size = string_length (name);
+  Lisp_Object name = symbol_name (XSYMBOL (obj));
+  Bytecount size = XSTRING_LENGTH (name);
   struct gcpro gcpro1, gcpro2;
 
   if (!escapeflag)
     {
       /* This deals with GC-relocation */
-      Lisp_Object nameobj;
-      XSETSTRING (nameobj, name);
-      output_string (printcharfun, 0, nameobj, 0, size);
+      output_string (printcharfun, 0, name, 0, size);
       return;
     }
   GCPRO2 (obj, printcharfun);
@@ -1470,8 +1647,8 @@ print_symbol (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
       /* #### Test whether this produces a noticeable slow-down for
          printing when print-gensym is non-nil.  */
       && !EQ (obj, oblookup (Vobarray,
-			     string_data (symbol_name (XSYMBOL (obj))),
-			     string_length (symbol_name (XSYMBOL (obj))))))
+			     XSTRING_DATA (symbol_name (XSYMBOL (obj))),
+			     XSTRING_LENGTH (symbol_name (XSYMBOL (obj))))))
     {
       if (print_depth > 1)
 	{
@@ -1492,10 +1669,10 @@ print_symbol (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
                      have to be careful.  */
 		  CHECK_CONS (XCAR (Vprint_gensym_alist));
 		  CHECK_INT (XCDR (XCAR (Vprint_gensym_alist)));
-		  XSETINT (tem, XINT (XCDR (XCAR (Vprint_gensym_alist))) + 1);
+		  tem = make_int (XINT (XCDR (XCAR (Vprint_gensym_alist))) + 1);
 		}
 	      else
-		XSETINT (tem, 1);
+		tem = make_int (1);
 	      Vprint_gensym_alist = Fcons (Fcons (obj, tem), Vprint_gensym_alist);
 
 	      write_char_internal ("#", printcharfun);
@@ -1508,7 +1685,7 @@ print_symbol (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 
   /* Does it look like an integer or a float? */
   {
-    Intbyte *data = string_data (name);
+    Intbyte *data = XSTRING_DATA (name);
     Bytecount confusing = 0;
 
     if (size == 0)
@@ -1544,14 +1721,12 @@ print_symbol (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
   }
 
   {
-    Lisp_Object nameobj;
     Bytecount i;
     Bytecount last = 0;
 
-    XSETSTRING (nameobj, name);
     for (i = 0; i < size; i++)
       {
-	switch (string_byte (name, i))
+	switch (XSTRING_BYTE (name, i))
 	  {
 	  case  0: case  1: case  2: case  3:
 	  case  4: case  5: case  6: case  7:
@@ -1566,12 +1741,12 @@ print_symbol (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 	  case ',': case '.' : case '`' :
 	  case '[': case ']' : case '?' :
 	    if (i > last)
-	      output_string (printcharfun, 0, nameobj, last, i - last);
+	      output_string (printcharfun, 0, name, last, i - last);
 	    write_char_internal ("\\", printcharfun);
 	    last = i;
 	  }
       }
-    output_string (printcharfun, 0, nameobj, last, size - last);
+    output_string (printcharfun, 0, name, last, size - last);
   }
   UNGCPRO;
 }
@@ -1593,18 +1768,33 @@ to 0.
 {
   Intbyte str[MAX_EMCHAR_LEN];
   Bytecount len;
-  int extlen;
-  const Extbyte *extptr;
 
   CHECK_CHAR_COERCE_INT (character);
   len = set_charptr_emchar (str, XCHAR (character));
-  TO_EXTERNAL_FORMAT (DATA, (str, len),
-		      ALLOCA, (extptr, extlen),
-		      Qterminal);
+  write_string_to_alternate_debugging_output (str, len);
+  
+  return character;
+}
+
+static void
+write_string_to_alternate_debugging_output (Intbyte *str, Bytecount len)
+{
+  int extlen;
+  const Extbyte *extptr;
+#if 0 /* We want to see the internal representation, don't we? */
+  if (initialized && !inhibit_non_essential_printing_operations)
+    TO_EXTERNAL_FORMAT (DATA, (str, len),
+			ALLOCA, (extptr, extlen),
+			Qterminal);
+  else
+#endif /* 0 */
+    {
+      extlen = len;
+      extptr = (Extbyte *) str;
+    }
   memcpy (alternate_do_string + alternate_do_pointer, extptr, extlen);
   alternate_do_pointer += extlen;
   alternate_do_string[alternate_do_pointer] = 0;
-  return character;
 }
 
 DEFUN ("external-debugging-output", Fexternal_debugging_output, 1, 3, 0, /*
@@ -1650,8 +1840,8 @@ the output also will be logged to this file.
   if (STRINGP (char_or_string))
     write_string_to_stdio_stream (file, con,
 				  XSTRING_DATA (char_or_string),
-				  0, XSTRING_LENGTH (char_or_string),
-				  Qterminal, 1);
+				  XSTRING_LENGTH (char_or_string),
+				  print_unbuffered);
   else
     {
       Intbyte str[MAX_EMCHAR_LEN];
@@ -1659,7 +1849,7 @@ the output also will be logged to this file.
 
       CHECK_CHAR_COERCE_INT (char_or_string);
       len = set_charptr_emchar (str, XCHAR (char_or_string));
-      write_string_to_stdio_stream (file, con, str, 0, len, Qterminal, 1);
+      write_string_to_stdio_stream (file, con, str, len, print_unbuffered);
     }
 
   return char_or_string;
@@ -1674,50 +1864,40 @@ FILENAME = nil means just close any termscript file currently open.
   /* This function can GC */
   if (termscript != 0)
     {
-      fclose (termscript);
+      retry_fclose (termscript);
       termscript = 0;
     }
 
   if (! NILP (filename))
     {
       filename = Fexpand_file_name (filename, Qnil);
-      termscript = fopen ((char *) XSTRING_DATA (filename), "w");
+      termscript = qxe_fopen (XSTRING_DATA (filename), "w");
       if (termscript == NULL)
 	report_file_error ("Opening termscript", filename);
     }
   return Qnil;
 }
 
-#if 1
-/* Debugging kludge -- unbuffered */
 static int debug_print_length   = 50;
 static int debug_print_level    = 15;
 static int debug_print_readably = -1;
 
+/* Debugging kludge -- unbuffered */
 static void
 debug_print_no_newline (Lisp_Object debug_print_obj)
 {
   /* This function can GC */
-  int save_print_readably = print_readably;
-  int save_print_depth    = print_depth;
-  Lisp_Object save_Vprint_length = Vprint_length;
-  Lisp_Object save_Vprint_level  = Vprint_level;
-  Lisp_Object save_Vinhibit_quit = Vinhibit_quit;
-  struct gcpro gcpro1, gcpro2, gcpro3;
-  GCPRO3 (save_Vprint_level, save_Vprint_length, save_Vinhibit_quit);
-
-  if (gc_in_progress)
-    stderr_out ("** gc-in-progress!  Bad idea to print anything! **\n");
-
-  print_depth = 0;
-  print_readably = debug_print_readably != -1 ? debug_print_readably : 0;
-  print_unbuffered++;
-  /* Could use unwind-protect, but why bother? */
+  int specdepth = internal_bind_int (&print_depth, 0);
+  internal_bind_int (&print_readably,
+		     debug_print_readably != -1 ? debug_print_readably : 0);
+  internal_bind_int (&print_unbuffered, print_unbuffered + 1);
   if (debug_print_length > 0)
-    Vprint_length = make_int (debug_print_length);
+    internal_bind_lisp_object (&Vprint_length, make_int (debug_print_length));
   if (debug_print_level > 0)
-    Vprint_level = make_int (debug_print_level);
-
+    internal_bind_lisp_object (&Vprint_level, make_int (debug_print_level));
+  /* #### Do we need this?  It was in the old code. */
+  internal_bind_lisp_object (&Vinhibit_quit, Vinhibit_quit);
+  
   print_internal (debug_print_obj, Qexternal_debugging_output, 1);
   alternate_do_pointer = 0;
   print_internal (debug_print_obj, Qalternate_debugging_output, 1);
@@ -1726,61 +1906,36 @@ debug_print_no_newline (Lisp_Object debug_print_obj)
   print_internal (debug_print_obj, Qmswindows_debugging_output, 1);
 #endif
 
-  Vinhibit_quit  = save_Vinhibit_quit;
-  Vprint_level   = save_Vprint_level;
-  Vprint_length  = save_Vprint_length;
-  print_depth    = save_print_depth;
-  print_readably = save_print_readably;
-  print_unbuffered--;
-  UNGCPRO;
+  unbind_to (specdepth);
 }
 
 void
 debug_print (Lisp_Object debug_print_obj)
 {
   debug_print_no_newline (debug_print_obj);
-  stderr_out ("\n");
+  debug_out ("\n");
 }
 
 /* Debugging kludge -- unbuffered */
 /* This function provided for the benefit of the debugger.  */
-void debug_backtrace (void);
 void
 debug_backtrace (void)
 {
   /* This function can GC */
-  int         old_print_readably = print_readably;
-  int         old_print_depth    = print_depth;
-  Lisp_Object old_print_length   = Vprint_length;
-  Lisp_Object old_print_level    = Vprint_level;
-  Lisp_Object old_inhibit_quit   = Vinhibit_quit;
-
-  struct gcpro gcpro1, gcpro2, gcpro3;
-  GCPRO3 (old_print_level, old_print_length, old_inhibit_quit);
-
-  if (gc_in_progress)
-    stderr_out ("** gc-in-progress!  Bad idea to print anything! **\n");
-
-  print_depth = 0;
-  print_readably = 0;
-  print_unbuffered++;
-  /* Could use unwind-protect, but why bother? */
+  int specdepth = internal_bind_int (&print_depth, 0);
+  internal_bind_int (&print_readably, 0);
+  internal_bind_int (&print_unbuffered, print_unbuffered + 1);
   if (debug_print_length > 0)
-    Vprint_length = make_int (debug_print_length);
+    internal_bind_lisp_object (&Vprint_length, make_int (debug_print_length));
   if (debug_print_level > 0)
-    Vprint_level = make_int (debug_print_level);
+    internal_bind_lisp_object (&Vprint_level, make_int (debug_print_level));
+  /* #### Do we need this?  It was in the old code. */
+  internal_bind_lisp_object (&Vinhibit_quit, Vinhibit_quit);
 
   Fbacktrace (Qexternal_debugging_output, Qt);
   stderr_out ("\n");
 
-  Vinhibit_quit  = old_inhibit_quit;
-  Vprint_level   = old_print_level;
-  Vprint_length  = old_print_length;
-  print_depth    = old_print_depth;
-  print_readably = old_print_readably;
-  print_unbuffered--;
-
-  UNGCPRO;
+  unbind_to (specdepth);
 }
 
 void
@@ -1788,12 +1943,12 @@ debug_short_backtrace (int length)
 {
   int first = 1;
   struct backtrace *bt = backtrace_list;
-  stderr_out ("   [");
+  debug_out ("   [");
   while (length > 0 && bt)
     {
       if (!first)
 	{
-	  stderr_out (", ");
+	  debug_out (", ");
 	}
       if (COMPILED_FUNCTIONP (*bt->function))
 	{
@@ -1805,13 +1960,13 @@ debug_short_backtrace (int length)
 #endif
 	  if (!NILP (ann))
 	    {
-	      stderr_out ("<compiled-function from ");
+	      debug_out ("<compiled-function from ");
 	      debug_print_no_newline (ann);
-	      stderr_out (">");
+	      debug_out (">");
 	    }
 	  else
 	    {
-	      stderr_out ("<compiled-function of unknown origin>");
+	      debug_out ("<compiled-function of unknown origin>");
 	    }
 	}
       else
@@ -1820,11 +1975,9 @@ debug_short_backtrace (int length)
       length--;
       bt = bt->next;
     }
-  stderr_out ("]\n");
+  debug_out ("]\n");
 }
 
-#endif /* debugging kludge */
-
 
 void
 syms_of_print (void)
