@@ -24,7 +24,6 @@ Boston, MA 02111-1307, USA.  */
 #if defined (NO_SUBPROCESSES)
 #undef XPROCESS
 #undef CHECK_PROCESS
-#undef XSETPROCESS
 #define PROCESSP(x) 0
 #define PROCESS_LIVE_P(x) 0
 #define Fprocess_status(x) Qnil
@@ -42,7 +41,6 @@ void wait_without_blocking (void);
 
 DECLARE_LRECORD (process, Lisp_Process);
 #define XPROCESS(x) XRECORD (x, process, Lisp_Process)
-#define XSETPROCESS(x, p) XSETRECORD (x, p, process)
 #define wrap_process(p) wrap_record (p, process)
 #define PROCESSP(x) RECORDP (x, process)
 #define CHECK_PROCESS(x) CHECK_RECORD (x, process)
@@ -59,6 +57,8 @@ DECLARE_LRECORD (process, Lisp_Process);
 EXFUN (Fprocess_kill_without_query, 2);
 EXFUN (Fprocess_id, 1);
 
+extern Lisp_Object Qprocess_live_p;
+
 Lisp_Object connect_to_file_descriptor (Lisp_Object name,
 					Lisp_Object buffer,
 					Lisp_Object infd,
@@ -116,7 +116,7 @@ int
 void
 #endif
 child_setup (int in, int out, int err,
-		  char **new_argv, const char *current_dir);
+	     Intbyte **new_argv, Lisp_Object current_dir);
 
 Charcount read_process_output (Lisp_Object proc);
 
