@@ -1,6 +1,6 @@
 /* XEmacs routines to deal with range tables.
    Copyright (C) 1995 Sun Microsystems, Inc.
-   Copyright (C) 1995 Ben Wing.
+   Copyright (C) 1995, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -55,7 +55,6 @@ static void
 print_range_table (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
   Lisp_Range_Table *rt = XRANGE_TABLE (obj);
-  char buf[200];
   int i;
 
   write_c_string ("#s(range-table data (", printcharfun);
@@ -65,10 +64,10 @@ print_range_table (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
       if (i > 0)
 	write_c_string (" ", printcharfun);
       if (rte->first == rte->last)
-	sprintf (buf, "%ld ", (long) (rte->first));
+	write_fmt_string (printcharfun, "%ld ", (long) (rte->first));
       else
-	sprintf (buf, "(%ld %ld) ", (long) (rte->first), (long) (rte->last));
-      write_c_string (buf, printcharfun);
+	write_fmt_string (printcharfun, "(%ld %ld) ", (long) (rte->first),
+			  (long) (rte->last));
       print_internal (rte->val, printcharfun, 1);
     }
   write_c_string ("))", printcharfun);
@@ -167,7 +166,7 @@ DEFINE_LRECORD_IMPLEMENTATION ("range-table", range_table,
 /*                        Range table operations                        */
 /************************************************************************/
 
-#ifdef ERROR_CHECK_TYPECHECK
+#ifdef ERROR_CHECK_STRUCTURES
 
 static void
 verify_range_table (Lisp_Range_Table *rt)
@@ -232,12 +231,10 @@ You can manipulate it using `put-range-table', `get-range-table',
 */
        ())
 {
-  Lisp_Object obj;
   Lisp_Range_Table *rt = alloc_lcrecord_type (Lisp_Range_Table,
 					      &lrecord_range_table);
   rt->entries = Dynarr_new (range_table_entry);
-  XSETRANGE_TABLE (obj, rt);
-  return obj;
+  return wrap_range_table (rt);
 }
 
 DEFUN ("copy-range-table", Fcopy_range_table, 1, 1, 0, /*
@@ -248,7 +245,6 @@ The values will not themselves be copied.
        (range_table))
 {
   Lisp_Range_Table *rt, *rtnew;
-  Lisp_Object obj;
 
   CHECK_RANGE_TABLE (range_table);
   rt = XRANGE_TABLE (range_table);
@@ -258,8 +254,7 @@ The values will not themselves be copied.
 
   Dynarr_add_many (rtnew->entries, Dynarr_atp (rt->entries, 0),
 		   Dynarr_length (rt->entries));
-  XSETRANGE_TABLE (obj, rtnew);
-  return obj;
+  return wrap_range_table (rtnew);
 }
 
 DEFUN ("get-range-table", Fget_range_table, 2, 3, 0, /*
