@@ -1,7 +1,7 @@
 /* Communication module for TTY terminals.
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
    Copyright (C) 1995 Sun Microsystems, Inc.
-   Copyright (C) 1995, 1996 Ben Wing.
+   Copyright (C) 1995, 1996, 2002 Ben Wing.
    Copyright (C) 1996 Chuck Thompson.
 
 This file is part of XEmacs.
@@ -33,17 +33,19 @@ Boston, MA 02111-1307, USA.  */
 #include "lisp.h"
 
 #include "buffer.h"
-#include "console-tty.h"
+#include "device.h"
 #include "events.h"
 #include "faces.h"
 #include "frame.h"
 #include "glyphs.h"
 #include "lstream.h"
-#include "objects-tty.h"
 #include "redisplay.h"
 #include "sysdep.h"
 #include "window.h"
 
+#include "console-tty.h"
+#include "objects-tty.h"
+
 /* These headers #define all kinds of common words like "columns"...
    What a bunch of losers.  If we were to include them, we'd have to
    include them last to prevent them from messing up our own header
@@ -323,9 +325,9 @@ tty_output_display_block (struct window *w, struct display_line *dl, int block,
 	      Lisp_Object window;
 	      Lisp_Object instance;
 
-	      XSETWINDOW (window, w);
+	      window = wrap_window (w);
 	      instance = glyph_image_instance (rb->object.dglyph.glyph,
-					       window, ERROR_ME_NOT, 1);
+					       window, ERROR_ME_DEBUG_WARN, 1);
 
 	      if (IMAGE_INSTANCEP (instance))
 		{
@@ -475,9 +477,8 @@ tty_clear_to_window_end (struct window *w, int ypos1, int ypos2)
     }
   else
     {
-      Lisp_Object window;
+      Lisp_Object window = wrap_window (w);
 
-      XSETWINDOW (window, w);
       redisplay_clear_region (window, DEFAULT_INDEX, x, ypos1, width, ypos2 - ypos1);
     }
 }
@@ -796,7 +797,7 @@ tty_turn_on_frame_face (struct frame *f, Lisp_Object face)
   Lisp_Object frame;
   struct console *c = XCONSOLE (FRAME_CONSOLE (f));
 
-  XSETFRAME (frame, f);
+  frame = wrap_frame (f);
   tty_turn_on_face_1 (c,
 		      FACE_HIGHLIGHT_P (face, frame),
 		      FACE_BLINKING_P (face, frame),
@@ -820,7 +821,7 @@ tty_turn_off_frame_face (struct frame *f, Lisp_Object face)
   Lisp_Object frame;
   struct console *c = XCONSOLE (FRAME_CONSOLE (f));
 
-  XSETFRAME (frame, f);
+  frame = wrap_frame (f);
 
   if (FACE_REVERSE_P (face, frame))
     {
