@@ -28,14 +28,16 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
+#include "device.h"
+#include "frame.h"
+#include "window.h"
+
 #include "console-x.h"
-#include "EmacsFrame.h"
 #include "glyphs-x.h"
 #include "gui-x.h"
 #include "scrollbar-x.h"
 
-#include "frame.h"
-#include "window.h"
+#include "EmacsFrame.h"
 
 static void x_update_vertical_scrollbar_callback (Widget widget, LWLIB_ID id,
 						  XtPointer client_data);
@@ -673,9 +675,8 @@ x_update_horizontal_scrollbar_callback (Widget widget, LWLIB_ID id,
 static void
 x_scrollbar_pointer_changed_in_window (struct window *w)
 {
-  Lisp_Object window;
+  Lisp_Object window = wrap_window (w);
 
-  XSETWINDOW (window, w);
   x_scrollbar_loop (X_SET_SCROLLBAR_POINTER, window, find_window_mirror (w),
 		    0, (Window) NULL);
 }
