@@ -1,6 +1,7 @@
 /* String search routines for XEmacs.
    Copyright (C) 1985, 1986, 1987, 1992-1995 Free Software Foundation, Inc.
    Copyright (C) 1995 Sun Microsystems, Inc.
+   Copyright (C) 2001, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -123,6 +124,47 @@ static Charbpos search_buffer (struct buffer *buf, Lisp_Object str,
 			     Lisp_Object trt, Lisp_Object inverse_trt,
 			     int posix);
 
+struct regex_reentrancy
+{
+  struct syntax_cache cache;
+  struct buffer *regex_emacs_buffer;
+  Lisp_Object regex_match_object;
+};
+
+typedef struct
+{
+  Dynarr_declare (struct regex_reentrancy);
+} regex_reentrancy_dynarr;
+
+static regex_reentrancy_dynarr *the_regex_reentrancy_dynarr;
+
+static Lisp_Object
+restore_regex_reentrancy (Lisp_Object dummy)
+{
+  struct regex_reentrancy rr = Dynarr_pop (the_regex_reentrancy_dynarr);
+  syntax_cache = rr.cache;
+  regex_emacs_buffer = rr.regex_emacs_buffer;
+  regex_match_object = rr.regex_match_object;
+  return Qnil;
+}
+
+static int
+begin_regex_reentrancy (void)
+{
+  /* #### there is still a potential problem with the regex cache --
+     the compiled regex could be overwritten.  we'd need 20-fold
+     reentrancy, though. */
+  struct regex_reentrancy rr;
+  rr.cache = syntax_cache;
+  rr.regex_emacs_buffer = regex_emacs_buffer;
+  rr.regex_match_object = regex_match_object;
+  if (!the_regex_reentrancy_dynarr)
+    the_regex_reentrancy_dynarr = Dynarr_new2 (regex_reentrancy_dynarr,
+					       struct regex_reentrancy);
+  Dynarr_add (the_regex_reentrancy_dynarr, rr);
+  return record_unwind_protect (restore_regex_reentrancy, Qnil);
+}
+
 static void
 matcher_overflow (void)
 {
@@ -272,14 +314,12 @@ fixup_search_regs_for_string (Lisp_Object string)
       if (search_regs.start[i] > 0)
 	{
 	  search_regs.start[i] =
-	    bytecount_to_charcount (XSTRING_DATA (string),
-				    search_regs.start[i]);
+	    string_index_byte_to_char (string, search_regs.start[i]);
 	}
       if (search_regs.end[i] > 0)
 	{
 	  search_regs.end[i] =
-	    bytecount_to_charcount (XSTRING_DATA (string),
-				    search_regs.end[i]);
+	    string_index_byte_to_char (string, search_regs.end[i]);
 	}
     }
 }
@@ -294,6 +334,7 @@ looking_at_1 (Lisp_Object string, struct buffer *buf, int posix)
   Bytecount s1, s2;
   REGISTER int i;
   struct re_pattern_buffer *bufp;
+  int count = begin_regex_reentrancy ();
 
   if (running_asynch_code)
     save_search_regs ();
@@ -326,7 +367,7 @@ looking_at_1 (Lisp_Object string, struct buffer *buf, int posix)
 
   val = (0 <= i ? Qt : Qnil);
   if (NILP (val))
-    return Qnil;
+    return unbind_to (count);
   {
     int num_regs = search_regs.num_regs;
     for (i = 0; i < num_regs; i++)
@@ -336,9 +377,9 @@ looking_at_1 (Lisp_Object string, struct buffer *buf, int posix)
 	  search_regs.end[i] += BI_BUF_BEGV (buf);
 	}
   }
-  XSETBUFFER (last_thing_searched, buf);
+  last_thing_searched = wrap_buffer (buf);
   fixup_search_regs_for_buffer (buf);
-  return val;
+  return unbind_to_1 (count, val);
 }
 
 DEFUN ("looking-at", Flooking_at, 1, 2, 0, /*
@@ -376,6 +417,7 @@ string_match_1 (Lisp_Object regexp, Lisp_Object string, Lisp_Object start,
   Bytecount val;
   Charcount s;
   struct re_pattern_buffer *bufp;
+  int count = begin_regex_reentrancy ();
 
   if (running_asynch_code)
     save_search_regs ();
@@ -404,7 +446,7 @@ string_match_1 (Lisp_Object regexp, Lisp_Object string, Lisp_Object start,
 			  0, ERROR_ME);
   QUIT;
   {
-    Bytecount bis = charcount_to_bytecount (XSTRING_DATA (string), s);
+    Bytecount bis = string_index_char_to_byte (string, s);
     regex_match_object = string;
     regex_emacs_buffer = buf;
     val = re_search (bufp, (char *) XSTRING_DATA (string),
@@ -414,10 +456,12 @@ string_match_1 (Lisp_Object regexp, Lisp_Object string, Lisp_Object start,
   }
   if (val == -2)
     matcher_overflow ();
-  if (val < 0) return Qnil;
+  if (val < 0) return unbind_to (count);
   last_thing_searched = Qt;
   fixup_search_regs_for_string (string);
-  return make_int (bytecount_to_charcount (XSTRING_DATA (string), val));
+  return
+    unbind_to_1 (count,
+	       make_int (string_index_byte_to_char (string, val)));
 }
 
 DEFUN ("string-match", Fstring_match, 2, 4, 0, /*
@@ -467,10 +511,11 @@ fast_string_match (Lisp_Object regexp,  const Intbyte *nonreloc,
   Bytecount val;
   Intbyte *newnonreloc = (Intbyte *) nonreloc;
   struct re_pattern_buffer *bufp;
+  int count;
 
   bufp = compile_pattern (regexp, 0,
 			  (case_fold_search
-			   ? XCASE_TABLE_DOWNCASE (current_buffer->case_table)
+			   ? XCASE_TABLE_DOWNCASE (Vstandard_case_table)
 			   : Qnil),
 			  0, errb);
   if (!bufp)
@@ -482,20 +527,12 @@ fast_string_match (Lisp_Object regexp,  const Intbyte *nonreloc,
 
   fixup_internal_substring (nonreloc, reloc, offset, &length);
 
+  /* Don't need to protect against GC inside of re_search() due to QUIT;
+     QUIT is GC-inhibited. */
   if (!NILP (reloc))
-    {
-      if (no_quit)
-	newnonreloc = XSTRING_DATA (reloc);
-      else
-	{
-	  /* QUIT could relocate RELOC.  Therefore we must alloca()
-	     and copy.  No way around this except some serious
-	     rewriting of re_search(). */
-	  newnonreloc = (Intbyte *) alloca (length);
-	  memcpy (newnonreloc, XSTRING_DATA (reloc), length);
-	}
-    }
+    newnonreloc = XSTRING_DATA (reloc);
 
+  count = begin_regex_reentrancy ();
   /* #### evil current-buffer dependency */
   regex_match_object = reloc;
   regex_emacs_buffer = current_buffer;
@@ -503,6 +540,7 @@ fast_string_match (Lisp_Object regexp,  const Intbyte *nonreloc,
 		   length, 0);
 
   no_quit_in_re_search = 0;
+  unbind_to (count);
   return val;
 }
 
@@ -716,12 +754,12 @@ find_next_newline (struct buffer *buf, Charbpos from, int count)
 }
 
 Bytebpos
-bi_find_next_emchar_in_string (Lisp_String* str, Emchar target, Bytebpos st,
+bi_find_next_emchar_in_string (Lisp_Object str, Emchar target, Bytebpos st,
 			       EMACS_INT count)
 {
   /* This function has been Mule-ized. */
-  Bytebpos lim = string_length (str) -1;
-  Intbyte* s = string_data (str);
+  Bytebpos lim = XSTRING_LENGTH (str) -1;
+  Intbyte *s = XSTRING_DATA (str);
 
   assert (count >= 0);
 
@@ -735,7 +773,7 @@ bi_find_next_emchar_in_string (Lisp_String* str, Emchar target, Bytebpos st,
     {
       while (st < lim && count > 0)
 	{
-	  if (string_char (str, st) == target)
+	  if (XSTRING_CHAR (str, st) == target)
 	    count--;
 	  INC_CHARBYTEBPOS (s, st);
 	}
@@ -1141,6 +1179,7 @@ search_buffer (struct buffer *buf, Lisp_Object string, Charbpos charbpos,
   Bytebpos p1, p2;
   Bytecount s1, s2;
   Bytebpos pos, lim;
+  int count;
 
   if (running_asynch_code)
     save_search_regs ();
@@ -1161,6 +1200,7 @@ search_buffer (struct buffer *buf, Lisp_Object string, Charbpos charbpos,
   if (RE && !trivial_regexp_p (string))
     {
       struct re_pattern_buffer *bufp;
+      count = begin_regex_reentrancy ();
 
       bufp = compile_pattern (string, &search_regs, trt, posix,
 			      ERROR_ME);
@@ -1199,7 +1239,7 @@ search_buffer (struct buffer *buf, Lisp_Object string, Charbpos charbpos,
 		    search_regs.start[i] += j;
 		    search_regs.end[i] += j;
 		  }
-	      XSETBUFFER (last_thing_searched, buf);
+	      last_thing_searched = wrap_buffer (buf);
 	      /* Set pos to the new position. */
 	      pos = search_regs.start[0];
 	      fixup_search_regs_for_buffer (buf);
@@ -1208,6 +1248,7 @@ search_buffer (struct buffer *buf, Lisp_Object string, Charbpos charbpos,
 	    }
 	  else
 	    {
+	      unbind_to (count);
 	      return n;
 	    }
 	  n++;
@@ -1236,7 +1277,7 @@ search_buffer (struct buffer *buf, Lisp_Object string, Charbpos charbpos,
 		    search_regs.start[i] += j;
 		    search_regs.end[i] += j;
 		  }
-	      XSETBUFFER (last_thing_searched, buf);
+	      last_thing_searched = wrap_buffer (buf);
 	      /* Set pos to the new position. */
 	      pos = search_regs.end[0];
 	      fixup_search_regs_for_buffer (buf);
@@ -1245,10 +1286,12 @@ search_buffer (struct buffer *buf, Lisp_Object string, Charbpos charbpos,
 	    }
 	  else
 	    {
+	      unbind_to (count);
 	      return 0 - n;
 	    }
 	  n--;
 	}
+      unbind_to (count);
       return charbpos;
     }
   else				/* non-RE case */
@@ -1633,7 +1676,7 @@ boyer_moore (struct buffer *buf, Intbyte *base_pat, Bytecount len,
 		  /* For all the characters that map into CH,
 		     set up simple_translate to map the last byte
 		     into STARTING_J.  */
-		  simple_translate[j] = starting_j;
+		  simple_translate[j] = (Intbyte) starting_j;
 		  if (ch == starting_ch)
 		    break;
 		  BM_tab[j] = dirlen - i;
@@ -1923,7 +1966,7 @@ set_search_regs (struct buffer *buf, Charbpos beg, Charcount len)
 
   search_regs.start[0] = beg;
   search_regs.end[0] = beg + len;
-  XSETBUFFER (last_thing_searched, buf);
+  last_thing_searched = wrap_buffer (buf);
 }
 
 
@@ -1943,14 +1986,14 @@ wordify (Lisp_Object buffer, Lisp_Object string)
   len = XSTRING_CHAR_LENGTH (string);
 
   for (i = 0; i < len; i++)
-    if (!WORD_SYNTAX_P (syntax_table, string_char (XSTRING (string), i)))
+    if (!WORD_SYNTAX_P (syntax_table, XSTRING_CHAR (string, i)))
       {
 	punct_count++;
 	if (i > 0 && WORD_SYNTAX_P (syntax_table,
-				    string_char (XSTRING (string), i - 1)))
+				    XSTRING_CHAR (string, i - 1)))
           word_count++;
       }
-  if (WORD_SYNTAX_P (syntax_table, string_char (XSTRING (string), len - 1)))
+  if (WORD_SYNTAX_P (syntax_table, XSTRING_CHAR (string, len - 1)))
     word_count++;
   if (!word_count) return build_string ("");
 
@@ -1967,13 +2010,13 @@ wordify (Lisp_Object buffer, Lisp_Object string)
 
     for (i = 0; i < len; i++)
       {
-	Emchar ch = string_char (XSTRING (string), i);
+	Emchar ch = XSTRING_CHAR (string, i);
 
 	if (WORD_SYNTAX_P (syntax_table, ch))
 	  o += set_charptr_emchar (o, ch);
 	else if (i > 0
 		 && WORD_SYNTAX_P (syntax_table,
-				   string_char (XSTRING (string), i - 1))
+				   XSTRING_CHAR (string, i - 1))
 		 && --word_count)
 	  {
 	    *o++ = '\\';
@@ -2289,7 +2332,7 @@ match since only regular expressions have distinguished subexpressions.
 	 the last string matched and the buffer used for that
 	 matching.  But of course we can't change it as it is. */
       buf = decode_buffer (strbuffer, 0);
-      XSETBUFFER (buffer, buf);
+      buffer = wrap_buffer (buf);
     }
   else
     {
@@ -2351,7 +2394,7 @@ match since only regular expressions have distinguished subexpressions.
 	  if (NILP (string))
 	    c = BUF_FETCH_CHAR (buf, pos);
 	  else
-	    c = string_char (XSTRING (string), pos);
+	    c = XSTRING_CHAR (string, pos);
 
 	  if (LOWERCASEP (buf, c))
 	    {
@@ -2436,10 +2479,10 @@ match since only regular expressions have distinguished subexpressions.
 	      Charcount substart = -1;
 	      Charcount subend   = -1;
 
-	      c = string_char (XSTRING (replacement), strpos);
+	      c = XSTRING_CHAR (replacement, strpos);
 	      if (c == '\\' && strpos < stlen - 1)
 		{
-		  c = string_char (XSTRING (replacement), ++strpos);
+		  c = XSTRING_CHAR (replacement, ++strpos);
 		  if (c == '&')
 		    {
 		      literal_end = strpos - 1;
@@ -2529,7 +2572,7 @@ match since only regular expressions have distinguished subexpressions.
 
 	  for (strpos = 0; strpos < stlen; strpos++)
 	    {
-	      Emchar curchar = string_char (XSTRING (replacement), strpos);
+	      Emchar curchar = XSTRING_CHAR (replacement, strpos);
 	      Emchar newchar = -1;
 	      if (i < Dynarr_length (ul_pos_dynarr) &&
 		  strpos == Dynarr_at (ul_pos_dynarr, i))
@@ -2553,12 +2596,12 @@ match since only regular expressions have distinguished subexpressions.
 		    newchar = curchar;
 		}
 	      if (newchar != curchar)
-		set_string_char (XSTRING (replacement), strpos, newchar);
+		set_string_char (replacement, strpos, newchar);
 	    }
 	}
 
       /* frees the Dynarrs if necessary. */
-      unbind_to (speccount, Qnil);
+      unbind_to (speccount);
       return concat3 (before, replacement, after);
     }
 
@@ -2590,7 +2633,7 @@ match since only regular expressions have distinguished subexpressions.
 	   */
 	  Charcount offset = BUF_PT (buf) - search_regs.start[sub];
 
-	  c = string_char (XSTRING (replacement), strpos);
+	  c = XSTRING_CHAR (replacement, strpos);
 	  if (c == '\\' && strpos < stlen - 1)
 	    {
 	      /* XXX FIXME: replacing just a substring non-literally
@@ -2599,7 +2642,7 @@ match since only regular expressions have distinguished subexpressions.
 		 <duwe@caldera.de> claims Finsert_buffer_substring already
 		 handles this correctly.
 	      */
-	      c = string_char (XSTRING (replacement), ++strpos);
+	      c = XSTRING_CHAR (replacement, ++strpos);
 	      if (c == '&')
 		Finsert_buffer_substring
                   (buffer,
@@ -2691,7 +2734,7 @@ match since only regular expressions have distinguished subexpressions.
     }
 
   /* frees the Dynarrs if necessary. */
-  unbind_to (speccount, Qnil);
+  unbind_to (speccount);
   end_multiple_change (buf, mc_count);
 
   return Qnil;
@@ -2872,7 +2915,7 @@ LIST should have been created by calling `match-data' previously.
 	      if (XMARKER (marker)->buffer == 0)
 		marker = Qzero;
 	      else
-		XSETBUFFER (last_thing_searched, XMARKER (marker)->buffer);
+		last_thing_searched = wrap_buffer (XMARKER (marker)->buffer);
 	    }
 
 	  CHECK_INT_COERCE_MARKER (marker);
