@@ -1,5 +1,6 @@
 /* X Selection processing for XEmacs
    Copyright (C) 1990, 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
+   Copyright (C) 2001, 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -25,14 +26,16 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include "lisp.h"
 
-#include "buffer.h"
+#include "charset.h"
+#include "device.h"
+#include "frame.h"
+#include "opaque.h"
+#include "select.h"
+
 #include "console-x.h"
 #include "objects-x.h"
 
-#include "frame.h"
-#include "opaque.h"
 #include "systime.h"
-#include "select.h"
 
 int lisp_to_time (Lisp_Object, time_t *);
 Lisp_Object time_to_lisp (time_t);
@@ -176,8 +179,8 @@ x_atom_to_symbol (struct device *d, Atom atom)
 #endif
 
   {
-    char *intstr;
-    char *str = XGetAtomName (display, atom);
+    Intbyte *intstr;
+    Extbyte *str = XGetAtomName (display, atom);
 
     if (! str) return Qnil;
 
@@ -185,7 +188,7 @@ x_atom_to_symbol (struct device *d, Atom atom)
 			C_STRING_ALLOCA, intstr,
 			Qctext);
     XFree (str);
-    return intern (intstr);
+    return intern_int (intstr);
   }
 }
 
@@ -638,7 +641,7 @@ x_handle_selection_request (XSelectionRequestEvent *event)
     xfree (data);
   }
 
-  unbind_to (count, Qnil);
+  unbind_to (count);
 
  DONE_LABEL:
 
@@ -795,8 +798,11 @@ x_handle_property_notify (XPropertyEvent *event)
 	{
 #if 0
 	  stderr_out ("Saw expected prop-%s on %s\n",
-		   (event->state == PropertyDelete ? "delete" : "change"),
-		      (char *) string_data (XSYMBOL (x_atom_to_symbol (get_device_from_display (event->display), event->atom))->name);
+		      (event->state == PropertyDelete ? "delete" : "change"),
+		      XSTRING_DATA
+		      (XSYMBOL (x_atom_to_symbol
+				(get_device_from_display (event->display),
+				 event->atom))->name);
 #endif
 	  if (prev)
 	    prev->next = rest->next;
@@ -810,8 +816,10 @@ x_handle_property_notify (XPropertyEvent *event)
     }
 #if 0
   stderr_out ("Saw UNexpected prop-%s on %s\n",
-	   (event->state == PropertyDelete ? "delete" : "change"),
-	   (char *) string_data (XSYMBOL (x_atom_to_symbol (get_device_from_display (event->display), event->atom))->name));
+	      (event->state == PropertyDelete ? "delete" : "change"),
+	      XSTRING_DATA (XSYMBOL (x_atom_to_symbol
+				     (get_device_from_display (event->display),
+				      event->atom))->name));
 #endif
 }
 
@@ -930,7 +938,7 @@ x_get_foreign_selection (Lisp_Object selection_symbol, Lisp_Object target_type)
   if (selection_reply_timed_out)
     signal_error (Qselection_conversion_error, "timed out waiting for reply from selection owner", Qunbound);
 
-  unbind_to (speccount, Qnil);
+  unbind_to (speccount);
 
   /* otherwise, the selection is waiting for us on the requested property. */
 
