@@ -21,6 +21,8 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include <config.h>
 #include "lisp.h"
 
+#include "sysfile.h"
+
 #include <unistd.h>
 #include <sheap-adjust.h>
 
@@ -106,7 +108,7 @@ static_heap_base, static_heap_ptr);
 static void
 sheap_adjust_h ()
 {
-  FILE *stream = fopen ("sheap-adjust.h", "w");
+  FILE *stream = retry_fopen ("sheap-adjust.h", "w");
 
   if (stream == NULL)
     report_file_error ("Opening sheap adjustment file",
@@ -117,7 +119,7 @@ sheap_adjust_h ()
 	   "\tAutomatically generated by XEmacs */\n"
 	   "# define SHEAP_ADJUSTMENT (%d)\n",
 	   ((static_heap_ptr - static_heap_buffer) - STATIC_HEAP_BASE));
-  fclose (stream);
+  retry_fclose (stream);
 }
 
 void report_sheap_usage (int die_if_pure_storage_exceeded);
