@@ -1,6 +1,6 @@
 /* Generic specifier list implementation
    Copyright (C) 1994, 1995 Board of Trustees, University of Illinois.
-   Copyright (C) 1995 Ben Wing
+   Copyright (C) 1995, 2002 Ben Wing
 
 This file is part of XEmacs.
 
@@ -194,7 +194,22 @@ struct specifier_methods
   void (*after_change_method) (Lisp_Object specifier,
 			       Lisp_Object locale);
 
+  /* Specifier extra data: Specifier objects can have extra data,
+     specific to the type of specifier, stored at the end of the
+     object.  To have this, a specifier declares a structure of type
+     `struct TYPE_specifier' containing the data and uses
+     INITIALIZE_SPECIFIER_TYPE_WITH_DATA instead of
+     INITIALIZE_SPECIFIER_TYPE.  Then, a pointer to the `struct
+     TYPE_specifier' can be obtained from a specifier object using
+     SPECIFIER_TYPE_DATA. */
+
+  /* Pdump description of the extra data; required, and must be named
+     TYPE_specifier_description.  Initialized when
+     INITIALIZE_SPECIFIER_TYPE_WITH_DATA is called. */
   const struct lrecord_description *extra_description;
+
+  /* Size of extra data structure; initialized when
+     INITIALIZE_SPECIFIER_TYPE_WITH_DATA is called. */
   int extra_data_size;
 };
 
@@ -241,7 +256,6 @@ typedef struct Lisp_Specifier Lisp_Specifier;
 
 DECLARE_LRECORD (specifier, Lisp_Specifier);
 #define XSPECIFIER(x) XRECORD (x, specifier, Lisp_Specifier)
-#define XSETSPECIFIER(x, p) XSETRECORD (x, p, specifier)
 #define wrap_specifier(p) wrap_record (p, specifier)
 #define SPECIFIERP(x) RECORDP (x, specifier)
 #define CHECK_SPECIFIER(x) CHECK_RECORD (x, specifier)
@@ -265,7 +279,7 @@ DECLARE_LRECORD (specifier, Lisp_Specifier);
 #define specifier_data_offset offsetof (Lisp_Specifier, data)
 extern const struct lrecord_description specifier_empty_extra_description[];
 
-#ifdef ERROR_CHECK_TYPECHECK
+#ifdef ERROR_CHECK_TYPES
 #define DECLARE_SPECIFIER_TYPE(type)					\
 extern struct specifier_methods * type##_specifier_methods;		\
 INLINE_HEADER struct type##_specifier *					\
@@ -296,20 +310,20 @@ DECLARE_NOTHING
 #else
 #define DECLARE_SPECIFIER_TYPE(type)					\
 extern struct specifier_methods * type##_specifier_methods
-#endif /* ERROR_CHECK_TYPECHECK */
+#endif /* ERROR_CHECK_TYPES */
 
 #define DEFINE_SPECIFIER_TYPE(type)					\
 struct specifier_methods * type##_specifier_methods
 
-#define INITIALIZE_SPECIFIER_TYPE(type, obj_name, pred_sym) do {		\
-  type##_specifier_methods = xnew_and_zero (struct specifier_methods);		\
-  type##_specifier_methods->name = obj_name;					\
-  type##_specifier_methods->extra_description =					\
-    specifier_empty_extra_description;						\
-  defsymbol_nodump (&type##_specifier_methods->predicate_symbol, pred_sym);	\
-  add_entry_to_specifier_type_list (Q##type, type##_specifier_methods);		\
-  dump_add_root_struct_ptr (&type##_specifier_methods,				\
-			    &specifier_methods_description);			\
+#define INITIALIZE_SPECIFIER_TYPE(type, obj_name, pred_sym) do {	    \
+  type##_specifier_methods = xnew_and_zero (struct specifier_methods);	    \
+  type##_specifier_methods->name = obj_name;				    \
+  type##_specifier_methods->extra_description =				    \
+    specifier_empty_extra_description;					    \
+  defsymbol_nodump (&type##_specifier_methods->predicate_symbol, pred_sym); \
+  add_entry_to_specifier_type_list (Q##type, type##_specifier_methods);	    \
+  dump_add_root_struct_ptr (&type##_specifier_methods,			    \
+			    &specifier_methods_description);		    \
 } while (0)
 
 #define REINITIALIZE_SPECIFIER_TYPE(type) do {				\
@@ -344,7 +358,7 @@ do {									\
 
 #define GHOST_SPECIFIER(sp) XSPECIFIER ((sp)->fallback)
 
-#ifdef ERROR_CHECK_TYPECHECK
+#ifdef ERROR_CHECK_TYPES
 # define SPECIFIER_TYPE_DATA(sp, type) \
    error_check_##type##_specifier_data (sp)
 #else
@@ -355,18 +369,12 @@ do {									\
      : (sp)->data))
 #endif
 
-#ifdef ERROR_CHECK_TYPECHECK
+#ifdef ERROR_CHECK_TYPES
 # define XSPECIFIER_TYPE(x, type)	\
    error_check_##type##_specifier_type (x)
-# define XSETSPECIFIER_TYPE(x, p, type)	do		\
-{							\
-  XSETSPECIFIER (x, p);					\
-  assert (SPECIFIER_TYPEP (XSPECIFIER(x), type));	\
-} while (0)
 #else
 # define XSPECIFIER_TYPE(x, type) XSPECIFIER (x)
-# define XSETSPECIFIER_TYPE(x, p, type) XSETSPECIFIER (x, p)
-#endif /* ERROR_CHECK_TYPE_CHECK */
+#endif /* ERROR_CHECK_TYPES */
 
 #define SPECIFIER_TYPEP(x, type)			\
   (SPECIFIERP (x) && SPECIFIER_TYPE_P (XSPECIFIER (x), type))
@@ -507,35 +515,30 @@ void kill_specifier_buffer_locals (Lisp_Object buffer);
 
 DECLARE_SPECIFIER_TYPE (generic);
 #define XGENERIC_SPECIFIER(x) XSPECIFIER_TYPE (x, generic)
-#define XSETGENERIC_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, generic)
 #define GENERIC_SPECIFIERP(x) SPECIFIER_TYPEP (x, generic)
 #define CHECK_GENERIC_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, generic)
 #define CONCHECK_GENERIC_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, generic)
 
 DECLARE_SPECIFIER_TYPE (integer);
 #define XINTEGER_SPECIFIER(x) XSPECIFIER_TYPE (x, integer)
-#define XSETINTEGER_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, integer)
 #define INTEGER_SPECIFIERP(x) SPECIFIER_TYPEP (x, integer)
 #define CHECK_INTEGER_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, integer)
 #define CONCHECK_INTEGER_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, integer)
 
 DECLARE_SPECIFIER_TYPE (natnum);
 #define XNATNUM_SPECIFIER(x) XSPECIFIER_TYPE (x, natnum)
-#define XSETNATNUM_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, natnum)
 #define NATNUM_SPECIFIERP(x) SPECIFIER_TYPEP (x, natnum)
 #define CHECK_NATNUM_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, natnum)
 #define CONCHECK_NATNUM_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, natnum)
 
 DECLARE_SPECIFIER_TYPE (boolean);
 #define XBOOLEAN_SPECIFIER(x) XSPECIFIER_TYPE (x, boolean)
-#define XSETBOOLEAN_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, boolean)
 #define BOOLEAN_SPECIFIERP(x) SPECIFIER_TYPEP (x, boolean)
 #define CHECK_BOOLEAN_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, boolean)
 #define CONCHECK_BOOLEAN_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, boolean)
 
 DECLARE_SPECIFIER_TYPE (display_table);
 #define XDISPLAYTABLE_SPECIFIER(x) XSPECIFIER_TYPE (x, display_table)
-#define XSETDISPLAYTABLE_SPECIFIER(x, p) XSETSPECIFIER_TYPE (x, p, display_table)
 #define DISPLAYTABLE_SPECIFIERP(x) SPECIFIER_TYPEP (x, display_table)
 #define CHECK_DISPLAYTABLE_SPECIFIER(x) CHECK_SPECIFIER_TYPE (x, display_table)
 #define CONCHECK_DISPLAYTABLE_SPECIFIER(x) CONCHECK_SPECIFIER_TYPE (x, display_table)
