@@ -39,29 +39,24 @@ Lisp_Object Vusage_tracking;
 #include <ut.h>
 #endif
 
-DEFUN  ("ut-log-text", 
-	Fut_log_text, 
-	Sut_log_text, 
-	1, MANY, 0 /*
+DEFUN ("ut-log-text", Fut_log_text, 1, MANY, 0, /*
 Log a usage-tracking message if `usage-tracking' is non-nil.
 Args are the same as to `format'.  Returns whether the message was
 actually logged.  If usage-tracking support was not compiled in, this
 function has no effect and always returns `nil'.  See function
 `has-usage-tracking-p'.
-*/ )
-     (nargs, args)
-     int nargs;
-     Lisp_Object *args;
+*/
+       (int nargs, Lisp_Object *args))
 {
 #ifdef USAGE_TRACKING
   Lisp_Object xs;
-  unsigned char *s; /* #### Does not support I18N4. */
+  unsigned char *s;
 
   if (!NILP (Vusage_tracking))
     {
       xs = Fformat (nargs, args);
       CHECK_STRING (xs);
-      s = string_data (XSTRING (xs));
+      s = XSTRING_DATA (xs);
       ut_log_text ((char *) s);
     }
   return Vusage_tracking;
@@ -78,7 +73,7 @@ function has no effect and always returns `nil'.  See function
 void
 syms_of_sunpro (void)
 {
-  defsubr (&Sut_log_text);
+  DEFSUBR (Fut_log_text);
 }
 
 void
@@ -99,17 +94,12 @@ Has no effect if usage tracking support has not been compiled in.
 void
 init_sunpro (void)
 {
+  Vusage_tracking = Qnil;
 #ifdef USAGE_TRACKING
   if (!purify_flag)
     {	       /* Enabled only when not dumping an executable */
       Vusage_tracking = Qt;
       ut_initialize ("xemacs", NULL, NULL);
     }
-  else
-    {
-      Vusage_tracking = Qnil;
-    }
-#else
-  Vusage_tracking = Qnil;
 #endif
 }
