@@ -1,5 +1,6 @@
 /* Declarations having to do with XEmacs syntax tables.
    Copyright (C) 1985, 1992, 1993 Free Software Foundation, Inc.
+   Copyright (C) 2002 Ben Wing.
 
 This file is part of XEmacs.
 
@@ -326,21 +327,21 @@ void update_syntax_cache (int pos, int count, int init);
    The value is meant for use in the UPDATE_SYNTAX_TABLE... macros.
    These macros do nothing when parse_sexp_lookup_properties is 0,
    so we return 0 in that case, for speed.  */
-#define SYNTAX_CACHE_BYTE_TO_CHAR(bytepos)					\
-  (! lookup_syntax_properties							\
-   ? 0										\
-   : STRINGP (syntax_cache.object)						\
-   ? bytecount_to_charcount (XSTRING_DATA (syntax_cache.object), bytepos)	\
-   : (BUFFERP (syntax_cache.object) || NILP (syntax_cache.object))		\
-   ? bytebpos_to_charbpos (syntax_cache.buffer,					\
-		       bytepos + BI_BUF_BEGV (syntax_cache.buffer))		\
+#define SYNTAX_CACHE_BYTE_TO_CHAR(bytepos)				\
+  (! lookup_syntax_properties						\
+   ? 0									\
+   : STRINGP (syntax_cache.object)					\
+   ? string_index_byte_to_char (syntax_cache.object, bytepos)		\
+   : (BUFFERP (syntax_cache.object) || NILP (syntax_cache.object))	\
+   ? bytebpos_to_charbpos (syntax_cache.buffer,				\
+		       bytepos + BI_BUF_BEGV (syntax_cache.buffer))	\
    : (bytepos))
 
 #define SYNTAX_CACHE_OBJECT_BYTE_TO_CHAR(obj, buf, bytepos)	\
   (! lookup_syntax_properties					\
    ? 0								\
    : STRINGP (obj)						\
-   ? bytecount_to_charcount (XSTRING_DATA (obj), bytepos)	\
+   ? string_index_byte_to_char (obj, bytepos)			\
    : (BUFFERP (obj) || NILP (obj))				\
    ? bytebpos_to_charbpos (buf, bytepos + BI_BUF_BEGV (buf))	\
    : (bytepos))
