@@ -164,14 +164,12 @@ print_tooltalk_message (Lisp_Object obj, Lisp_Object printcharfun,
 {
   Lisp_Tooltalk_Message *p = XTOOLTALK_MESSAGE (obj);
 
-  char buf[200];
-
   if (print_readably)
     printing_unreadable_object ("#<tooltalk_message 0x%x>",
 				p->header.uid);
 
-  sprintf (buf, "#<tooltalk_message id:0x%lx 0x%x>", (long) (p->m), p->header.uid);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, "#<tooltalk_message id:0x%lx 0x%x>",
+		    (long) (p->m), p->header.uid);
 }
 
 DEFINE_LRECORD_IMPLEMENTATION ("tooltalk-message", tooltalk_message,
@@ -189,8 +187,7 @@ make_tooltalk_message (Tt_message m)
   msg->m = m;
   msg->callback = Qnil;
   msg->plist_sym = Fmake_symbol (Tooltalk_Message_plist_str);
-  XSETTOOLTALK_MESSAGE (val, msg);
-  return val;
+  return wrap_tooltalk_message (msg);
 }
 
 Tt_message
@@ -237,14 +234,12 @@ print_tooltalk_pattern (Lisp_Object obj, Lisp_Object printcharfun,
 {
   Lisp_Tooltalk_Pattern *p = XTOOLTALK_PATTERN (obj);
 
-  char buf[200];
-
   if (print_readably)
     printing_unreadable_object ("#<tooltalk_pattern 0x%x>",
 				p->header.uid);
 
-  sprintf (buf, "#<tooltalk_pattern id:0x%lx 0x%x>", (long) (p->p), p->header.uid);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, "#<tooltalk_pattern id:0x%lx 0x%x>",
+		    (long) (p->p), p->header.uid);
 }
 
 DEFINE_LRECORD_IMPLEMENTATION ("tooltalk-pattern", tooltalk_pattern,
@@ -263,8 +258,7 @@ make_tooltalk_pattern (Tt_pattern p)
   pat->callback = Qnil;
   pat->plist_sym = Fmake_symbol (Tooltalk_Pattern_plist_str);
 
-  XSETTOOLTALK_PATTERN (val, pat);
-  return val;
+  return wrap_tooltalk_pattern (pat);
 }
 
 static Tt_pattern
@@ -503,7 +497,7 @@ tt_state_symbol (Tt_state n)
 }
 
 static Lisp_Object
-tt_build_string (char *s)
+tt_build_c_string (char *s)
 {
   return build_string (s ? s : "");
 }
@@ -600,7 +594,7 @@ value returned by 'arg_bval like a string is fine.
     return tt_mode_symbol (tt_message_arg_mode (m, n));
 
   else if (EQ (attribute, Qtt_arg_type))
-    return tt_build_string (tt_message_arg_type (m, n));
+    return tt_build_c_string (tt_message_arg_type (m, n));
 
   else if (EQ (attribute, Qtt_arg_val))
     return tt_message_arg_bval_vector (m, n);
@@ -618,40 +612,40 @@ value returned by 'arg_bval like a string is fine.
     return make_int (tt_message_disposition (m));
 
   else if (EQ (attribute, Qtt_file))
-    return tt_build_string (tt_message_file (m));
+    return tt_build_c_string (tt_message_file (m));
 
   else if (EQ (attribute, Qtt_gid))
     return make_int (tt_message_gid (m));
 
   else if (EQ (attribute, Qtt_handler))
-    return tt_build_string (tt_message_handler (m));
+    return tt_build_c_string (tt_message_handler (m));
 
   else if (EQ (attribute, Qtt_handler_ptype))
-    return tt_build_string (tt_message_handler_ptype (m));
+    return tt_build_c_string (tt_message_handler_ptype (m));
 
   else if (EQ (attribute, Qtt_object))
-    return tt_build_string (tt_message_object (m));
+    return tt_build_c_string (tt_message_object (m));
 
   else if (EQ (attribute, Qtt_op))
-    return tt_build_string (tt_message_op (m));
+    return tt_build_c_string (tt_message_op (m));
 
   else if (EQ (attribute, Qtt_opnum))
     return tt_opnum_string (tt_message_opnum (m));
 
   else if (EQ (attribute, Qtt_otype))
-    return tt_build_string (tt_message_otype (m));
+    return tt_build_c_string (tt_message_otype (m));
 
   else if (EQ (attribute, Qtt_scope))
     return tt_scope_symbol (tt_message_scope (m));
 
   else if (EQ (attribute, Qtt_sender))
-    return tt_build_string (tt_message_sender (m));
+    return tt_build_c_string (tt_message_sender (m));
 
   else if (EQ (attribute, Qtt_sender_ptype))
-    return tt_build_string (tt_message_sender_ptype (m));
+    return tt_build_c_string (tt_message_sender_ptype (m));
 
   else if (EQ (attribute, Qtt_session))
-    return tt_build_string (tt_message_session (m));
+    return tt_build_c_string (tt_message_session (m));
 
   else if (EQ (attribute, Qtt_state))
     return tt_state_symbol (tt_message_state (m));
@@ -660,7 +654,7 @@ value returned by 'arg_bval like a string is fine.
     return make_int (tt_message_status (m));
 
   else if (EQ (attribute, Qtt_status_string))
-    return tt_build_string (tt_message_status_string (m));
+    return tt_build_c_string (tt_message_status_string (m));
 
   else if (EQ (attribute, Qtt_uid))
     return make_int (tt_message_uid (m));
@@ -1262,7 +1256,7 @@ init_tooltalk (void)
     {
       /* Don't ask the user for confirmation when exiting Emacs */
       Fprocess_kill_without_query (lp, Qnil);
-      XSETSUBR (fil, &SFreceive_tooltalk_message);
+      fil = wrap_subr (&SFreceive_tooltalk_message);
       set_process_filter (lp, fil, 1);
     }
   else
@@ -1412,8 +1406,8 @@ Unprocessed messages are messages that didn't match any patterns.
 */ );
   Vtooltalk_unprocessed_message_hook = Qnil;
 
-  Tooltalk_Message_plist_str = build_string ("Tooltalk Message plist");
-  Tooltalk_Pattern_plist_str = build_string ("Tooltalk Pattern p plist");
+  Tooltalk_Message_plist_str = build_msg_string ("Tooltalk Message plist");
+  Tooltalk_Pattern_plist_str = build_msg_string ("Tooltalk Pattern plist");
 
   staticpro(&Tooltalk_Message_plist_str);
   staticpro(&Tooltalk_Pattern_plist_str);
