@@ -22,6 +22,7 @@
 #include "hash.h"
 #include "events.h"
 #include "elhash.h"
+#include "event-gtk.h"
 
 /* XEmacs specific GTK types */
 #include "gtk-glue.c"
@@ -36,7 +37,9 @@ static GHashTable *dll_cache;
 
 Lisp_Object gtk_type_to_lisp (GtkArg *arg);
 int lisp_to_gtk_type (Lisp_Object obj, GtkArg *arg);
+#if 0
 void describe_gtk_arg (GtkArg *arg);
+#endif
 guint symbol_to_enum (Lisp_Object obj, GtkType t);
 static guint lisp_to_flag (Lisp_Object obj, GtkType t);
 static Lisp_Object flags_to_list (guint value, GtkType t);
@@ -307,20 +310,13 @@ mark_ffi_data (Lisp_Object obj)
 static void
 ffi_object_printer (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
-  char buf[200];
-
   if (print_readably)
     printing_unreadable_object ("#<ffi %p>", XFFI (obj)->function_ptr);
 
-  write_c_string ("#<ffi ", printcharfun);
-  print_internal (XFFI (obj)->function_name, printcharfun, 1);
+  write_fmt_string_lisp (printcharfun, "#<ffi %S", 1, XFFI (obj)->function_name);
   if (XFFI (obj)->n_args)
-    {
-      sprintf (buf, " %d arguments", XFFI (obj)->n_args);
-      write_c_string (buf, printcharfun);
-    }
-  sprintf (buf, " %p>", (void *)XFFI (obj)->function_ptr);
-  write_c_string (buf, printcharfun);
+    write_fmt_string (printcharfun, " %d arguments", XFFI (obj)->n_args);
+  write_fmt_string (printcharfun, " %p>", (void *)XFFI (obj)->function_ptr);
 }
 
 DEFINE_LRECORD_IMPLEMENTATION ("ffi", emacs_ffi,
@@ -367,7 +363,7 @@ typedef GList * (*__LIST_fn) ();
     if (freep) xfree(v);				\
   } while (0)
 
-gpointer __allocate_object_storage (GtkType t)
+static gpointer __allocate_object_storage (GtkType t)
 {
   size_t s = 0;
   void *rval = NULL;
@@ -437,7 +433,7 @@ gpointer __allocate_object_storage (GtkType t)
   return (rval);
 }
 
-Lisp_Object type_to_marshaller_type (GtkType t)
+static Lisp_Object type_to_marshaller_type (GtkType t)
 {
   switch (GTK_FUNDAMENTAL_TYPE (t))
     {
@@ -656,7 +652,7 @@ Import a function into the XEmacs namespace.
   data->function_ptr = name_func;
   data->marshal = marshaller_func;
 
-  XSETFFI (rval, data);
+  rval = wrap_emacs_ffi (data);
   return (rval);
 }
 
@@ -772,8 +768,6 @@ Call an external function.
 static void
 emacs_gtk_object_printer (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
-  char buf[200];
-
   if (print_readably)
     printing_unreadable_object ("#<GtkObject %p>", XGTK_OBJECT (obj)->object);
 
@@ -782,8 +776,7 @@ emacs_gtk_object_printer (Lisp_Object obj, Lisp_Object printcharfun, int escapef
     write_c_string (gtk_type_name (GTK_OBJECT_TYPE (XGTK_OBJECT (obj)->object)), printcharfun);
   else
     write_c_string ("dead", printcharfun);
-  sprintf (buf, ") %p>", (void *) XGTK_OBJECT (obj)->object);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, ") %p>", (void *) XGTK_OBJECT (obj)->object);
 }
 
 static Lisp_Object
@@ -905,8 +898,8 @@ emacs_gtk_object_finalizer (void *header, int for_disksave)
 
   if (for_disksave)
     {
-      Lisp_Object obj;
-      XSETGTK_OBJECT (obj, data);
+      Lisp_Object obj = wrap_emacs_gtk_object (data);
+
 
       invalid_operation
 	("Can't dump an emacs containing GtkObject objects", obj);
@@ -974,7 +967,7 @@ Lisp_Object build_gtk_object (GtkObject *obj)
 
       data->object = obj;
       data->alive_p = TRUE;
-      XSETGTK_OBJECT (retval, data);
+      retval = wrap_emacs_gtk_object (data);
 
       id = new_gui_id ();
       gtk_object_set_data (obj, "xemacs::gui_id", (gpointer) id);
@@ -1083,15 +1076,12 @@ DEFUN ("gtk-signal-connect", Fgtk_signal_connect, 3, 6, 0, /*
 static void
 emacs_gtk_boxed_printer (Lisp_Object obj, Lisp_Object printcharfun, int escapeflag)
 {
-  char buf[200];
-
   if (print_readably)
     printing_unreadable_object ("#<GtkBoxed %p>", XGTK_BOXED (obj)->object);
 
   write_c_string ("#<GtkBoxed (", printcharfun);
   write_c_string (gtk_type_name (XGTK_BOXED (obj)->object_type), printcharfun);
-  sprintf (buf, ") %p>", (void *) XGTK_BOXED (obj)->object);
-  write_c_string (buf, printcharfun);
+  write_fmt_string (printcharfun, ") %p>", (void *) XGTK_BOXED (obj)->object);
 }
 
 static int
@@ -1162,7 +1152,7 @@ Lisp_Object build_gtk_boxed (void *obj, GtkType t)
   data->object = obj;
   data->object_type = t;
 
-  XSETGTK_BOXED (retval, data);
+  retval = wrap_emacs_gtk_boxed (data);
 
   return (retval);
 }
@@ -1363,6 +1353,7 @@ Do NOT modify unless you really understand ui-gtk.c.
 
 
 /* Various utility functions */
+#if 0
 void describe_gtk_arg (GtkArg *arg)
 {
   GtkArg a = *arg;
@@ -1441,6 +1432,7 @@ void describe_gtk_arg (GtkArg *arg)
       abort();
     }
 }
+#endif
 
 Lisp_Object gtk_type_to_lisp (GtkArg *arg)
 {
@@ -1620,7 +1612,8 @@ int lisp_to_gtk_type (Lisp_Object obj, GtkArg *arg)
 	  if (GLYPHP (obj))
 	    {
 	      Lisp_Object window = Fselected_window (Qnil);
-	      Lisp_Object instance = glyph_image_instance (obj, window, ERROR_ME_NOT, 1);
+	      Lisp_Object instance =
+		glyph_image_instance (obj, window, ERROR_ME_DEBUG_WARN, 1);
 	      struct Lisp_Image_Instance *p = XIMAGE_INSTANCE (instance);
 
 	      switch (XIMAGE_INSTANCE_TYPE (instance))
