@@ -80,7 +80,7 @@ undo_prelude (struct buffer *b, int hack_pending_boundary)
 	  != BUFFER_BASE_BUFFER (XBUFFER (last_undo_buffer))))
     {
       undo_boundary (b);
-      XSETBUFFER (last_undo_buffer, b);
+      last_undo_buffer = wrap_buffer (b);
     }
 
   /* Allocate a cons cell to be the undo boundary after this command.  */
@@ -536,7 +536,7 @@ Return what remains of the list.
     }
 
   UNGCPRO;
-  return unbind_to (speccount, list);
+  return unbind_to_1 (speccount, list);
 }
 
 void
