@@ -2,8 +2,8 @@
 emacs_is_beta=t
 emacs_major_version=21
 emacs_minor_version=5
-emacs_beta_version=5
-xemacs_codename="beets"
+emacs_beta_version=6
+xemacs_codename="bok choi"
 emacs_kit_version=
 infodock_major_version=4
 infodock_minor_version=0
